@@ -1,9 +1,10 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: rombios32.c 11180 2012-05-13 20:06:51Z vruppert $
+// $Id: rombios32.c 12720 2015-04-18 13:18:01Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  32 bit Bochs BIOS init code
-//  Copyright (C) 2006 Fabrice Bellard
+//  Copyright (C) 2006       Fabrice Bellard
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -372,7 +373,7 @@ void delay_ms(int n)
     int i, j;
     for(i = 0; i < n; i++) {
 #ifdef BX_QEMU
-        volatile int k;
+        volatile int k = 0;
         /* approximative ! */
         for(j = 0; j < 1000000; j++) {
           k++;
@@ -732,12 +733,13 @@ static void pci_bios_init_bridges(PCIDevice *d)
     device_id = pci_config_readw(d, PCI_DEVICE_ID);
 
     if (vendor_id == PCI_VENDOR_ID_INTEL &&
-       (device_id == PCI_DEVICE_ID_INTEL_82371SB_0 ||
+       (device_id == PCI_DEVICE_ID_INTEL_82371FB_0 ||
+        device_id == PCI_DEVICE_ID_INTEL_82371SB_0 ||
         device_id == PCI_DEVICE_ID_INTEL_82371AB_0)) {
         int i, irq;
         uint8_t elcr[2];
 
-        /* PIIX3/PIIX4 PCI to ISA bridge */
+        /* PIIX/PIIX3/PIIX4 PCI to ISA bridge */
 
         elcr[0] = 0x00;
         elcr[1] = 0x00;
@@ -752,8 +754,9 @@ static void pci_bios_init_bridges(PCIDevice *d)
         outb(0x4d1, elcr[1]);
         BX_INFO("PIIX3/PIIX4 init: elcr=%02x %02x\n",
                 elcr[0], elcr[1]);
-    } else if (vendor_id == PCI_VENDOR_ID_INTEL && device_id == PCI_DEVICE_ID_INTEL_82441) {
-        /* i440 PCI bridge */
+    } else if (vendor_id == PCI_VENDOR_ID_INTEL &&
+              (device_id == PCI_DEVICE_ID_INTEL_82441 || device_id == PCI_DEVICE_ID_INTEL_82437)) {
+        /* i440FX / i430FX PCI bridge */
         bios_shadow_init(d);
     }
 }
@@ -881,7 +884,8 @@ static void pci_bios_init_device(PCIDevice *d)
     switch(class) {
     case PCI_CLASS_STORAGE_IDE:
         if (vendor_id == PCI_VENDOR_ID_INTEL &&
-           (device_id == PCI_DEVICE_ID_INTEL_82371SB_1 ||
+           (device_id == PCI_DEVICE_ID_INTEL_82371FB_1 ||
+            device_id == PCI_DEVICE_ID_INTEL_82371SB_1 ||
             device_id == PCI_DEVICE_ID_INTEL_82371AB)) {
             /* PIIX3/PIIX4 IDE */
             pci_config_writew(d, 0x40, 0x8000); // enable IDE0
@@ -1609,8 +1613,12 @@ void acpi_bios_init(void)
             (unsigned long)rsdt, acpi_tables_size);
 
     /* RSDP */
+    // Copying "RSD PTR " must be split in two parts. Otherwise a guest OS
+    // may incorrectly assume it has found the RSDT table within the rombios32
+    // string table.
     memset(rsdp, 0, sizeof(*rsdp));
-    memcpy(rsdp->signature, "RSD PTR ", 8);
+    memcpy(rsdp->signature, "RSD ", 4);
+    memcpy(rsdp->signature+4, "PTR ", 4);
 #ifdef BX_QEMU
     memcpy(rsdp->oem_id, "QEMU  ", 6);
 #else
@@ -2274,7 +2282,11 @@ static uint32_t find_resume_vector(void)
 #endif
 
     for (addr = start; addr < end; addr += 16) {
-        if (!memcmp((void*)addr, "RSD PTR ", 8)) {
+        // The check for "RSD PTR " must be split in two parts. Otherwise a
+        // guest OS may incorrectly assume it has found the RSDT table within
+        // the rombios32 string table.
+        if ((!memcmp((void*)addr, "RSD ", 4)) &&
+            (!memcmp((void*)addr+4, "PTR ", 4))) {
             struct rsdp_descriptor *rsdp = (void*)addr;
             struct rsdt_descriptor_rev1 *rsdt = (void*)rsdp->rsdt_physical_address;
             struct fadt_descriptor_rev1 *fadt = (void*)rsdt->table_offset_entry[0];
@@ -2293,7 +2305,8 @@ static void find_440fx(PCIDevice *d)
     vendor_id = pci_config_readw(d, PCI_VENDOR_ID);
     device_id = pci_config_readw(d, PCI_DEVICE_ID);
 
-    if (vendor_id == PCI_VENDOR_ID_INTEL && device_id == PCI_DEVICE_ID_INTEL_82441)
+    if (vendor_id == PCI_VENDOR_ID_INTEL &&
+       (device_id == PCI_DEVICE_ID_INTEL_82441 || device_id == PCI_DEVICE_ID_INTEL_82437))
         i440_pcidev = *d;
 }
 
