@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith16.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: arith16.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_RX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EwR(bxInstruction_c *i)
 {
   Bit32u rx = ++BX_READ_16BIT_REG(i->dst());
   SET_FLAGS_OSZAP_ADD_16(rx - 1, 0, rx);
@@ -32,7 +32,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_RX(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_RX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EwR(bxInstruction_c *i)
 {
   Bit32u rx = --BX_READ_16BIT_REG(i->dst());
   SET_FLAGS_OSZAP_SUB_16(rx + 1, 0, rx);
@@ -48,7 +48,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EwGwM(bxInstruction_c *i)
   Bit32u op2_16 = BX_READ_16BIT_REG(i->src());
   Bit32u sum_16 = op1_16 + op2_16;
 
-  write_RMW_virtual_word(sum_16);
+  write_RMW_linear_word(sum_16);
 
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 
@@ -91,7 +91,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EwGwM(bxInstruction_c *i)
   Bit32u op2_16 = BX_READ_16BIT_REG(i->src());
   Bit32u sum_16 = op1_16 + op2_16 + getB_CF();
 
-  write_RMW_virtual_word(sum_16);
+  write_RMW_linear_word(sum_16);
 
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 
@@ -134,7 +134,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EwGwM(bxInstruction_c *i)
   Bit32u op2_16 = BX_READ_16BIT_REG(i->src());
   Bit32u diff_16 = op1_16 - (op2_16 + getB_CF());
 
-  write_RMW_virtual_word(diff_16);
+  write_RMW_linear_word(diff_16);
 
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 
@@ -177,7 +177,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EwIwM(bxInstruction_c *i)
   Bit32u op2_16 = i->Iw();
   Bit32u diff_16 = op1_16 - (op2_16 + getB_CF());
 
-  write_RMW_virtual_word(diff_16);
+  write_RMW_linear_word(diff_16);
 
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 
@@ -205,7 +205,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EwGwM(bxInstruction_c *i)
   Bit32u op2_16 = BX_READ_16BIT_REG(i->src());
   Bit32u diff_16 = op1_16 - op2_16;
 
-  write_RMW_virtual_word(diff_16);
+  write_RMW_linear_word(diff_16);
 
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 
@@ -312,7 +312,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EwGwM(bxInstruction_c *i)
   Bit32u op2_16 = BX_READ_16BIT_REG(i->src());
   Bit32u sum_16 = op1_16 + op2_16;
 
-  write_RMW_virtual_word(sum_16);
+  write_RMW_linear_word(sum_16);
 
   /* and write destination into source */
   BX_WRITE_16BIT_REG(i->src(), op1_16);
@@ -354,7 +354,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EwIwM(bxInstruction_c *i)
   Bit32u op2_16 = i->Iw();
   Bit32u sum_16 = op1_16 + op2_16;
 
-  write_RMW_virtual_word(sum_16);
+  write_RMW_linear_word(sum_16);
 
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 
@@ -381,7 +381,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EwIwM(bxInstruction_c *i)
   Bit32u op2_16 = i->Iw();
   Bit32u sum_16 = op1_16 + op2_16 + getB_CF();
 
-  write_RMW_virtual_word(sum_16);
+  write_RMW_linear_word(sum_16);
 
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 
@@ -409,7 +409,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EwIwM(bxInstruction_c *i)
   Bit32u op2_16 = i->Iw();
   Bit32u diff_16 = op1_16 - op2_16;
 
-  write_RMW_virtual_word(diff_16);
+  write_RMW_linear_word(diff_16);
 
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 
@@ -459,7 +459,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EwM(bxInstruction_c *i)
 
   Bit32u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16 = 0 - (Bit32s)(Bit16s)(op1_16);
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAPC_SUB_16(0, 0 - op1_16, op1_16);
 
@@ -483,7 +483,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EwM(bxInstruction_c *i)
 
   Bit32u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16++;
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAP_ADD_16(op1_16 - 1, 0, op1_16);
 
@@ -496,7 +496,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EwM(bxInstruction_c *i)
 
   Bit32u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16--;
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAP_SUB_16(op1_16 + 1, 0, op1_16);
 
@@ -514,10 +514,11 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EwGwM(bxInstruction_c *i)
 
   if (diff_16 == 0) {  // if accumulator == dest
     // dest <-- src
-    write_RMW_virtual_word(BX_READ_16BIT_REG(i->src()));
+    write_RMW_linear_word(BX_READ_16BIT_REG(i->src()));
   }
   else {
     // accumulator <-- dest
+    write_RMW_linear_word(op1_16);
     AX = op1_16;
   }
 
