@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit16.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: bit16.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,8 @@
 
 #if BX_CPU_LEVEL >= 3
 
+#include "scalar_arith.h"
+
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GwEwR(bxInstruction_c *i)
 {
   Bit16u op2_16 = BX_READ_16BIT_REG(i->src());
@@ -34,12 +36,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GwEwR(bxInstruction_c *i)
     assert_ZF(); /* op1_16 undefined */
   }
   else {
-    Bit16u op1_16 = 0;
-    while ((op2_16 & 0x01) == 0) {
-      op1_16++;
-      op2_16 >>= 1;
-    }
-
+    Bit16u op1_16 = tzcntw(op2_16);
     SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
     clear_ZF();
 
@@ -123,7 +120,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EwGwM(bxInstruction_c *i)
   op1_16 = read_RMW_virtual_word(i->seg(), op1_addr & i->asize_mask());
   bit_i = (op1_16 >> index) & 0x01;
   op1_16 |= (1 << index);
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   set_CF(bit_i);
 
@@ -165,7 +162,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EwGwM(bxInstruction_c *i)
   op1_16 &= ~(1 << index);
 
   /* now write back to destination */
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   set_CF(temp_cf);
 
@@ -204,7 +201,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EwGwM(bxInstruction_c *i)
   op1_16 = read_RMW_virtual_word(i->seg(), op1_addr & i->asize_mask());
   bx_bool temp_CF = (op1_16 >> index_16) & 0x01;
   op1_16 ^= (1 << index_16);  /* toggle bit */
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   set_CF(temp_CF);
 
@@ -259,7 +256,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EwIbM(bxInstruction_c *i)
   Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   bx_bool temp_CF = (op1_16 >> op2_8) & 0x01;
   op1_16 |= (1 << op2_8);
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   set_CF(temp_CF);
 
@@ -289,7 +286,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EwIbM(bxInstruction_c *i)
   Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   bx_bool temp_CF = (op1_16 >> op2_8) & 0x01;
   op1_16 ^= (1 << op2_8);  /* toggle bit */
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   set_CF(temp_CF);
 
@@ -319,7 +316,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EwIbM(bxInstruction_c *i)
   Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   bx_bool temp_CF = (op1_16 >> op2_8) & 0x01;
   op1_16 &= ~(1 << op2_8);
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   set_CF(temp_CF);
 
@@ -343,18 +340,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EwIbR(bxInstruction_c *i)
 /* F3 0F B8 */
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GwEwR(bxInstruction_c *i)
 {
-  Bit16u op2_16 = BX_READ_16BIT_REG(i->src());
+  Bit16u op_16 = popcntw(BX_READ_16BIT_REG(i->src()));
 
-  Bit16u op1_16 = 0;
-  while (op2_16 != 0) {
-    if (op2_16 & 1) op1_16++;
-    op2_16 >>= 1;
-  }
+  clearEFlagsOSZAPC();
+  if (! op_16) assert_ZF();
 
-  Bit32u flags = op1_16 ? 0 : EFlagsZFMask;
-  setEFlagsOSZAPC(flags);
-
-  BX_WRITE_16BIT_REG(i->dst(), op1_16);
+  BX_WRITE_16BIT_REG(i->dst(), op_16);
 
   BX_NEXT_INSTR(i);
 }
@@ -363,12 +354,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GwEwR(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TZCNT_GwEwR(bxInstruction_c *i)
 {
   Bit16u op1_16 = BX_READ_16BIT_REG(i->src());
-  Bit16u mask = 0x1, result_16 = 0;
-
-  while ((op1_16 & mask) == 0 && mask) {
-    mask <<= 1;
-    result_16++;
-  }
+  Bit16u result_16 = (Bit16u) tzcntw(op1_16);
 
   set_CF(! op1_16);
   set_ZF(! result_16);
@@ -382,12 +368,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TZCNT_GwEwR(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LZCNT_GwEwR(bxInstruction_c *i)
 {
   Bit16u op1_16 = BX_READ_16BIT_REG(i->src());
-  Bit16u mask = 0x8000, result_16 = 0;
-
-  while ((op1_16 & mask) == 0 && mask) {
-    mask >>= 1;
-    result_16++;
-  }
+  Bit16u result_16 = (Bit16u) lzcntw(op1_16);
 
   set_CF(! op1_16);
   set_ZF(! result_16);
