@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit32.cc 11437 2012-09-21 14:56:56Z sshwarts $
+// $Id: bit32.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,8 @@
 
 #if BX_CPU_LEVEL >= 3
 
+#include "scalar_arith.h"
+
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GdEdR(bxInstruction_c *i)
 {
   Bit32u op2_32 = BX_READ_32BIT_REG(i->src());
@@ -34,12 +36,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GdEdR(bxInstruction_c *i)
     assert_ZF(); /* op1_32 undefined */
   }
   else {
-    Bit32u op1_32 = 0;
-    while ((op2_32 & 0x01) == 0) {
-      op1_32++;
-      op2_32 >>= 1;
-    }
-
+    Bit32u op1_32 = tzcntd(op2_32);
     SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
     clear_ZF();
 
@@ -126,7 +123,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdM(bxInstruction_c *i)
   bit_i = (op1_32 >> index) & 0x01;
   op1_32 |= (1 << index);
 
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   set_CF(bit_i);
 
@@ -169,7 +166,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdM(bxInstruction_c *i)
   op1_32 &= ~(1 << index);
 
   /* now write back to destination */
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   set_CF(temp_cf);
 
@@ -211,7 +208,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdM(bxInstruction_c *i)
   op1_32 ^= (1 << index_32);  /* toggle bit */
   set_CF(temp_CF);
 
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   BX_NEXT_INSTR(i);
 }
@@ -264,7 +261,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbM(bxInstruction_c *i)
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
   op1_32 |= (1 << op2_8);
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   set_CF(temp_CF);
 
@@ -294,7 +291,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbM(bxInstruction_c *i)
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
   op1_32 ^= (1 << op2_8);  /* toggle bit */
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   set_CF(temp_CF);
 
@@ -324,7 +321,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbM(bxInstruction_c *i)
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
   op1_32 &= ~(1 << op2_8);
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   set_CF(temp_CF);
 
@@ -348,18 +345,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbR(bxInstruction_c *i)
 /* F3 0F B8 */
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GdEdR(bxInstruction_c *i)
 {
-  Bit32u op2_32 = BX_READ_32BIT_REG(i->src());
+  Bit32u op_32 = popcntd(BX_READ_32BIT_REG(i->src()));
 
-  Bit32u op1_32 = 0;
-  while (op2_32 != 0) {
-    op2_32 &= (op2_32-1);
-    op1_32++;
-  }
+  clearEFlagsOSZAPC();
+  if (! op_32) assert_ZF();
 
-  Bit32u flags = op1_32 ? 0 : EFlagsZFMask;
-  setEFlagsOSZAPC(flags);
-
-  BX_WRITE_32BIT_REGZ(i->dst(), op1_32);
+  BX_WRITE_32BIT_REGZ(i->dst(), op_32);
 
   BX_NEXT_INSTR(i);
 }
@@ -368,12 +359,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GdEdR(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TZCNT_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->src());
-  Bit32u mask = 0x1, result_32 = 0;
-
-  while ((op1_32 & mask) == 0 && mask) {
-    mask <<= 1;
-    result_32++;
-  }
+  Bit32u result_32 = tzcntd(op1_32);
 
   set_CF(! op1_32);
   set_ZF(! result_32);
@@ -387,12 +373,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TZCNT_GdEdR(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LZCNT_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->src());
-  Bit32u mask = 0x80000000, result_32 = 0;
-
-  while ((op1_32 & mask) == 0 && mask) {
-    mask >>= 1;
-    result_32++;
-  }
+  Bit32u result_32 = lzcntd(op1_32);
 
   set_CF(! op1_32);
   set_ZF(! result_32);
