@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit64.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: bit64.cc 12619 2015-01-26 20:01:25Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,8 @@
 
 #if BX_SUPPORT_X86_64
 
+#include "scalar_arith.h"
+
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GqEqR(bxInstruction_c *i)
 {
   Bit64u op2_64 = BX_READ_64BIT_REG(i->src());
@@ -34,12 +36,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GqEqR(bxInstruction_c *i)
     assert_ZF(); /* op1_64 undefined */
   }
   else {
-    Bit64u op1_64 = 0;
-    while ((op2_64 & 0x01) == 0) {
-      op1_64++;
-      op2_64 >>= 1;
-    }
-
+    Bit64u op1_64 = tzcntq(op2_64);
     SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
     clear_ZF();
 
@@ -89,7 +86,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EqGqM(bxInstruction_c *i)
     op1_addr = (Bit32u) op1_addr;
 
   /* pointer, segment address pair */
-  op1_64 = read_virtual_qword_64(i->seg(), op1_addr);
+  op1_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), op1_addr));
 
   set_CF((op1_64 >> index) & 0x01);
 
@@ -125,10 +122,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EqGqM(bxInstruction_c *i)
     op1_addr = (Bit32u) op1_addr;
 
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), op1_addr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), op1_addr));
   bit_i = (op1_64 >> index) & 0x01;
   op1_64 |= (((Bit64u) 1) << index);
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   set_CF(bit_i);
 
@@ -167,11 +164,11 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EqGqM(bxInstruction_c *i)
     op1_addr = (Bit32u) op1_addr;
 
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), op1_addr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), op1_addr));
   bx_bool temp_cf = (op1_64 >> index) & 0x01;
   op1_64 &= ~(((Bit64u) 1) << index);
   /* now write back to destination */
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   set_CF(temp_cf);
 
@@ -210,12 +207,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EqGqM(bxInstruction_c *i)
   if (! i->as64L())
     op1_addr = (Bit32u) op1_addr;
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), op1_addr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), op1_addr));
   bx_bool temp_CF = (op1_64 >> index) & 0x01;
   op1_64 ^= (((Bit64u) 1) << index);  /* toggle bit */
   set_CF(temp_CF);
 
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   BX_NEXT_INSTR(i);
 }
@@ -241,7 +238,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EqIbM(bxInstruction_c *i)
 {
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   Bit8u  op2_8  = i->Ib() & 0x3f;
 
   set_CF((op1_64 >> op2_8) & 0x01);
@@ -265,10 +262,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EqIbM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   bx_bool temp_CF = (op1_64 >> op2_8) & 0x01;
   op1_64 |= (((Bit64u) 1) << op2_8);
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   set_CF(temp_CF);
 
@@ -295,10 +292,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EqIbM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   bx_bool temp_CF = (op1_64 >> op2_8) & 0x01;
   op1_64 ^= (((Bit64u) 1) << op2_8);  /* toggle bit */
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   set_CF(temp_CF);
 
@@ -325,10 +322,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EqIbM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   bx_bool temp_CF = (op1_64 >> op2_8) & 0x01;
   op1_64 &= ~(((Bit64u) 1) << op2_8);
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   set_CF(temp_CF);
 
@@ -352,18 +349,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EqIbR(bxInstruction_c *i)
 /* F3 0F B8 */
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GqEqR(bxInstruction_c *i)
 {
-  Bit64u op2_64 = BX_READ_64BIT_REG(i->src());
+  Bit32u op_32 = popcntq(BX_READ_64BIT_REG(i->src()));
 
-  Bit64u op1_64 = 0;
-  while (op2_64 != 0) {
-    if (op2_64 & 1) op1_64++;
-    op2_64 >>= 1;
-  }
+  clearEFlagsOSZAPC();
+  if (! op_32) assert_ZF();
 
-  Bit32u flags = op1_64 ? 0 : EFlagsZFMask;
-  setEFlagsOSZAPC(flags);
-
-  BX_WRITE_64BIT_REG(i->dst(), op1_64);
+  BX_WRITE_32BIT_REGZ(i->dst(), op_32);
 
   BX_NEXT_INSTR(i);
 }
@@ -372,12 +363,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GqEqR(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TZCNT_GqEqR(bxInstruction_c *i)
 {
   Bit64u op1_64 = BX_READ_64BIT_REG(i->src());
-  Bit64u mask = 0x1, result_64 = 0;
-
-  while ((op1_64 & mask) == 0 && mask) {
-    mask <<= 1;
-    result_64++;
-  }
+  Bit64u result_64 = tzcntq(op1_64);
 
   set_CF(! op1_64);
   set_ZF(! result_64);
@@ -390,13 +376,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TZCNT_GqEqR(bxInstruction_c *i)
 /* F3 0F BD */
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LZCNT_GqEqR(bxInstruction_c *i)
 {
-  Bit64u op1_64 = BX_READ_64BIT_REG(i->src()), result_64 = 0;
-  Bit64u mask = BX_CONST64(0x8000000000000000);
-
-  while ((op1_64 & mask) == 0 && mask) {
-    mask >>= 1;
-    result_64++;
-  }
+  Bit64u op1_64 = BX_READ_64BIT_REG(i->src());
+  Bit64u result_64 = lzcntq(op1_64);
 
   set_CF(! op1_64);
   set_ZF(! result_64);
