@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: athlon64_venice.h 11033 2012-02-13 23:29:01Z sshwarts $
+// $Id: athlon64_venice.h 12640 2015-02-11 21:31:17Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2011 Stanislav Shwartsman
+//   Copyright (c) 2011-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -36,9 +36,6 @@ public:
   // return CPU name
   virtual const char *get_name(void) const { return "athlon64_venice"; }
 
-  virtual Bit64u get_isa_extensions_bitmask(void) const;
-  virtual Bit32u get_cpu_extensions_bitmask(void) const;
-
   virtual void get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_function_t *leaf) const;
 
   virtual void dump_cpuid(void) const;
@@ -49,13 +46,10 @@ private:
 
   void get_ext_cpuid_leaf_0(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const;
-  void get_ext_cpuid_brand_string_leaf(Bit32u function, cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_5(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_6(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_7(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_8(cpuid_function_t *leaf) const;
-
-  void get_cpuid_hidden_level(cpuid_function_t *leaf) const;
 };
 
 extern bx_cpuid_t *create_athlon64_venice_cpuid(BX_CPU_C *cpu);
