@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: corei7_ivy_bridge_3770K.cc 11687 2013-05-20 18:15:35Z sshwarts $
+// $Id: corei7_ivy_bridge_3770K.cc 12642 2015-02-12 20:18:35Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2012 Stanislav Shwartsman
+//   Copyright (c) 2013-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -35,9 +35,6 @@ corei7_ivy_bridge_3770k_t::corei7_ivy_bridge_3770k_t(BX_CPU_C *cpu): bx_cpuid_t(
   if (! BX_SUPPORT_X86_64)
     BX_PANIC(("You must enable x86-64 for Intel Core i7 Ivy Bridge configuration"));
 
-  if (! BX_SUPPORT_AVX)
-    BX_PANIC(("You must enable AVX for Intel Core i7 Ivy Bridge configuration"));
-
   if (BX_SUPPORT_VMX == 1)
     BX_INFO(("You must compile with --enable-vmx=2 for Intel Core i7 Ivy Bridge VMX configuration"));
 
@@ -45,10 +42,61 @@ corei7_ivy_bridge_3770k_t::corei7_ivy_bridge_3770k_t(BX_CPU_C *cpu): bx_cpuid_t(
     BX_INFO(("WARNING: MONITOR/MWAIT support is not compiled in !"));
 
   BX_INFO(("WARNING: RDRAND would not produce true random numbers !"));
+
+  enable_cpu_extension(BX_ISA_X87);
+  enable_cpu_extension(BX_ISA_486);
+  enable_cpu_extension(BX_ISA_PENTIUM);
+  enable_cpu_extension(BX_ISA_P6);
+  enable_cpu_extension(BX_ISA_MMX);
+  enable_cpu_extension(BX_ISA_SYSENTER_SYSEXIT);
+  enable_cpu_extension(BX_ISA_CLFLUSH);
+  enable_cpu_extension(BX_ISA_DEBUG_EXTENSIONS);
+  enable_cpu_extension(BX_ISA_VME);
+  enable_cpu_extension(BX_ISA_PSE);
+  enable_cpu_extension(BX_ISA_PAE);
+  enable_cpu_extension(BX_ISA_PGE);
+#if BX_PHY_ADDRESS_LONG
+  enable_cpu_extension(BX_ISA_PSE36);
+#endif
+  enable_cpu_extension(BX_ISA_MTRR);
+  enable_cpu_extension(BX_ISA_PAT);
+  enable_cpu_extension(BX_ISA_XAPIC);
+  enable_cpu_extension(BX_ISA_X2APIC);
+  enable_cpu_extension(BX_ISA_LONG_MODE);
+  enable_cpu_extension(BX_ISA_LM_LAHF_SAHF);
+  enable_cpu_extension(BX_ISA_CMPXCHG16B);
+  enable_cpu_extension(BX_ISA_NX);
+  enable_cpu_extension(BX_ISA_PCID);
+  enable_cpu_extension(BX_ISA_TSC_DEADLINE);
+  enable_cpu_extension(BX_ISA_SSE);
+  enable_cpu_extension(BX_ISA_SSE2);
+  enable_cpu_extension(BX_ISA_SSE3);
+  enable_cpu_extension(BX_ISA_SSSE3);
+  enable_cpu_extension(BX_ISA_SSE4_1);
+  enable_cpu_extension(BX_ISA_SSE4_2);
+  enable_cpu_extension(BX_ISA_POPCNT);
+#if BX_SUPPORT_MONITOR_MWAIT
+  enable_cpu_extension(BX_ISA_MONITOR_MWAIT);
+#endif
+#if BX_SUPPORT_VMX >= 2
+  enable_cpu_extension(BX_ISA_VMX);
+#endif
+//enable_cpu_extension(BX_ISA_SMX);
+  enable_cpu_extension(BX_ISA_RDTSCP);
+  enable_cpu_extension(BX_ISA_XSAVE);
+  enable_cpu_extension(BX_ISA_XSAVEOPT);
+  enable_cpu_extension(BX_ISA_AES_PCLMULQDQ);
+  enable_cpu_extension(BX_ISA_AVX);
+  enable_cpu_extension(BX_ISA_AVX_F16C);
+  enable_cpu_extension(BX_ISA_FSGSBASE);
+  enable_cpu_extension(BX_ISA_SMEP);
+  enable_cpu_extension(BX_ISA_RDRAND);
 }
 
 void corei7_ivy_bridge_3770k_t::get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_function_t *leaf) const
 {
+  static const char* brand_string = "       Intel(R) Core(TM) i7-3770K CPU @ 3.50GHz";
+
   static bx_bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
   if (cpuid_limit_winnt)
     if (function > 2 && function < 0x80000000) function = 2;
@@ -63,7 +111,7 @@ void corei7_ivy_bridge_3770k_t::get_cpuid_leaf(Bit32u function, Bit32u subfuncti
   case 0x80000002:
   case 0x80000003:
   case 0x80000004:
-    get_ext_cpuid_brand_string_leaf(function, leaf);
+    get_ext_cpuid_brand_string_leaf(brand_string, function, leaf);
     return;
   case 0x80000005:
     get_reserved_leaf(leaf);
@@ -121,60 +169,6 @@ void corei7_ivy_bridge_3770k_t::get_cpuid_leaf(Bit32u function, Bit32u subfuncti
   }
 }
 
-Bit64u corei7_ivy_bridge_3770k_t::get_isa_extensions_bitmask(void) const
-{
-  return BX_ISA_X87 |
-         BX_ISA_486 |
-         BX_ISA_PENTIUM |
-         BX_ISA_P6 |
-         BX_ISA_MMX |
-         BX_ISA_SYSENTER_SYSEXIT |
-         BX_ISA_CLFLUSH |
-         BX_ISA_SSE |
-         BX_ISA_SSE2 |
-         BX_ISA_SSE3 |
-         BX_ISA_SSSE3 |
-         BX_ISA_SSE4_1 |
-         BX_ISA_SSE4_2 |
-         BX_ISA_POPCNT |
-#if BX_SUPPORT_MONITOR_MWAIT
-         BX_ISA_MONITOR_MWAIT |
-#endif
-#if BX_SUPPORT_VMX >= 2
-         BX_ISA_VMX |
-#endif
-      /* BX_ISA_SMX | */
-         BX_ISA_RDTSCP |
-         BX_ISA_XSAVE |
-         BX_ISA_XSAVEOPT |
-         BX_ISA_AES_PCLMULQDQ |
-         BX_ISA_FSGSBASE |
-         BX_ISA_AVX |
-         BX_ISA_AVX_F16C |
-         BX_ISA_RDRAND |
-         BX_ISA_CMPXCHG16B |
-         BX_ISA_LM_LAHF_SAHF;
-}
-
-Bit32u corei7_ivy_bridge_3770k_t::get_cpu_extensions_bitmask(void) const
-{
-  return BX_CPU_DEBUG_EXTENSIONS |
-         BX_CPU_VME |
-         BX_CPU_PSE |
-         BX_CPU_PAE |
-         BX_CPU_PGE |
-         BX_CPU_PSE36 |
-         BX_CPU_MTRR |
-         BX_CPU_PAT |
-         BX_CPU_XAPIC |
-         BX_CPU_X2APIC |
-         BX_CPU_LONG_MODE |
-         BX_CPU_NX |
-         BX_CPU_PCID |
-         BX_CPU_SMEP |
-         BX_CPU_TSC_DEADLINE;
-}
-
 #if BX_SUPPORT_VMX >= 2
 
 // 
@@ -227,21 +221,12 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_0(cpuid_function_t *leaf) con
   // EBX: vendor ID string
   // EDX: vendor ID string
   // ECX: vendor ID string
+  unsigned max_leaf = 0xD;
   static bx_bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
   if (cpuid_limit_winnt)
-    leaf->eax = 0x2;
-  else
-    leaf->eax = 0xD;
+    max_leaf = 0x2;
 
-  // CPUID vendor string (e.g. GenuineIntel, AuthenticAMD, CentaurHauls, ...)
-  memcpy(&(leaf->ebx), vendor_string,     4);
-  memcpy(&(leaf->edx), vendor_string + 4, 4);
-  memcpy(&(leaf->ecx), vendor_string + 8, 4);
-#ifdef BX_BIG_ENDIAN
-  leaf->ebx = bx_bswap32(leaf->ebx);
-  leaf->ecx = bx_bswap32(leaf->ecx);
-  leaf->edx = bx_bswap32(leaf->edx);
-#endif
+  get_leaf_0(max_leaf, vendor_string, leaf);
 }
 
 // leaf 0x00000001 //
@@ -523,17 +508,25 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_f
   case 0:
     leaf->eax = 0; /* report max sub-leaf that supported in leaf 7 */
 
-    // * [0:0]    FS/GS BASE access instructions
-    //   [2:1]    reserved
-    //   [3:3]    BMI1: Advanced Bit Manipulation Extensions
-    //   [4:4]    reserved
-    //   [5:5]    AVX2
-    //   [6:6]    reserved
-    // * [7:7]    SMEP: Supervisor Mode Execution Protection
-    //   [8:8]    BMI2: Advanced Bit Manipulation Extensions
-    // * [9:9]    Support for Enhanced REP MOVSB/STOSB
-    //   [10:10]  Support for INVPCID instruction
-    //   [31:10]  reserved
+    // * [0:0]   FS/GS BASE access instructions
+    //   [1:1]   Support for IA32_TSC_ADJUST MSR
+    //   [2:2]   reserved
+    //   [3:3]   BMI1: Advanced Bit Manipulation Extensions
+    //   [4:4]   HLE: Hardware Lock Elision
+    //   [5:5]   AVX2
+    //   [6:6]   reserved
+    // * [7:7]   SMEP: Supervisor Mode Execution Protection
+    //   [8:8]   BMI2: Advanced Bit Manipulation Extensions
+    // * [9:9]   Support for Enhanced REP MOVSB/STOSB
+    //   [10:10] Support for INVPCID instruction
+    //   [11:11] RTM: Restricted Transactional Memory
+    //   [12:12] Supports Quality of Service (QoS) capability
+    //   [13:13] Deprecates FPU CS and FPU DS values
+    //   [17:14] reserved
+    //   [18:18] RDSEED instruction support
+    //   [19:19] ADCX/ADOX instructions support
+    //   [20:20] SMAP: Supervisor Mode Access Prevention
+    //   [31:21] reserved
     leaf->ebx = BX_CPUID_EXT3_FSGSBASE | 
                 BX_CPUID_EXT3_SMEP | 
                 BX_CPUID_EXT3_ENCHANCED_REP_STRINGS;
@@ -555,94 +548,45 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_7(Bit32u subfunction, cpuid_f
 void corei7_ivy_bridge_3770k_t::get_std_cpuid_leaf_A(cpuid_function_t *leaf) const
 {
   // CPUID function 0x0000000A - Architectural Performance Monitoring Leaf
-/*
+
+  // EAX:
+  //   [7:0] Version ID of architectural performance monitoring
+  //  [15:8] Number of general-purpose performance monitoring counters per logical processor
+  // [23:16] Bit width of general-purpose, performance monitoring counter
+  // [31:24] Length of EBX bit vector to enumerate architectural performance
+  //         monitoring events.
+
+  // EBX:
+  //     [0] Core cycle event not available if 1
+  //     [1] Instruction retired event not available if 1
+  //     [2] Reference cycles event not available if 1
+  //     [3] Last-level cache reference event not available if 1
+  //     [4] Last-level cache misses event not available if 1
+  //     [5] Branch instruction retired event not available if 1
+  //     [6] Branch mispredict retired event not available if 1
+  //  [31:7] reserved
+
+  // ECX: reserved
+
+  // EDX:
+  //   [4:0] Number of fixed performance counters (if Version ID > 1)
+  //  [12:5] Bit width of fixed-function performance counters (if Version ID > 1)
+  // [31:13] reserved
+
   leaf->eax = 0x07300403;
   leaf->ebx = 0x00000000;
   leaf->ecx = 0x00000000;
   leaf->edx = 0x00000603;
-*/
-  leaf->eax = 0; // reporting true capabilities breaks Win7 x64 installation
+/*
+  leaf->eax = 0; // reporting true capabilities without supporting it breaks Win7 x64 installation
   leaf->ebx = 0;
   leaf->ecx = 0;
   leaf->edx = 0;
+*/
 
   BX_INFO(("WARNING: Architectural Performance Monitoring is not implemented"));
 }
 
-BX_CPP_INLINE static Bit32u ilog2(Bit32u x)
-{
-  Bit32u count = 0;
-  while(x>>=1) count++;
-  return count;
-}
-
-// leaf 0x0000000B //
-void corei7_ivy_bridge_3770k_t::get_std_cpuid_extended_topology_leaf(Bit32u subfunction, cpuid_function_t *leaf) const
-{
-  // CPUID function 0x0000000B - Extended Topology Leaf
-  leaf->eax = 0;
-  leaf->ebx = 0;
-  leaf->ecx = subfunction;
-  leaf->edx = cpu->get_apic_id();
-
-#if BX_SUPPORT_SMP
-  switch(subfunction) {
-  case 0:
-     if (nthreads > 1) {
-        leaf->eax = ilog2(nthreads-1)+1;
-        leaf->ebx = nthreads;
-        leaf->ecx |= (1<<8);
-     }
-     else if (ncores > 1) {
-        leaf->eax = ilog2(ncores-1)+1;
-        leaf->ebx = ncores;
-        leaf->ecx |= (2<<8);
-     }
-     else if (nprocessors > 1) {
-        leaf->eax = ilog2(nprocessors-1)+1;
-        leaf->ebx = nprocessors;
-     }
-     else {
-        leaf->eax = 1;
-        leaf->ebx = 1; // number of logical CPUs at this level
-     }
-     break;
-
-  case 1:
-     if (nthreads > 1) {
-        if (ncores > 1) {
-           leaf->eax = ilog2(ncores-1)+1;
-           leaf->ebx = ncores;
-           leaf->ecx |= (2<<8);
-        }
-        else if (nprocessors > 1) {
-           leaf->eax = ilog2(nprocessors-1)+1;
-           leaf->ebx = nprocessors;
-        }
-     }
-     else if (ncores > 1) {
-        if (nprocessors > 1) {
-           leaf->eax = ilog2(nprocessors-1)+1;
-           leaf->ebx = nprocessors;
-        }
-     }
-     break;
-
-  case 2:
-     if (nthreads > 1) {
-        if (nprocessors > 1) {
-           leaf->eax = ilog2(nprocessors-1)+1;
-           leaf->ebx = nprocessors;
-        }
-     }
-     break;
-
-  default:
-     break;
-  }
-#endif
-}
-
 // leaf 0x0000000C reserved //
 
 // leaf 0x0000000D //
@@ -656,9 +600,9 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_xsave_leaf(Bit32u subfunction, cpu
     // EDX - valid bits of XCR0 (upper part)
     leaf->eax = cpu->xcr0_suppmask;
     leaf->ebx = 512+64;
-    if (cpu->xcr0.get_AVX())
-      leaf->ebx += 256;
-    leaf->ecx = 512+64+256 /* AVX */;
+    if (cpu->xcr0.get_YMM())
+      leaf->ebx = XSAVE_YMM_STATE_OFFSET + XSAVE_YMM_STATE_LEN;
+    leaf->ecx = XSAVE_YMM_STATE_OFFSET + XSAVE_YMM_STATE_LEN;
     leaf->edx = 0;
     return;
 
@@ -669,9 +613,9 @@ void corei7_ivy_bridge_3770k_t::get_std_cpuid_xsave_leaf(Bit32u subfunction, cpu
     leaf->edx = 0;
     return;
 
-  case 2: // AVX leaf
-    leaf->eax = 256;
-    leaf->ebx = 576;
+  case 2: // YMM leaf
+    leaf->eax = XSAVE_YMM_STATE_LEN;
+    leaf->ebx = XSAVE_YMM_STATE_OFFSET;
     leaf->ecx = 0;
     leaf->edx = 0;
     return;
@@ -693,10 +637,7 @@ void corei7_ivy_bridge_3770k_t::get_ext_cpuid_leaf_0(cpuid_function_t *leaf) con
   // EBX: reserved
   // EDX: reserved
   // ECX: reserved
-  leaf->eax = 0x80000008;
-  leaf->ebx = 0;
-  leaf->edx = 0; // Reserved for Intel
-  leaf->ecx = 0;
+  get_leaf_0(0x80000008, NULL, leaf);
 }
 
 // leaf 0x80000001 //
@@ -715,14 +656,26 @@ void corei7_ivy_bridge_3770k_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) con
   //   [3:3]   Extended APIC Space
   //   [4:4]   AltMovCR8: LOCK MOV CR0 means MOV CR8
   //   [5:5]   LZCNT: LZCNT instruction support
-  //   [6:6]   SSE4A: SSE4A Instructions support (deprecated?)
+  //   [6:6]   SSE4A: SSE4A Instructions support
   //   [7:7]   Misaligned SSE support
   //   [8:8]   PREFETCHW: PREFETCHW instruction support
   //   [9:9]   OSVW: OS visible workarounds (AMD)
-  //   [11:10] reserved
+  //   [10:10] IBS: Instruction based sampling
+  //   [11:11] XOP: Extended Operations Support and XOP Prefix
   //   [12:12] SKINIT support
   //   [13:13] WDT: Watchdog timer support
-  //   [31:14] reserved
+  //   [14:14] Reserved
+  //   [15:15] LWP: Light weight profiling
+  //   [16:16] FMA4: Four-operand FMA instructions support
+  //   [17:17] Reserved
+  //   [18:18] Reserved
+  //   [19:19] NodeId: Indicates support for NodeId MSR (0xc001100c)
+  //   [20:20] Reserved
+  //   [21:21] TBM: trailing bit manipulation instructions support
+  //   [22:22] Topology extensions support
+  //   [23:23] PerfCtrExtCore: core perf counter extensions support
+  //   [24:24] PerfCtrExtNB: NB perf counter extensions support
+  //   [31:25] Reserved
 
   leaf->ecx = BX_CPUID_EXT2_LAHF_SAHF;
 
@@ -750,41 +703,6 @@ void corei7_ivy_bridge_3770k_t::get_ext_cpuid_leaf_1(cpuid_function_t *leaf) con
 // leaf 0x80000002 //
 // leaf 0x80000003 //
 // leaf 0x80000004 //
-void corei7_ivy_bridge_3770k_t::get_ext_cpuid_brand_string_leaf(Bit32u function, cpuid_function_t *leaf) const
-{
-  // CPUID function 0x80000002-0x80000004 - Processor Name String Identifier
-  static const char* brand_string = "       Intel(R) Core(TM) i7-3770K CPU @ 3.50GHz";
-
-  switch(function) {
-  case 0x80000002:
-    memcpy(&(leaf->eax), brand_string     , 4);
-    memcpy(&(leaf->ebx), brand_string +  4, 4);
-    memcpy(&(leaf->ecx), brand_string +  8, 4);
-    memcpy(&(leaf->edx), brand_string + 12, 4);
-    break;
-  case 0x80000003:
-    memcpy(&(leaf->eax), brand_string + 16, 4);
-    memcpy(&(leaf->ebx), brand_string + 20, 4);
-    memcpy(&(leaf->ecx), brand_string + 24, 4);
-    memcpy(&(leaf->edx), brand_string + 28, 4);
-    break;
-  case 0x80000004:
-    memcpy(&(leaf->eax), brand_string + 32, 4);
-    memcpy(&(leaf->ebx), brand_string + 36, 4);
-    memcpy(&(leaf->ecx), brand_string + 40, 4);
-    memcpy(&(leaf->edx), brand_string + 44, 4);
-    break;
-  default:
-    break;
-  }
-
-#ifdef BX_BIG_ENDIAN
-  leaf->eax = bx_bswap32(leaf->eax);
-  leaf->ebx = bx_bswap32(leaf->ebx);
-  leaf->ecx = bx_bswap32(leaf->ecx);
-  leaf->edx = bx_bswap32(leaf->edx);
-#endif
-}
 
 // leaf 0x80000005 - L1 Cache and TLB Identifiers (reserved for Intel)
 
@@ -820,18 +738,7 @@ void corei7_ivy_bridge_3770k_t::get_ext_cpuid_leaf_8(cpuid_function_t *leaf) con
 
 void corei7_ivy_bridge_3770k_t::dump_cpuid(void) const
 {
-  struct cpuid_function_t leaf;
-  unsigned n;
-
-  for (n=0; n<=0xd; n++) {
-    get_cpuid_leaf(n, 0x00000000, &leaf);
-    BX_INFO(("CPUID[0x%08x]: %08x %08x %08x %08x", n, leaf.eax, leaf.ebx, leaf.ecx, leaf.edx));
-  }
-
-  for (n=0x80000000; n<=0x80000008; n++) {
-    get_cpuid_leaf(n, 0x00000000, &leaf);
-    BX_INFO(("CPUID[0x%08x]: %08x %08x %08x %08x", n, leaf.eax, leaf.ebx, leaf.ecx, leaf.edx));
-  }
+  bx_cpuid_t::dump_cpuid(0xD, 0x8);
 }
 
 bx_cpuid_t *create_corei7_ivy_bridge_3770k_cpuid(BX_CPU_C *cpu) { return new corei7_ivy_bridge_3770k_t(cpu); }
