@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: p2_klamath.h 11033 2012-02-13 23:29:01Z sshwarts $
+// $Id: p2_klamath.h 12476 2014-08-31 18:39:18Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2011 Stanislav Shwartsman
+//   Copyright (c) 2011-2014 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -30,15 +30,12 @@
 
 class p2_klamath_t : public bx_cpuid_t {
 public:
-  p2_klamath_t(BX_CPU_C *cpu): bx_cpuid_t(cpu) {}
+  p2_klamath_t(BX_CPU_C *cpu);
   virtual ~p2_klamath_t() {}
 
   // return CPU name
   virtual const char *get_name(void) const { return "p2_klamath"; }
 
-  virtual Bit64u get_isa_extensions_bitmask(void) const;
-  virtual Bit32u get_cpu_extensions_bitmask(void) const;
-
   virtual void get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_function_t *leaf) const;
 
   virtual void dump_cpuid(void) const;
