@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: p4_willamette.h 10954 2012-01-07 17:06:03Z sshwarts $
+// $Id: p4_willamette.h 12476 2014-08-31 18:39:18Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2011 Stanislav Shwartsman
+//   Copyright (c) 2011-2014 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -30,15 +30,12 @@
 
 class p4_willamette_t : public bx_cpuid_t {
 public:
-  p4_willamette_t(BX_CPU_C *cpu): bx_cpuid_t(cpu) {}
+  p4_willamette_t(BX_CPU_C *cpu);
   virtual ~p4_willamette_t() {}
 
   // return CPU name
   virtual const char *get_name(void) const { return "p4_willamette"; }
 
-  virtual Bit64u get_isa_extensions_bitmask(void) const;
-  virtual Bit32u get_cpu_extensions_bitmask(void) const;
-
   virtual void get_cpuid_leaf(Bit32u function, Bit32u subfunction, cpuid_function_t *leaf) const;
 
   virtual void dump_cpuid(void) const;
@@ -50,7 +47,6 @@ private:
 
   void get_ext_cpuid_leaf_0(cpuid_function_t *leaf) const;
   void get_ext_cpuid_leaf_1(cpuid_function_t *leaf) const;
-  void get_ext_cpuid_brand_string_leaf(Bit32u function, cpuid_function_t *leaf) const;
 };
 
 extern bx_cpuid_t *create_p4_willamette_cpuid(BX_CPU_C *cpu);
