@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: crregs.cc 11679 2013-04-17 19:46:11Z sshwarts $
+// $Id: crregs.cc 12481 2014-08-31 20:05:25Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2010-2012 Stanislav Shwartsman
+//   Copyright (c) 2010-2014 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -36,7 +36,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DdRd(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 5
   if (BX_CPU_THIS_PTR cr4.get_DE()) {
     if ((i->dst() & 0xE) == 4) {
-      BX_ERROR(("MOV_DdRd: access to DR4/DR5 causes #UD"));
+      BX_ERROR(("%s: access to DR4/DR5 causes #UD", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
     }
   }
@@ -45,14 +45,14 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DdRd(bxInstruction_c *i)
   // Note: processor clears GD upon entering debug exception
   // handler, to allow access to the debug registers
   if (BX_CPU_THIS_PTR dr7.get_GD()) {
-    BX_ERROR(("MOV_DdRd: DR7 GD bit is set"));
+    BX_ERROR(("%s: DR7 GD bit is set", i->getIaOpcodeNameShort()));
     BX_CPU_THIS_PTR debug_trap |= BX_DEBUG_DR_ACCESS_BIT;
     exception(BX_DB_EXCEPTION, 0);
   }
 
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_DdRd: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -115,7 +115,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DdRd(bxInstruction_c *i)
           (BX_CPU_THIS_PTR dr7.get_R_W3() == BX_HWDebugInstruction && BX_CPU_THIS_PTR dr7.get_LEN3()))
       {
         // Instruction breakpoint with LENx not 00b (1-byte length)
-        BX_ERROR(("MOV_DdRd: write of 0x%08x, R/W=00b LEN!=00b", val_32));
+        BX_ERROR(("%s: write of 0x%08x, R/W=00b LEN!=00b", i->getIaOpcodeNameShort(), val_32));
       }
 #endif
 
@@ -123,7 +123,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DdRd(bxInstruction_c *i)
       break;
 
     default:
-      BX_ERROR(("MOV_DdRd: #UD - register index out of range"));
+      BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
   }
 
@@ -142,7 +142,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdDd(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 5
   if (BX_CPU_THIS_PTR cr4.get_DE()) {
     if ((i->src() & 0xE) == 4) {
-      BX_ERROR(("MOV_RdDd: access to DR4/DR5 causes #UD"));
+      BX_ERROR(("%s: access to DR4/DR5 causes #UD", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
     }
   }
@@ -151,14 +151,14 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdDd(bxInstruction_c *i)
   // Note: processor clears GD upon entering debug exception
   // handler, to allow access to the debug registers
   if (BX_CPU_THIS_PTR dr7.get_GD()) {
-    BX_ERROR(("MOV_RdDd: DR7 GD bit is set"));
+    BX_ERROR(("%s: DR7 GD bit is set", i->getIaOpcodeNameShort()));
     BX_CPU_THIS_PTR debug_trap |= BX_DEBUG_DR_ACCESS_BIT;
     exception(BX_DB_EXCEPTION, 0);
   }
 
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_RdDd: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -192,7 +192,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdDd(bxInstruction_c *i)
       break;
 
     default:
-      BX_ERROR(("MOV_RdDd: #UD - register index out of range"));
+      BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
   }
 
@@ -211,27 +211,27 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DqRq(bxInstruction_c *i)
 
   if (BX_CPU_THIS_PTR cr4.get_DE()) {
     if ((i->dst() & 0xE) == 4) {
-      BX_ERROR(("MOV_DqRq: access to DR4/DR5 causes #UD"));
+      BX_ERROR(("%s: access to DR4/DR5 causes #UD", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
     }
   }
 
   if (i->dst() >= 8) {
-    BX_ERROR(("MOV_DqRq: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   // Note: processor clears GD upon entering debug exception
   // handler, to allow access to the debug registers
   if (BX_CPU_THIS_PTR dr7.get_GD()) {
-    BX_ERROR(("MOV_DqRq: DR7 GD bit is set"));
+    BX_ERROR(("%s: DR7 GD bit is set", i->getIaOpcodeNameShort()));
     BX_CPU_THIS_PTR debug_trap |= BX_DEBUG_DR_ACCESS_BIT;
     exception(BX_DB_EXCEPTION, 0);
   }
 
   /* #GP(0) if CPL is not 0 */
   if (CPL != 0) {
-    BX_ERROR(("MOV_DqRq: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -260,7 +260,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DqRq(bxInstruction_c *i)
       // access to DR4 causes #UD
     case 6: // DR6
       if (GET32H(val_64)) {
-        BX_ERROR(("MOV_DqRq: attempt to set upper part of DR6"));
+        BX_ERROR(("%s: attempt to set upper part of DR6", i->getIaOpcodeNameShort()));
         exception(BX_GP_EXCEPTION, 0);
       }
       // On Pentium+, bit12 is always zero
@@ -277,7 +277,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DqRq(bxInstruction_c *i)
       // by setting the LE and/or GE flags.
 
       if (GET32H(val_64)) {
-        BX_ERROR(("MOV_DqRq: attempt to set upper part of DR7"));
+        BX_ERROR(("%s: attempt to set upper part of DR7", i->getIaOpcodeNameShort()));
         exception(BX_GP_EXCEPTION, 0);
       }
 
@@ -292,7 +292,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DqRq(bxInstruction_c *i)
           (BX_CPU_THIS_PTR dr7.get_R_W3() == BX_HWDebugInstruction && BX_CPU_THIS_PTR dr7.get_LEN3()))
       {
         // Instruction breakpoint with LENx not 00b (1-byte length)
-        BX_ERROR(("MOV_DqRq: write of 0x%08x, R/W=00b LEN!=00b", BX_CPU_THIS_PTR dr7.get32()));
+        BX_ERROR(("%s: write of 0x%08x, R/W=00b LEN!=00b", i->getIaOpcodeNameShort(), BX_CPU_THIS_PTR dr7.get32()));
       }
 #endif
 
@@ -300,7 +300,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_DqRq(bxInstruction_c *i)
       break;
 
     default:
-      BX_ERROR(("MOV_DqRq: #UD - register index out of range"));
+      BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
   }
 
@@ -318,27 +318,27 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqDq(bxInstruction_c *i)
 
   if (BX_CPU_THIS_PTR cr4.get_DE()) {
     if ((i->src() & 0xE) == 4) {
-      BX_ERROR(("MOV_RqDq: access to DR4/DR5 causes #UD"));
+      BX_ERROR(("%s: access to DR4/DR5 causes #UD", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
     }
   }
 
   if (i->src() >= 8) {
-    BX_ERROR(("MOV_RqDq: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   // Note: processor clears GD upon entering debug exception
   // handler, to allow access to the debug registers
   if (BX_CPU_THIS_PTR dr7.get_GD()) {
-    BX_ERROR(("MOV_RqDq: DR7 GD bit is set"));
+    BX_ERROR(("%s: DR7 GD bit is set", i->getIaOpcodeNameShort()));
     BX_CPU_THIS_PTR debug_trap |= BX_DEBUG_DR_ACCESS_BIT;
     exception(BX_DB_EXCEPTION, 0);
   }
 
   /* #GP(0) if CPL is not 0 */
   if (CPL != 0) {
-    BX_ERROR(("MOV_RqDq: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -372,7 +372,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqDq(bxInstruction_c *i)
       break;
 
     default:
-      BX_ERROR(("MOV_RqDq: #UD - register index out of range"));
+      BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
       exception(BX_UD_EXCEPTION, 0);
   }
 
@@ -388,7 +388,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR0Rd(bxInstruction_c *i)
 {
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_CR0Rd: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -421,7 +421,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR2Rd(bxInstruction_c *i)
 {
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_CR2Rd: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -445,7 +445,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR3Rd(bxInstruction_c *i)
 {
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_CR3Rd: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -472,7 +472,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR3Rd(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 6
   if (BX_CPU_THIS_PTR cr0.get_PG() && BX_CPU_THIS_PTR cr4.get_PAE() && !long_mode()) {
     if (! CheckPDPTR(val_32)) {
-      BX_ERROR(("MOV_CR3Rd(): PDPTR check failed !"));
+      BX_ERROR(("%s: PDPTR check failed !", i->getIaOpcodeNameShort()));
       exception(BX_GP_EXCEPTION, 0);
     }
   }
@@ -491,7 +491,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR4Rd(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 5
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_CR4Rd: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -515,7 +515,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdCR0(bxInstruction_c *i)
 {
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_RdCR0: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -548,7 +548,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdCR2(bxInstruction_c *i)
 {
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_RdCR2: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -568,7 +568,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdCR3(bxInstruction_c *i)
 {
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_RdCR3: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -596,7 +596,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdCR4(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 5
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("MOV_RdCR4: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -619,7 +619,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RdCR4(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR0Rq(bxInstruction_c *i)
 {
   if (CPL!=0) {
-    BX_ERROR(("MOV_CR0Rq: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -648,12 +648,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR0Rq(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR2Rq(bxInstruction_c *i)
 {
   if (i->dst() != 2) {
-    BX_ERROR(("MOV_CR2Rq: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL!=0) {
-    BX_ERROR(("MOV_CR2Rq: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -676,12 +676,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR2Rq(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR3Rq(bxInstruction_c *i)
 {
   if (i->dst() != 3) {
-    BX_ERROR(("MOV_CR3Rq: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL!=0) {
-    BX_ERROR(("MOV_CR3Rq: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -717,12 +717,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR3Rq(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR4Rq(bxInstruction_c *i)
 {
   if (i->dst() != 4) {
-    BX_ERROR(("MOV_CR4Rq: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL!=0) {
-    BX_ERROR(("MOV_CR4Rq: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -744,7 +744,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_CR4Rq(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqCR0(bxInstruction_c *i)
 {
   if (CPL!=0) {
-    BX_ERROR(("MOV_RqCR0: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -774,12 +774,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqCR0(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqCR2(bxInstruction_c *i)
 {
   if (i->src() != 2) {
-    BX_ERROR(("MOV_RqCR2: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL!=0) {
-    BX_ERROR(("MOV_RqCR2: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -798,12 +798,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqCR2(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqCR3(bxInstruction_c *i)
 {
   if (i->src() != 3) {
-    BX_ERROR(("MOV_RqCR3: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL!=0) {
-    BX_ERROR(("MOV_RqCR3: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -827,12 +827,12 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqCR3(bxInstruction_c *i)
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RqCR4(bxInstruction_c *i)
 {
   if (i->src() != 4) {
-    BX_ERROR(("MOV_RqCR4: #UD - register index out of range"));
+    BX_ERROR(("%s: #UD - register index out of range", i->getIaOpcodeNameShort()));
     exception(BX_UD_EXCEPTION, 0);
   }
 
   if (CPL!=0) {
-    BX_ERROR(("MOV_RqCR4: #GP(0) if CPL is not 0"));
+    BX_ERROR(("%s: #GP(0) if CPL is not 0", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -857,7 +857,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::LMSW_Ew(bxInstruction_c *i)
 
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("LMSW: CPL!=0 not in real mode"));
+    BX_ERROR(("%s: CPL!=0 not in real mode", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
@@ -1148,20 +1148,20 @@ Bit32u BX_CPU_C::get_cr4_allow_mask(void)
   //   [0]     VME: Virtual-8086 Mode Extensions R/W
 
   /* VME */
-  if (bx_cpuid_support_vme())
+  if (is_cpu_extension_supported(BX_ISA_VME))
     allowMask |= BX_CR4_VME_MASK | BX_CR4_PVI_MASK;
 
-  if (bx_cpuid_support_tsc())
+  if (is_cpu_extension_supported(BX_ISA_PENTIUM))
     allowMask |= BX_CR4_TSD_MASK;
 
-  if (bx_cpuid_support_debug_extensions())
+  if (is_cpu_extension_supported(BX_ISA_DEBUG_EXTENSIONS))
     allowMask |= BX_CR4_DE_MASK;
 
-  if (bx_cpuid_support_pse())
+  if (is_cpu_extension_supported(BX_ISA_PSE))
     allowMask |= BX_CR4_PSE_MASK;
 
 #if BX_CPU_LEVEL >= 6
-  if (bx_cpuid_support_pae())
+  if (is_cpu_extension_supported(BX_ISA_PAE))
     allowMask |= BX_CR4_PAE_MASK;
 #endif
 
@@ -1169,43 +1169,43 @@ Bit32u BX_CPU_C::get_cr4_allow_mask(void)
   allowMask |= BX_CR4_MCE_MASK;
 
 #if BX_CPU_LEVEL >= 6
-  if (bx_cpuid_support_pge())
+  if (is_cpu_extension_supported(BX_ISA_PGE))
     allowMask |= BX_CR4_PGE_MASK;
 
   allowMask |= BX_CR4_PCE_MASK;
 
   /* OSFXSR */
-  if (bx_cpuid_support_fxsave_fxrstor())
+  if (is_cpu_extension_supported(BX_ISA_SSE))
     allowMask |= BX_CR4_OSFXSR_MASK;
 
   /* OSXMMEXCPT */
-  if (bx_cpuid_support_sse())
+  if (is_cpu_extension_supported(BX_ISA_SSE))
     allowMask |= BX_CR4_OSXMMEXCPT_MASK;
 
 #if BX_SUPPORT_VMX
-  if (bx_cpuid_support_vmx())
+  if (is_cpu_extension_supported(BX_ISA_VMX))
     allowMask |= BX_CR4_VMXE_MASK;
 #endif
 
-  if (bx_cpuid_support_smx())
+  if (is_cpu_extension_supported(BX_ISA_SMX))
     allowMask |= BX_CR4_SMXE_MASK;
 
 #if BX_SUPPORT_X86_64
-  if (bx_cpuid_support_pcid())
+  if (is_cpu_extension_supported(BX_ISA_PCID))
     allowMask |= BX_CR4_PCIDE_MASK;
 
-  if (bx_cpuid_support_fsgsbase())
+  if (is_cpu_extension_supported(BX_ISA_FSGSBASE))
     allowMask |= BX_CR4_FSGSBASE_MASK;
 #endif
 
   /* OSXSAVE */
-  if (bx_cpuid_support_xsave())
+  if (is_cpu_extension_supported(BX_ISA_XSAVE))
     allowMask |= BX_CR4_OSXSAVE_MASK;
 
-  if (bx_cpuid_support_smep())
+  if (is_cpu_extension_supported(BX_ISA_SMEP))
     allowMask |= BX_CR4_SMEP_MASK;
 
-  if (bx_cpuid_support_smap())
+  if (is_cpu_extension_supported(BX_ISA_SMAP))
     allowMask |= BX_CR4_SMAP_MASK;
 #endif
 
@@ -1444,7 +1444,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::CLTS(bxInstruction_c *i)
 {
   // CPL is always 0 in real mode
   if (/* !real_mode() && */ CPL!=0) {
-    BX_ERROR(("CLTS: priveledge check failed, generate #GP(0)"));
+    BX_ERROR(("%s: priveledge check failed, generate #GP(0)", i->getIaOpcodeNameShort()));
     exception(BX_GP_EXCEPTION, 0);
   }
 
