@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: crregs.h 11572 2013-01-14 17:02:51Z sshwarts $
+// $Id: crregs.h 12242 2014-03-15 20:19:30Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2007-2011 Stanislav Shwartsman
+//   Copyright (c) 2007-2014 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -231,19 +231,49 @@ struct bx_efer_t {
 
 #if BX_CPU_LEVEL >= 6
 
+#define XSAVE_SSE_STATE_LEN           (256)
+#define XSAVE_YMM_STATE_LEN           (256)
+#define XSAVE_OPMASK_STATE_LEN         (64)
+#define XSAVE_ZMM_HI256_STATE_LEN     (512)
+#define XSAVE_HI_ZMM_STATE_LEN       (1024)
+
+#define XSAVE_SSE_STATE_OFFSET        (160)
+#define XSAVE_YMM_STATE_OFFSET        (576)
+#define XSAVE_OPMASK_STATE_OFFSET    (1088)
+#define XSAVE_ZMM_HI256_STATE_OFFSET (1152)
+#define XSAVE_HI_ZMM_STATE_OFFSET    (1664)
+
 struct xcr0_t {
   Bit32u  val32; // 32bit value of register
 
-#define BX_XCR0_FPU_BIT   0
-#define BX_XCR0_FPU_MASK (1<<BX_XCR0_FPU_BIT)
-#define BX_XCR0_SSE_BIT   1
-#define BX_XCR0_SSE_MASK (1<<BX_XCR0_SSE_BIT)
-#define BX_XCR0_AVX_BIT   2
-#define BX_XCR0_AVX_MASK (1<<BX_XCR0_AVX_BIT)
+  enum {
+    BX_XCR0_FPU_BIT = 0,
+    BX_XCR0_SSE_BIT = 1,
+    BX_XCR0_YMM_BIT = 2,
+    BX_XCR0_BNDREGS_BIT = 3,
+    BX_XCR0_BNDCFG_BIT = 4,
+    BX_XCR0_OPMASK_BIT = 5,
+    BX_XCR0_ZMM_HI256_BIT = 6,
+    BX_XCR0_HI_ZMM_BIT = 7
+  };
+
+#define BX_XCR0_FPU_MASK       (1 << xcr0_t::BX_XCR0_FPU_BIT)
+#define BX_XCR0_SSE_MASK       (1 << xcr0_t::BX_XCR0_SSE_BIT)
+#define BX_XCR0_YMM_MASK       (1 << xcr0_t::BX_XCR0_YMM_BIT)
+#define BX_XCR0_BNDREGS_MASK   (1 << xcr0_t::BX_XCR0_BNDREGS_BIT)
+#define BX_XCR0_BNDCFG_MASK    (1 << xcr0_t::BX_XCR0_BNDCFG_BIT)
+#define BX_XCR0_OPMASK_MASK    (1 << xcr0_t::BX_XCR0_OPMASK_BIT)
+#define BX_XCR0_ZMM_HI256_MASK (1 << xcr0_t::BX_XCR0_ZMM_HI256_BIT)
+#define BX_XCR0_HI_ZMM_MASK    (1 << xcr0_t::BX_XCR0_HI_ZMM_BIT)
 
   IMPLEMENT_CRREG_ACCESSORS(FPU, BX_XCR0_FPU_BIT);
   IMPLEMENT_CRREG_ACCESSORS(SSE, BX_XCR0_SSE_BIT);
-  IMPLEMENT_CRREG_ACCESSORS(AVX, BX_XCR0_AVX_BIT);
+  IMPLEMENT_CRREG_ACCESSORS(YMM, BX_XCR0_YMM_BIT);
+  IMPLEMENT_CRREG_ACCESSORS(BNDREGS, BX_XCR0_BNDREGS_BIT);
+  IMPLEMENT_CRREG_ACCESSORS(BNDCFG, BX_XCR0_BNDCFG_BIT);
+  IMPLEMENT_CRREG_ACCESSORS(OPMASK, BX_XCR0_OPMASK_BIT);
+  IMPLEMENT_CRREG_ACCESSORS(ZMM_HI256, BX_XCR0_ZMM_HI256_BIT);
+  IMPLEMENT_CRREG_ACCESSORS(HI_ZMM, BX_XCR0_HI_ZMM_BIT);
 
   BX_CPP_INLINE Bit32u get32() const { return val32; }
   BX_CPP_INLINE void set32(Bit32u val) { val32 = val; }
