@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer32.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: data_xfer32.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,22 +24,15 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_ERXEAX(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdIdM(bxInstruction_c *i)
 {
-#if BX_SUPPORT_X86_64
-  if (i->dst() == 0) { // 'xchg eax, eax' is NOP even in 64-bit mode
-    BX_NEXT_INSTR(i);
-  }
-#endif
-
-  Bit32u temp32 = EAX;
-  RAX = BX_READ_32BIT_REG(i->dst());
-  BX_WRITE_32BIT_REGZ(i->dst(), temp32);
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  write_virtual_dword(i->seg(), eaddr, i->Id());
 
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_ERXId(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdIdR(bxInstruction_c *i)
 {
   BX_WRITE_32BIT_REGZ(i->dst(), i->Id());
 
@@ -114,15 +107,6 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_OdEAX(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdIdM(bxInstruction_c *i)
-{
-  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  write_virtual_dword(i->seg(), eaddr, i->Id());
-
-  BX_NEXT_INSTR(i);
-}
-
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEbM(bxInstruction_c *i)
 {
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
@@ -217,7 +201,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EdGdM(bxInstruction_c *i)
 
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   Bit32u op2_32 = BX_READ_32BIT_REG(i->src());
-  write_RMW_virtual_dword(op2_32);
+  write_RMW_linear_dword(op2_32);
   BX_WRITE_32BIT_REGZ(i->src(), op1_32);
 
   BX_NEXT_INSTR(i);
