@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer8.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: data_xfer8.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -24,20 +24,13 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RLIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EbIbR(bxInstruction_c *i)
 {
   BX_WRITE_8BIT_REGx(i->dst(), i->extend8bitL(), i->Ib());
 
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RHIb(bxInstruction_c *i)
-{
-  BX_WRITE_8BIT_REGH(i->dst() & 0x3, i->Ib());
-
-  BX_NEXT_INSTR(i);
-}
-
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EbGbM(bxInstruction_c *i)
 {
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
@@ -92,7 +85,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XLAT(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-    AL = read_virtual_byte_64(i->seg(), RBX + AL);
+    AL = read_linear_byte(i->seg(), get_laddr64(i->seg(), RBX + AL));
   }
   else
 #endif
@@ -113,7 +106,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EbGbM(bxInstruction_c *i)
   Bit8u op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   Bit8u op2 = BX_READ_8BIT_REGx(i->src(), i->extend8bitL());
 
-  write_RMW_virtual_byte(op2);
+  write_RMW_linear_byte(op2);
   BX_WRITE_8BIT_REGx(i->src(), i->extend8bitL(), op1);
 
   BX_NEXT_INSTR(i);
