@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: exception.cc 11580 2013-01-19 20:45:03Z sshwarts $
+// $Id: exception.cc 12570 2014-12-18 19:45:03Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001-2013  The Bochs Project
@@ -77,7 +77,7 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_error
     exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
-  // if software interrupt, then gate descripor DPL must be >= CPL,
+  // if software interrupt, then gate descriptor DPL must be >= CPL,
   // else #GP(vector * 8 + 2 + EXT)
   if (soft_int && gate_descriptor.dpl < CPL)
   {
@@ -160,17 +160,17 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_error
     RSP_for_cpl_x &= BX_CONST64(0xfffffffffffffff0);
 
     // push old stack long pointer onto new stack
-    write_new_stack_qword_64(RSP_for_cpl_x -  8, cs_descriptor.dpl, old_SS);
-    write_new_stack_qword_64(RSP_for_cpl_x - 16, cs_descriptor.dpl, old_RSP);
-    write_new_stack_qword_64(RSP_for_cpl_x - 24, cs_descriptor.dpl, read_eflags());
+    write_new_stack_qword(RSP_for_cpl_x -  8, cs_descriptor.dpl, old_SS);
+    write_new_stack_qword(RSP_for_cpl_x - 16, cs_descriptor.dpl, old_RSP);
+    write_new_stack_qword(RSP_for_cpl_x - 24, cs_descriptor.dpl, read_eflags());
     // push long pointer to return address onto new stack
-    write_new_stack_qword_64(RSP_for_cpl_x - 32, cs_descriptor.dpl, old_CS);
-    write_new_stack_qword_64(RSP_for_cpl_x - 40, cs_descriptor.dpl, old_RIP);
+    write_new_stack_qword(RSP_for_cpl_x - 32, cs_descriptor.dpl, old_CS);
+    write_new_stack_qword(RSP_for_cpl_x - 40, cs_descriptor.dpl, old_RIP);
     RSP_for_cpl_x -= 40;
 
     if (push_error) {
       RSP_for_cpl_x -= 8;
-      write_new_stack_qword_64(RSP_for_cpl_x, cs_descriptor.dpl, error_code);
+      write_new_stack_qword(RSP_for_cpl_x, cs_descriptor.dpl, error_code);
     }
 
     // load CS:RIP (guaranteed to be in 64 bit mode)
@@ -201,17 +201,17 @@ void BX_CPU_C::long_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_error
     // push flags onto stack
     // push current CS selector onto stack
     // push return offset onto stack
-    write_new_stack_qword_64(RSP_for_cpl_x - 8,  cs_descriptor.dpl, old_SS);
-    write_new_stack_qword_64(RSP_for_cpl_x - 16, cs_descriptor.dpl, old_RSP);
-    write_new_stack_qword_64(RSP_for_cpl_x - 24, cs_descriptor.dpl, read_eflags());
+    write_new_stack_qword(RSP_for_cpl_x - 8,  cs_descriptor.dpl, old_SS);
+    write_new_stack_qword(RSP_for_cpl_x - 16, cs_descriptor.dpl, old_RSP);
+    write_new_stack_qword(RSP_for_cpl_x - 24, cs_descriptor.dpl, read_eflags());
     // push long pointer to return address onto new stack
-    write_new_stack_qword_64(RSP_for_cpl_x - 32, cs_descriptor.dpl, old_CS);
-    write_new_stack_qword_64(RSP_for_cpl_x - 40, cs_descriptor.dpl, old_RIP);
+    write_new_stack_qword(RSP_for_cpl_x - 32, cs_descriptor.dpl, old_CS);
+    write_new_stack_qword(RSP_for_cpl_x - 40, cs_descriptor.dpl, old_RIP);
     RSP_for_cpl_x -= 40;
 
     if (push_error) {
       RSP_for_cpl_x -= 8;
-      write_new_stack_qword_64(RSP_for_cpl_x, cs_descriptor.dpl, error_code);
+      write_new_stack_qword(RSP_for_cpl_x, cs_descriptor.dpl, error_code);
     }
 
     // set the RPL field of CS to CPL
@@ -281,7 +281,7 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_
     exception(BX_GP_EXCEPTION, vector*8 + 2);
   }
 
-  // if software interrupt, then gate descripor DPL must be >= CPL,
+  // if software interrupt, then gate descriptor DPL must be >= CPL,
   // else #GP(vector * 8 + 2 + EXT)
   if (soft_int && gate_descriptor.dpl < CPL) {
     BX_ERROR(("interrupt(): soft_int && (gate.dpl < CPL)"));
@@ -465,24 +465,24 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_
         if (is_v8086_mode)
         {
           if (gate_descriptor.type>=14) { // 386 int/trap gate
-            write_new_stack_dword_32(&new_stack, temp_ESP-4,  cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, temp_ESP-4,  cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value);
-            write_new_stack_dword_32(&new_stack, temp_ESP-8,  cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, temp_ESP-8,  cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value);
-            write_new_stack_dword_32(&new_stack, temp_ESP-12, cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, temp_ESP-12, cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value);
-            write_new_stack_dword_32(&new_stack, temp_ESP-16, cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, temp_ESP-16, cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value);
             temp_ESP -= 16;
           }
           else {
-            write_new_stack_word_32(&new_stack, temp_ESP-2, cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, temp_ESP-2, cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value);
-            write_new_stack_word_32(&new_stack, temp_ESP-4, cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, temp_ESP-4, cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value);
-            write_new_stack_word_32(&new_stack, temp_ESP-6, cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, temp_ESP-6, cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value);
-            write_new_stack_word_32(&new_stack, temp_ESP-8, cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, temp_ESP-8, cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value);
             temp_ESP -= 8;
           }
@@ -490,30 +490,30 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_
 
         if (gate_descriptor.type>=14) { // 386 int/trap gate
           // push long pointer to old stack onto new stack
-          write_new_stack_dword_32(&new_stack, temp_ESP-4,  cs_descriptor.dpl, old_SS);
-          write_new_stack_dword_32(&new_stack, temp_ESP-8,  cs_descriptor.dpl, old_ESP);
-          write_new_stack_dword_32(&new_stack, temp_ESP-12, cs_descriptor.dpl, read_eflags());
-          write_new_stack_dword_32(&new_stack, temp_ESP-16, cs_descriptor.dpl, old_CS);
-          write_new_stack_dword_32(&new_stack, temp_ESP-20, cs_descriptor.dpl, old_EIP);
+          write_new_stack_dword(&new_stack, temp_ESP-4,  cs_descriptor.dpl, old_SS);
+          write_new_stack_dword(&new_stack, temp_ESP-8,  cs_descriptor.dpl, old_ESP);
+          write_new_stack_dword(&new_stack, temp_ESP-12, cs_descriptor.dpl, read_eflags());
+          write_new_stack_dword(&new_stack, temp_ESP-16, cs_descriptor.dpl, old_CS);
+          write_new_stack_dword(&new_stack, temp_ESP-20, cs_descriptor.dpl, old_EIP);
           temp_ESP -= 20;
 
           if (push_error) {
             temp_ESP -= 4;
-            write_new_stack_dword_32(&new_stack, temp_ESP, cs_descriptor.dpl, error_code);
+            write_new_stack_dword(&new_stack, temp_ESP, cs_descriptor.dpl, error_code);
           }
         }
         else {                          // 286 int/trap gate
           // push long pointer to old stack onto new stack
-          write_new_stack_word_32(&new_stack, temp_ESP-2,  cs_descriptor.dpl, old_SS);
-          write_new_stack_word_32(&new_stack, temp_ESP-4,  cs_descriptor.dpl, (Bit16u) old_ESP);
-          write_new_stack_word_32(&new_stack, temp_ESP-6,  cs_descriptor.dpl, (Bit16u) read_eflags());
-          write_new_stack_word_32(&new_stack, temp_ESP-8,  cs_descriptor.dpl, old_CS);
-          write_new_stack_word_32(&new_stack, temp_ESP-10, cs_descriptor.dpl, (Bit16u) old_EIP);
+          write_new_stack_word(&new_stack, temp_ESP-2,  cs_descriptor.dpl, old_SS);
+          write_new_stack_word(&new_stack, temp_ESP-4,  cs_descriptor.dpl, (Bit16u) old_ESP);
+          write_new_stack_word(&new_stack, temp_ESP-6,  cs_descriptor.dpl, (Bit16u) read_eflags());
+          write_new_stack_word(&new_stack, temp_ESP-8,  cs_descriptor.dpl, old_CS);
+          write_new_stack_word(&new_stack, temp_ESP-10, cs_descriptor.dpl, (Bit16u) old_EIP);
           temp_ESP -= 10;
 
           if (push_error) {
             temp_ESP -= 2;
-            write_new_stack_word_32(&new_stack, temp_ESP, cs_descriptor.dpl, error_code);
+            write_new_stack_word(&new_stack, temp_ESP, cs_descriptor.dpl, error_code);
           }
         }
 
@@ -525,24 +525,24 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_
         if (is_v8086_mode)
         {
           if (gate_descriptor.type>=14) { // 386 int/trap gate
-            write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-4),  cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-4),  cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value);
-            write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-8),  cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-8),  cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value);
-            write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-12), cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-12), cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value);
-            write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-16), cs_descriptor.dpl,
+            write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-16), cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value);
             temp_SP -= 16;
           }
           else {
-            write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-2), cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, (Bit16u)(temp_SP-2), cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value);
-            write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-4), cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, (Bit16u)(temp_SP-4), cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value);
-            write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-6), cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, (Bit16u)(temp_SP-6), cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value);
-            write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-8), cs_descriptor.dpl,
+            write_new_stack_word(&new_stack, (Bit16u)(temp_SP-8), cs_descriptor.dpl,
                 BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value);
             temp_SP -= 8;
           }
@@ -550,30 +550,30 @@ void BX_CPU_C::protected_mode_int(Bit8u vector, unsigned soft_int, bx_bool push_
 
         if (gate_descriptor.type>=14) { // 386 int/trap gate
           // push long pointer to old stack onto new stack
-          write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-4),  cs_descriptor.dpl, old_SS);
-          write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-8),  cs_descriptor.dpl, old_ESP);
-          write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-12), cs_descriptor.dpl, read_eflags());
-          write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-16), cs_descriptor.dpl, old_CS);
-          write_new_stack_dword_32(&new_stack, (Bit16u)(temp_SP-20), cs_descriptor.dpl, old_EIP);
+          write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-4),  cs_descriptor.dpl, old_SS);
+          write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-8),  cs_descriptor.dpl, old_ESP);
+          write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-12), cs_descriptor.dpl, read_eflags());
+          write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-16), cs_descriptor.dpl, old_CS);
+          write_new_stack_dword(&new_stack, (Bit16u)(temp_SP-20), cs_descriptor.dpl, old_EIP);
           temp_SP -= 20;
 
           if (push_error) {
             temp_SP -= 4;
-            write_new_stack_dword_32(&new_stack, temp_SP, cs_descriptor.dpl, error_code);
+            write_new_stack_dword(&new_stack, temp_SP, cs_descriptor.dpl, error_code);
           }
         }
         else {                          // 286 int/trap gate
           // push long pointer to old stack onto new stack
-          write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-2),  cs_descriptor.dpl, old_SS);
-          write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-4),  cs_descriptor.dpl, (Bit16u) old_ESP);
-          write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-6),  cs_descriptor.dpl, (Bit16u) read_eflags());
-          write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-8),  cs_descriptor.dpl, old_CS);
-          write_new_stack_word_32(&new_stack, (Bit16u)(temp_SP-10), cs_descriptor.dpl, (Bit16u) old_EIP);
+          write_new_stack_word(&new_stack, (Bit16u)(temp_SP-2),  cs_descriptor.dpl, old_SS);
+          write_new_stack_word(&new_stack, (Bit16u)(temp_SP-4),  cs_descriptor.dpl, (Bit16u) old_ESP);
+          write_new_stack_word(&new_stack, (Bit16u)(temp_SP-6),  cs_descriptor.dpl, (Bit16u) read_eflags());
+          write_new_stack_word(&new_stack, (Bit16u)(temp_SP-8),  cs_descriptor.dpl, old_CS);
+          write_new_stack_word(&new_stack, (Bit16u)(temp_SP-10), cs_descriptor.dpl, (Bit16u) old_EIP);
           temp_SP -= 10;
 
           if (push_error) {
             temp_SP -= 2;
-            write_new_stack_word_32(&new_stack, temp_SP, cs_descriptor.dpl, error_code);
+            write_new_stack_word(&new_stack, temp_SP, cs_descriptor.dpl, error_code);
           }
         }
 
@@ -741,7 +741,7 @@ void BX_CPU_C::interrupt(Bit8u vector, unsigned type, bx_bool push_error, Bit16u
   {
     RSP_SPECULATIVE;
 
-    // software interrupt can be redirefcted in v8086 mode
+    // software interrupt can be redirected in v8086 mode
     if (type != BX_SOFTWARE_INTERRUPT || !v8086_mode() || !v86_redirect_interrupt(vector))
     {
       if(real_mode()) {
@@ -825,14 +825,6 @@ struct BxExceptionInfo exceptions_info[BX_CPU_HANDLED_EXCEPTIONS] = {
 // error_code: if exception generates and error, push this error code
 void BX_CPU_C::exception(unsigned vector, Bit16u error_code)
 {
-  BX_INSTR_EXCEPTION(BX_CPU_ID, vector, error_code);
-
-#if BX_DEBUGGER
-  bx_dbg_exception(BX_CPU_ID, vector, error_code);
-#endif
-
-  BX_DEBUG(("exception(0x%02x): error_code=%04x", vector, error_code));
-
   unsigned exception_type = 0;
   unsigned exception_class = BX_EXCEPTION_CLASS_FAULT;
   bx_bool push_error = 0;
@@ -846,11 +838,23 @@ void BX_CPU_C::exception(unsigned vector, Bit16u error_code)
      BX_PANIC(("exception(%u): bad vector", vector));
   }
 
+  /* Excluding page faults and double faults, error_code may not have the
+   * least significant bit set correctly. This correction is applied first
+   * to make the change transparent to any instrumentation.
+   */
   if (vector != BX_PF_EXCEPTION && vector != BX_DF_EXCEPTION) {
     // Page faults have different format
     error_code = (error_code & 0xfffe) | BX_CPU_THIS_PTR EXT;
   }
 
+  BX_INSTR_EXCEPTION(BX_CPU_ID, vector, error_code);
+
+#if BX_DEBUGGER
+  bx_dbg_exception(BX_CPU_ID, vector, error_code);
+#endif
+
+  BX_DEBUG(("exception(0x%02x): error_code=%04x", vector, error_code));
+
 #if BX_SUPPORT_VMX
   VMexit_Event(BX_HARDWARE_EXCEPTION, vector, error_code, push_error);
 #endif
@@ -865,6 +869,7 @@ void BX_CPU_C::exception(unsigned vector, Bit16u error_code)
     RIP = BX_CPU_THIS_PTR prev_rip;
     if (BX_CPU_THIS_PTR speculative_rsp)
       RSP = BX_CPU_THIS_PTR prev_rsp;
+    BX_CPU_THIS_PTR speculative_rsp = 0;
 
     if (BX_CPU_THIS_PTR last_exception_type == BX_ET_DOUBLE_FAULT)
     {
@@ -873,7 +878,7 @@ void BX_CPU_C::exception(unsigned vector, Bit16u error_code)
       VMexit_TripleFault();
 #endif
 #if BX_DEBUGGER
-      // trap into debugger (similar as done when PANIC occured)
+      // trap into debugger (the same as when a PANIC occurs)
       bx_debug_break();
 #endif
       if (SIM->get_param_bool(BXPN_RESET_ON_TRIPLE_FAULT)->get()) {
