@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_load_store.cc 11415 2012-09-12 21:08:40Z vruppert $
+// $Id: fpu_load_store.cc 12736 2015-05-02 20:08:36Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003-2012 Stanislav Shwartsman
@@ -30,7 +30,7 @@
 
 #define swap_values16u(a, b) { Bit16u tmp = a; a = b; b = tmp; }
 
-extern float_status_t FPU_pre_exception_handling(Bit16u control_word);
+extern float_status_t i387cw_to_softfloat_status_word(Bit16u control_word);
 
 #include "softfloatx80.h"
 
@@ -83,7 +83,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_SINGLE_REAL(bxInstruction_c *i
   }
 
   float_status_t status =
-    FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+    i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   // convert to floatx80 format
   floatx80 result = float32_to_floatx80(load_reg, status);
@@ -114,7 +114,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD_DOUBLE_REAL(bxInstruction_c *i
   }
 
   float_status_t status =
-    FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+    i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   // convert to floatx80 format
   floatx80 result = float64_to_floatx80(load_reg, status);
@@ -321,7 +321,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_SINGLE_REAL(bxInstruction_c *i
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_float32(BX_READ_FPU_REG(0), status);
 
@@ -369,7 +369,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FST_DOUBLE_REAL(bxInstruction_c *i
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_float64(BX_READ_FPU_REG(0), status);
 
@@ -450,7 +450,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_WORD_INTEGER(bxInstruction_c
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_int16(BX_READ_FPU_REG(0), status);
 
@@ -498,7 +498,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FIST_DWORD_INTEGER(bxInstruction_c
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_int32(BX_READ_FPU_REG(0), status);
 
@@ -542,7 +542,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTP_QWORD_INTEGER(bxInstruction_
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_int64(BX_READ_FPU_REG(0), status);
 
@@ -592,7 +592,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FBSTP_PACKED_BCD(bxInstruction_c *
   else
   {
      float_status_t status =
-        FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+        i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      floatx80 reg = BX_READ_FPU_REG(0);
 
@@ -665,7 +665,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP16(bxInstruction_c *i)
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_int16_round_to_zero(BX_READ_FPU_REG(0), status);
 
@@ -710,7 +710,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP32(bxInstruction_c *i)
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_int32_round_to_zero(BX_READ_FPU_REG(0), status);
 
@@ -755,7 +755,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::FISTTP64(bxInstruction_c *i)
   else
   {
      float_status_t status =
-         FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+         i387cw_to_softfloat_status_word(BX_CPU_THIS_PTR the_i387.get_control_word());
 
      save_reg = floatx80_to_int64_round_to_zero(BX_READ_FPU_REG(0), status);
 
