@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: icache.h 11389 2012-09-01 19:13:01Z sshwarts $
+// $Id: icache.h 12678 2015-03-01 20:55:23Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2007-2011 Stanislav Shwartsman
+//   Copyright (c) 2007-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -38,7 +38,7 @@ public:
   }
  ~bxPageWriteStampTable() { delete [] fineGranularityMapping; }
 
-  BX_CPP_INLINE Bit32u hash(bx_phy_address pAddr) const {
+  BX_CPP_INLINE static Bit32u hash(bx_phy_address pAddr) {
     // can share writeStamps between multiple pages if >32 bit phy address
     return ((Bit32u) pAddr) >> 12;
   }
@@ -101,7 +101,7 @@ BX_CPP_INLINE void bxPageWriteStampTable::resetWriteStamps(void)
 
 extern bxPageWriteStampTable pageWriteStampTable;
 
-#define BxICacheEntries (256 * 1024)  // Must be a power of 2.
+#define BxICacheEntries (64  * 1024)  // Must be a power of 2.
 #define BxICacheMemPool (576 * 1024)
 
 #define BX_MAX_TRACE_LENGTH 32
@@ -117,12 +117,27 @@ struct bxICacheEntry_c
 
 #define BX_ICACHE_INVALID_PHY_ADDRESS (bx_phy_address(-1))
 
+BX_CPP_INLINE void flushSMC(bxICacheEntry_c *e)
+{
+  if (e->pAddr != BX_ICACHE_INVALID_PHY_ADDRESS) {
+    e->pAddr = BX_ICACHE_INVALID_PHY_ADDRESS;
+#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
+    extern void genDummyICacheEntry(bxInstruction_c *i);
+//  for (unsigned instr=0;instr < e->tlen; instr++)
+//    genDummyICacheEntry(e->i + instr);
+    genDummyICacheEntry(e->i);
+#endif
+  }
+}
+
 class BOCHSAPI bxICache_c {
 public:
   bxICacheEntry_c entry[BxICacheEntries];
   bxInstruction_c mpool[BxICacheMemPool];
   unsigned mpindex;
 
+  Bit32u traceLinkTimeStamp;
+
 #define BX_ICACHE_PAGE_SPLIT_ENTRIES 8 /* must be power of two */
   struct pageSplitEntryIndex {
     bx_phy_address ppf; // Physical address of 2nd page of the trace 
@@ -140,7 +155,7 @@ public:
 public:
   bxICache_c() { flushICacheEntries(); }
 
-  BX_CPP_INLINE unsigned hash(bx_phy_address pAddr, unsigned fetchModeMask) const
+  BX_CPP_INLINE static unsigned hash(bx_phy_address pAddr, unsigned fetchModeMask)
   {
 //  return ((pAddr + (pAddr << 2) + (pAddr>>6)) & (BxICacheEntries-1)) ^ fetchModeMask;
     return ((pAddr) & (BxICacheEntries-1)) ^ fetchModeMask;
@@ -158,16 +173,16 @@ public:
 
   BX_CPP_INLINE void commit_trace(unsigned len) { mpindex += len; }
 
-  BX_CPP_INLINE void commit_page_split_trace(bx_phy_address paddr, bxICacheEntry_c *entry)
+  BX_CPP_INLINE void commit_page_split_trace(bx_phy_address paddr, bxICacheEntry_c *e)
   {
-    mpindex += entry->tlen;
+    mpindex += e->tlen;
 
     // register page split entry
     if (pageSplitIndex[nextPageSplitIndex].ppf != BX_ICACHE_INVALID_PHY_ADDRESS)
       pageSplitIndex[nextPageSplitIndex].e->pAddr = BX_ICACHE_INVALID_PHY_ADDRESS;
 
     pageSplitIndex[nextPageSplitIndex].ppf = paddr;
-    pageSplitIndex[nextPageSplitIndex].e = entry;
+    pageSplitIndex[nextPageSplitIndex].e = e;
 
     nextPageSplitIndex = (nextPageSplitIndex+1) & (BX_ICACHE_PAGE_SPLIT_ENTRIES-1);
   }
@@ -184,11 +199,11 @@ public:
     return NULL;
   }
 
-  BX_CPP_INLINE void victim_entry(bxICacheEntry_c *entry, Bit32u fetchModeMask)
+  BX_CPP_INLINE void victim_entry(bxICacheEntry_c *e, Bit32u fetchModeMask)
   {
-    if (entry->pAddr != BX_ICACHE_INVALID_PHY_ADDRESS) {
+    if (e->pAddr != BX_ICACHE_INVALID_PHY_ADDRESS) {
       victimCache[nextVictimCacheIndex].fetchModeMask = fetchModeMask;
-      victimCache[nextVictimCacheIndex].vc_entry = *entry;
+      victimCache[nextVictimCacheIndex].vc_entry = *e;
       nextVictimCacheIndex = (nextVictimCacheIndex+1) & (BX_ICACHE_VICTIM_ENTRIES-1);
     }
   }
@@ -201,6 +216,15 @@ public:
   {
     return &(entry[hash(pAddr, fetchModeMask)]);
   }
+
+  BX_CPP_INLINE bxICacheEntry_c* find_entry(bx_phy_address pAddr, unsigned fetchModeMask)
+  {
+    bxICacheEntry_c* e = &entry[hash(pAddr, fetchModeMask)];
+    if (e->pAddr != pAddr)
+       e = lookup_victim_cache(pAddr, fetchModeMask);
+
+    return e;
+  }
 };
 
 BX_CPP_INLINE void bxICache_c::flushICacheEntries(void)
@@ -222,46 +246,59 @@ BX_CPP_INLINE void bxICache_c::flushICacheEntries(void)
     victimCache[i].vc_entry.pAddr = BX_ICACHE_INVALID_PHY_ADDRESS;
 
   mpindex = 0;
-}
 
-#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
-extern void genDummyICacheEntry(bxInstruction_c *i);
-#endif
+  traceLinkTimeStamp = 0;
+}
 
 BX_CPP_INLINE void bxICache_c::handleSMC(bx_phy_address pAddr, Bit32u mask)
 {
-  pAddr = LPFOf(pAddr);
+  Bit32u pAddrIndex = bxPageWriteStampTable::hash(pAddr);
 
-  unsigned i;
+  // break all links bewteen traces
+  if (++traceLinkTimeStamp == 0xffffffff) {
+    flushICacheEntries();
+    return;
+  }
+
+  // Need to invalidate all traces in the trace cache that might include an
+  // instruction that was modified.  But this is not enough, it is possible
+  // that some another trace is linked into  invalidated trace and it won't
+  // be invalidated. In order to solve this issue  replace all instructions
+  // from the invalidated trace with dummy EndOfTrace opcodes.
+
+  // Another corner case that has to be handled - pageWriteStampTable wrap.
+  // Multiple physical addresses could be mapped into single pageWriteStampTable
+  // entry and all of them have to be invalidated here now.
 
   if (mask & 0x1) {
     // the store touched 1st cache line in the page, check for
     // page split traces to invalidate.
-    for (i=0;i<BX_ICACHE_PAGE_SPLIT_ENTRIES;i++) {
-      if (pAddr == pageSplitIndex[i].ppf) {
-        pageSplitIndex[i].ppf = BX_ICACHE_INVALID_PHY_ADDRESS;
+    for (unsigned i=0;i<BX_ICACHE_PAGE_SPLIT_ENTRIES;i++) {
+      if (pageSplitIndex[i].ppf != BX_ICACHE_INVALID_PHY_ADDRESS) {
+        if (pAddrIndex == bxPageWriteStampTable::hash(pageSplitIndex[i].ppf)) {
+          pageSplitIndex[i].ppf = BX_ICACHE_INVALID_PHY_ADDRESS;
+          flushSMC(pageSplitIndex[i].e);
+        }
       }
     }
   }
 
-  for (i=0;i < BX_ICACHE_VICTIM_ENTRIES; i++) {
-    bxVictimCacheEntry *e = &victimCache[i];
-    e->vc_entry.pAddr = BX_ICACHE_INVALID_PHY_ADDRESS;
+  for (unsigned i=0;i < BX_ICACHE_VICTIM_ENTRIES; i++) {
+    bxICacheEntry_c *e = &victimCache[i].vc_entry;
+    if (pAddrIndex == bxPageWriteStampTable::hash(e->pAddr) && (e->traceMask & mask) != 0) {
+      flushSMC(e);
+    }
   }
 
-  bxICacheEntry_c *e = get_entry(pAddr, 0);
+  bxICacheEntry_c *e = get_entry(LPFOf(pAddr), 0);
 
   // go over 32 "cache lines" of 128 byte each
   for (unsigned n=0; n < 32; n++) {
     Bit32u line_mask = (1 << n);
     if (line_mask > mask) break;
     for (unsigned index=0; index < 128; index++, e++) {
-      if (pAddr == LPFOf(e->pAddr) && (e->traceMask & mask) != 0) {
-        e->pAddr = BX_ICACHE_INVALID_PHY_ADDRESS;
-#if BX_SUPPORT_HANDLERS_CHAINING_SPEEDUPS
-        for (unsigned instr=0;instr < e->tlen; instr++)
-          genDummyICacheEntry(e->i + instr);
-#endif
+      if (pAddrIndex == bxPageWriteStampTable::hash(e->pAddr) && (e->traceMask & mask) != 0) {
+        flushSMC(e);
       }
     }
   }
