@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: init.cc 11674 2013-04-09 15:43:15Z sshwarts $
+// $Id: init.cc 12697 2015-03-27 21:39:24Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 #define LOG_THIS BX_CPU_THIS_PTR
 
 #include "param_names.h"
+#include "cpustats.h"
 
 #include <stdlib.h>
 
@@ -45,8 +46,13 @@ BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
   sprintf(logname, "cpu%x", bx_cpuid);
   put(logname, name);
 
-  isa_extensions_bitmask = BX_SUPPORT_FPU ? BX_ISA_X87 : 0;
-  cpu_extensions_bitmask = 0;
+  for (unsigned n=0;n<BX_ISA_EXTENSIONS_ARRAY_SIZE;n++)
+    ia_extensions_bitmask[n] = 0;
+
+  ia_extensions_bitmask[0] = (1 << BX_ISA_386);
+  if (BX_SUPPORT_FPU)
+    ia_extensions_bitmask[0] = (1 << BX_ISA_X87);
+
 #if BX_SUPPORT_VMX
   vmx_extensions_bitmask = 0;
 #endif
@@ -54,6 +60,8 @@ BX_CPU_C::BX_CPU_C(unsigned id): bx_cpuid(id)
   svm_extensions_bitmask = 0;
 #endif
 
+  stats = NULL;
+
   srand(time(NULL)); // initialize random generator for RDRAND/RDSEED
 }
 
@@ -94,8 +102,8 @@ void BX_CPU_C::initialize(void)
   if (! BX_CPU_THIS_PTR cpuid)
     BX_PANIC(("Failed to create CPUID module !"));
 
-  BX_CPU_THIS_PTR isa_extensions_bitmask = cpuid->get_isa_extensions_bitmask();
-  BX_CPU_THIS_PTR cpu_extensions_bitmask = cpuid->get_cpu_extensions_bitmask();
+  cpuid->get_cpu_extensions(BX_CPU_THIS_PTR ia_extensions_bitmask);
+
 #if BX_SUPPORT_VMX
   BX_CPU_THIS_PTR vmx_extensions_bitmask = cpuid->get_vmx_extensions_bitmask();
 #endif
@@ -123,27 +131,68 @@ void BX_CPU_C::initialize(void)
 
 #if BX_SUPPORT_VMX
   init_VMCS();
+#endif
+
+  init_statistics();
+}
+
+// statistics
+void BX_CPU_C::init_statistics(void)
+{
+#if InstrumentCPU
+  stats = new bx_cpu_statistics;
+
+  bx_list_c *cpu = new bx_list_c(SIM->get_statistics_root(), get_name(), get_name());
+
+#if InstrumentICACHE
+  new bx_shadow_num_c(cpu, "iCacheLookups", &stats->iCacheLookups);
+  new bx_shadow_num_c(cpu, "iCachePrefetch", &stats->iCachePrefetch);
+  new bx_shadow_num_c(cpu, "iCacheMisses", &stats->iCacheMisses);
+#endif
+
+#if InstrumentTLB
+  new bx_shadow_num_c(cpu, "tlbLookups", &stats->tlbLookups);
+  new bx_shadow_num_c(cpu, "tlbMisses", &stats->tlbMisses);
+#endif
+
+#if InstrumentTLBFlush
+  new bx_shadow_num_c(cpu, "tlbGlobalFlushes", &stats->tlbGlobalFlushes);
+  new bx_shadow_num_c(cpu, "tlbNonGlobalFlushes", &stats->tlbNonGlobalFlushes);
+#endif
+
+#if InstrumentStackPrefetch
+  new bx_shadow_num_c(cpu, "stackPrefetch", &stats->stackPrefetch);
+#endif
+
+#if InstrumentSMC
+  new bx_shadow_num_c(cpu, "smc", &stats->smc);
+#endif
+
 #endif
 }
 
 // save/restore functionality
 void BX_CPU_C::register_state(void)
 {
+  char name[128]; // to fit long enough name
   unsigned n;
-  char name[10];
 
   sprintf(name, "cpu%d", BX_CPU_ID);
 
   bx_list_c *cpu = new bx_list_c(SIM->get_bochs_root(), name, name);
 
-  BXRS_HEX_PARAM_SIMPLE(cpu, isa_extensions_bitmask);
-  BXRS_HEX_PARAM_SIMPLE(cpu, cpu_extensions_bitmask);
+  for (n=0;n<BX_ISA_EXTENSIONS_ARRAY_SIZE;n++) {
+    sprintf(name, "ia_extensions_bitmask_%u", n);
+    new bx_shadow_num_c(cpu, name, &ia_extensions_bitmask[n], BASE_HEX);
+  }
+
 #if BX_SUPPORT_VMX
   BXRS_HEX_PARAM_SIMPLE(cpu, vmx_extensions_bitmask);
 #endif
 #if BX_SUPPORT_SVM
   BXRS_HEX_PARAM_SIMPLE(cpu, svm_extensions_bitmask);
 #endif
+
   BXRS_DEC_PARAM_SIMPLE(cpu, cpu_mode);
   BXRS_HEX_PARAM_SIMPLE(cpu, activity_state);
   BXRS_HEX_PARAM_SIMPLE(cpu, inhibit_mask);
@@ -151,6 +200,7 @@ void BX_CPU_C::register_state(void)
   BXRS_HEX_PARAM_SIMPLE(cpu, debug_trap);
   BXRS_DEC_PARAM_SIMPLE(cpu, icount);
   BXRS_DEC_PARAM_SIMPLE(cpu, icount_last_sync);
+
 #if BX_SUPPORT_X86_64
   BXRS_HEX_PARAM_SIMPLE(cpu, RAX);
   BXRS_HEX_PARAM_SIMPLE(cpu, RBX);
@@ -182,6 +232,7 @@ void BX_CPU_C::register_state(void)
 #endif
   BXRS_PARAM_SPECIAL32(cpu, EFLAGS,
          param_save_handler, param_restore_handler);
+
 #if BX_CPU_LEVEL >= 3
   BXRS_HEX_PARAM_FIELD(cpu, DR0, dr[0]);
   BXRS_HEX_PARAM_FIELD(cpu, DR1, dr[1]);
@@ -190,23 +241,27 @@ void BX_CPU_C::register_state(void)
   BXRS_HEX_PARAM_FIELD(cpu, DR6, dr6.val32);
   BXRS_HEX_PARAM_FIELD(cpu, DR7, dr7.val32);
 #endif
+
   BXRS_HEX_PARAM_FIELD(cpu, CR0, cr0.val32);
   BXRS_HEX_PARAM_FIELD(cpu, CR2, cr2);
   BXRS_HEX_PARAM_FIELD(cpu, CR3, cr3);
 #if BX_CPU_LEVEL >= 5
   BXRS_HEX_PARAM_FIELD(cpu, CR4, cr4.val32);
 #endif
+
 #if BX_CPU_LEVEL >= 6
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_XSAVE)) {
     BXRS_HEX_PARAM_FIELD(cpu, XCR0, xcr0.val32);
   }
 #endif
+
 #if BX_CPU_LEVEL >= 5
   BXRS_HEX_PARAM_FIELD(cpu, tsc_last_reset, tsc_last_reset);
 #if BX_SUPPORT_VMX || BX_SUPPORT_SVM
   BXRS_HEX_PARAM_FIELD(cpu, tsc_offset, tsc_offset);
 #endif
 #endif
+
   for(n=0; n<6; n++) {
     bx_segment_reg_t *segment = &BX_CPU_THIS_PTR sregs[n];
     bx_list_c *sreg = new bx_list_c(cpu, strseg(segment));
@@ -280,7 +335,7 @@ void BX_CPU_C::register_state(void)
   BXRS_HEX_PARAM_FIELD(MSR, EFER, efer.val32);
   BXRS_HEX_PARAM_FIELD(MSR,  star, msr.star);
 #if BX_SUPPORT_X86_64
-  if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_LONG_MODE)) {
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_LONG_MODE)) {
     BXRS_HEX_PARAM_FIELD(MSR, lstar, msr.lstar);
     BXRS_HEX_PARAM_FIELD(MSR, cstar, msr.cstar);
     BXRS_HEX_PARAM_FIELD(MSR, fmask, msr.fmask);
@@ -309,20 +364,20 @@ void BX_CPU_C::register_state(void)
   BXRS_HEX_PARAM_FIELD(MSR, mtrrphysbase7, msr.mtrrphys[14]);
   BXRS_HEX_PARAM_FIELD(MSR, mtrrphysmask7, msr.mtrrphys[15]);
 
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix64k_00000, msr.mtrrfix64k_00000);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix16k_80000, msr.mtrrfix16k[0]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix16k_a0000, msr.mtrrfix16k[1]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix64k, msr.mtrrfix64k.u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix16k_80000, msr.mtrrfix16k[0].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix16k_a0000, msr.mtrrfix16k[1].u64);
 
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_c0000, msr.mtrrfix4k[0]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_c8000, msr.mtrrfix4k[1]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_d0000, msr.mtrrfix4k[2]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_d8000, msr.mtrrfix4k[3]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_e0000, msr.mtrrfix4k[4]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_e8000, msr.mtrrfix4k[5]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_f0000, msr.mtrrfix4k[6]);
-  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_f8000, msr.mtrrfix4k[7]);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_c0000, msr.mtrrfix4k[0].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_c8000, msr.mtrrfix4k[1].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_d0000, msr.mtrrfix4k[2].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_d8000, msr.mtrrfix4k[3].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_e0000, msr.mtrrfix4k[4].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_e8000, msr.mtrrfix4k[5].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_f0000, msr.mtrrfix4k[6].u64);
+  BXRS_HEX_PARAM_FIELD(MSR, mtrrfix4k_f8000, msr.mtrrfix4k[7].u64);
 
-  BXRS_HEX_PARAM_FIELD(MSR, pat, msr.pat);
+  BXRS_HEX_PARAM_FIELD(MSR, pat, msr.pat.u64);
   BXRS_HEX_PARAM_FIELD(MSR, mtrr_deftype, msr.mtrr_deftype);
 #endif
 #if BX_CONFIGURE_MSRS
@@ -367,15 +422,20 @@ void BX_CPU_C::register_state(void)
     for (n=0; n<BX_XMM_REGISTERS; n++) {
       for(unsigned j=0;j < BX_VLMAX*2;j++) {
         sprintf(name, "xmm%02d_%d", n, j);
-#if BX_SUPPORT_AVX
-        new bx_shadow_num_c(sse, name, &vmm[n].avx64u(j), BASE_HEX);
-#else
-        new bx_shadow_num_c(sse, name, &vmm[n].xmm64u(j), BASE_HEX);
-#endif
+        new bx_shadow_num_c(sse, name, &vmm[n].vmm64u(j), BASE_HEX);
       }
     }
   }
+#if BX_SUPPORT_EVEX
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512)) {
+    bx_list_c *mask = new bx_list_c(cpu, "OPMASK");
+    for (n=0; n<8; n++) {
+      sprintf(name, "k%d", n);
+      new bx_shadow_num_c(mask, name, &opmask[n].rrx, BASE_HEX);
+    }
+  }
 #endif
+#endif // BX_CPU_LEVEL >= 6
 
 #if BX_SUPPORT_MONITOR_MWAIT
   bx_list_c *monitor_list = new bx_list_c(cpu, "MONITOR");
@@ -411,7 +471,7 @@ void BX_CPU_C::register_state(void)
   BXRS_PARAM_BOOL(tlb, split_large, TLB.split_large);
 #endif
   for (n=0; n<BX_TLB_SIZE; n++) {
-    sprintf(name, "entry%d", n);
+    sprintf(name, "entry%u", n);
     bx_list_c *tlb_entry = new bx_list_c(tlb, name);
     BXRS_HEX_PARAM_FIELD(tlb_entry, lpf, TLB.entry[n].lpf);
     BXRS_HEX_PARAM_FIELD(tlb_entry, lpf_mask, TLB.entry[n].lpf_mask);
@@ -542,6 +602,10 @@ BX_CPU_C::~BX_CPU_C()
   delete cpuid;
 #endif
 
+#if InstrumentCPU
+  delete stats;
+#endif
+
   BX_INSTR_EXIT(BX_CPU_ID);
   BX_DEBUG(("Exit."));
 }
@@ -568,7 +632,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR eflags = 0x2; // Bit1 is always set
   // clear lazy flags state to satisfy Valgrind uninitialized variables checker
   memset(&BX_CPU_THIS_PTR oszapc, 0, sizeof(BX_CPU_THIS_PTR oszapc));
-  setEFlagsOSZAPC(0);	        // update lazy flags state
+  clearEFlagsOSZAPC();	        // update lazy flags state
 
   if (source == BX_RESET_HARDWARE)
     BX_CPU_THIS_PTR icount = 0;
@@ -663,7 +727,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR ldtr.selector.ti    = 0;
   BX_CPU_THIS_PTR ldtr.selector.rpl   = 0;
 
-  BX_CPU_THIS_PTR ldtr.cache.valid    = 1; /* valid */
+  BX_CPU_THIS_PTR ldtr.cache.valid    = SegValidCache; /* valid */
   BX_CPU_THIS_PTR ldtr.cache.p        = 1; /* present */
   BX_CPU_THIS_PTR ldtr.cache.dpl      = 0; /* field not used */
   BX_CPU_THIS_PTR ldtr.cache.segment  = 0; /* system segment */
@@ -679,7 +743,7 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR tr.selector.ti    = 0;
   BX_CPU_THIS_PTR tr.selector.rpl   = 0;
 
-  BX_CPU_THIS_PTR tr.cache.valid    = 1; /* valid */
+  BX_CPU_THIS_PTR tr.cache.valid    = SegValidCache; /* valid */
   BX_CPU_THIS_PTR tr.cache.p        = 1; /* present */
   BX_CPU_THIS_PTR tr.cache.dpl      = 0; /* field not used */
   BX_CPU_THIS_PTR tr.cache.segment  = 0; /* system segment */
@@ -738,9 +802,13 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR xcr0_suppmask = 0x3;
 #if BX_SUPPORT_AVX
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX))
-    BX_CPU_THIS_PTR xcr0_suppmask |= BX_XCR0_AVX_MASK;
-#endif
+    BX_CPU_THIS_PTR xcr0_suppmask |= BX_XCR0_YMM_MASK;
+#if BX_SUPPORT_EVEX
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_AVX512))
+    BX_CPU_THIS_PTR xcr0_suppmask |= BX_XCR0_OPMASK_MASK | BX_XCR0_ZMM_HI256_MASK | BX_XCR0_HI_ZMM_MASK;
 #endif
+#endif // BX_SUPPORT_AVX
+#endif // BX_CPU_LEVEL >= 6
 
 /* initialise MSR registers to defaults */
 #if BX_CPU_LEVEL >= 5
@@ -751,21 +819,21 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR msr.apicbase |= 0x900;
   BX_CPU_THIS_PTR lapic.set_base(BX_CPU_THIS_PTR msr.apicbase);
 #if BX_CPU_LEVEL >= 6
-  if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_XAPIC_EXT))
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_XAPIC_EXT))
     BX_CPU_THIS_PTR lapic.enable_xapic_extensions();
 #endif
 #endif
 
   BX_CPU_THIS_PTR efer.set32(0);
   BX_CPU_THIS_PTR efer_suppmask = 0;
-  if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_NX))
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_NX))
     BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_NXE_MASK;
   if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SYSCALL_SYSRET_LEGACY))
     BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_SCE_MASK;
 #if BX_SUPPORT_X86_64
-  if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_LONG_MODE)) {
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_LONG_MODE)) {
     BX_CPU_THIS_PTR efer_suppmask |= (BX_EFER_SCE_MASK | BX_EFER_LME_MASK | BX_EFER_LMA_MASK);
-    if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_FFXSR))
+    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_FFXSR))
       BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_FFXSR_MASK;
     if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SVM))
       BX_CPU_THIS_PTR efer_suppmask |= BX_EFER_SVME_MASK;
@@ -774,7 +842,7 @@ void BX_CPU_C::reset(unsigned source)
 
   BX_CPU_THIS_PTR msr.star = 0;
 #if BX_SUPPORT_X86_64
-  if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_LONG_MODE)) {
+  if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_LONG_MODE)) {
     BX_CPU_THIS_PTR msr.lstar = 0;
     BX_CPU_THIS_PTR msr.cstar = 0;
     BX_CPU_THIS_PTR msr.fmask = 0x00020200;
@@ -803,14 +871,13 @@ void BX_CPU_C::reset(unsigned source)
     for (n=0; n<16; n++)
       BX_CPU_THIS_PTR msr.mtrrphys[n] = 0;
 
-    BX_CPU_THIS_PTR msr.mtrrfix64k_00000 = 0; // all fix range MTRRs undefined according to manual
-    BX_CPU_THIS_PTR msr.mtrrfix16k[0] = 0;
-    BX_CPU_THIS_PTR msr.mtrrfix16k[1] = 0;
-
+    BX_CPU_THIS_PTR msr.mtrrfix64k = (Bit64u) 0; // all fix range MTRRs undefined according to manual
+    BX_CPU_THIS_PTR msr.mtrrfix16k[0] = (Bit64u) 0;
+    BX_CPU_THIS_PTR msr.mtrrfix16k[1] = (Bit64u) 0;
     for (n=0; n<8; n++)
-      BX_CPU_THIS_PTR msr.mtrrfix4k[n] = 0;
+      BX_CPU_THIS_PTR msr.mtrrfix4k[n] = (Bit64u) 0;
 
-    BX_CPU_THIS_PTR msr.pat = BX_CONST64(0x0007040600070406);
+    BX_CPU_THIS_PTR msr.pat = (Bit64u) BX_CONST64(0x0007040600070406);
     BX_CPU_THIS_PTR msr.mtrr_deftype = 0;
   }
 #endif
@@ -837,6 +904,9 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR espPageBias = 0;
   BX_CPU_THIS_PTR espPageWindowSize = 0;
   BX_CPU_THIS_PTR espHostPtr = NULL;
+#if BX_SUPPORT_MEMTYPE
+  BX_CPU_THIS_PTR espPageMemtype = BX_MEMTYPE_UC;
+#endif
 
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR stop_reason = STOP_NO_REASON;
@@ -863,17 +933,24 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR avx_ok = 0;
 #endif
 
+#if BX_SUPPORT_EVEX
+  BX_CPU_THIS_PTR opmask_ok = BX_CPU_THIS_PTR evex_ok = 0;
+
+  for (n=0; n<8; n++)
+    BX_WRITE_OPMASK(n, 0);
+#endif
+
   // Reset XMM state - unchanged on #INIT
   if (source == BX_RESET_HARDWARE) {
-    static BxPackedXmmRegister xmmnil; /* compiler will clear the variable */
-    for(n=0; n<BX_XMM_REGISTERS; n++)
-      BX_WRITE_XMM_REG_CLEAR_HIGH(n, xmmnil);
+    for(n=0; n<BX_XMM_REGISTERS; n++) {
+      BX_CLEAR_AVX_REG(n);
+    }
 
     BX_CPU_THIS_PTR mxcsr.mxcsr = MXCSR_RESET;
     BX_CPU_THIS_PTR mxcsr_mask = 0x0000ffbf;
     if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_SSE2))
       BX_CPU_THIS_PTR mxcsr_mask |= MXCSR_DAZ;
-    if (BX_CPUID_SUPPORT_CPU_EXTENSION(BX_CPU_MISALIGNED_SSE))
+    if (BX_CPUID_SUPPORT_ISA_EXTENSION(BX_ISA_MISALIGNED_SSE))
       BX_CPU_THIS_PTR mxcsr_mask |= MXCSR_MISALIGNED_EXCEPTION_MASK;
   }
 #endif
@@ -882,10 +959,12 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR in_vmx = BX_CPU_THIS_PTR in_vmx_guest = 0;
   BX_CPU_THIS_PTR in_smm_vmx = BX_CPU_THIS_PTR in_smm_vmx_guest = 0;
   BX_CPU_THIS_PTR vmcsptr = BX_CPU_THIS_PTR vmxonptr = BX_INVALID_VMCSPTR;
-  BX_CPU_THIS_PTR vmcshostptr = 0;
-  /* enable VMX, should be done in BIOS instead */
-  BX_CPU_THIS_PTR msr.ia32_feature_ctrl =
-    /*BX_IA32_FEATURE_CONTROL_LOCK_BIT | */BX_IA32_FEATURE_CONTROL_VMX_ENABLE_BIT;
+  set_VMCSPTR(BX_CPU_THIS_PTR vmcsptr);
+  if (source == BX_RESET_HARDWARE) {
+    /* enable VMX, should be done in BIOS instead */
+    BX_CPU_THIS_PTR msr.ia32_feature_ctrl =
+      /*BX_IA32_FEATURE_CONTROL_LOCK_BIT | */BX_IA32_FEATURE_CONTROL_VMX_ENABLE_BIT;
+  }
 #endif
 
 #if BX_SUPPORT_SVM
@@ -893,6 +972,9 @@ void BX_CPU_C::reset(unsigned source)
   BX_CPU_THIS_PTR svm_gif = 1;
   BX_CPU_THIS_PTR vmcbptr = 0;
   BX_CPU_THIS_PTR vmcbhostptr = 0;
+#if BX_SUPPORT_MEMTYPE
+  BX_CPU_THIS_PTR vmcb_memtype = BX_MEMTYPE_UC;
+#endif
 #endif
 
 #if BX_SUPPORT_VMX || BX_SUPPORT_SVM
@@ -1005,6 +1087,12 @@ void BX_CPU_C::sanity_checks(void)
   if (sizeof(Bit64u) != 8  ||  sizeof(Bit64s) != 8)
     BX_PANIC(("data type Bit64u or Bit64u is not of length 8 bytes!"));
 
+  if (sizeof(void*) != sizeof(bx_ptr_equiv_t))
+    BX_PANIC(("data type bx_ptr_equiv_t is not equivalent to 'void*' pointer"));
+
+  if (sizeof(int) < 4)
+    BX_PANIC(("Bochs assumes that 'int' type is at least 4 bytes wide!"));
+
   BX_DEBUG(("#(%u)all sanity checks passed!", BX_CPU_ID));
 }
 
