@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical16.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: logical16.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,7 +33,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EwGwM(bxInstruction_c *i)
   op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->src());
   op1_16 ^= op2_16;
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
 
@@ -78,7 +78,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EwIwM(bxInstruction_c *i)
 
   op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16 ^= i->Iw();
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
 
@@ -104,7 +104,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EwIwM(bxInstruction_c *i)
 
   op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16 |= i->Iw();
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
 
@@ -128,7 +128,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EwM(bxInstruction_c *i)
 
   Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16 = ~op1_16;
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   BX_NEXT_INSTR(i);
 }
@@ -151,7 +151,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EwGwM(bxInstruction_c *i)
   op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->src());
   op1_16 |= op2_16;
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
 
@@ -197,7 +197,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EwGwM(bxInstruction_c *i)
   op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->src());
   op1_16 &= op2_16;
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
 
@@ -242,7 +242,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EwIwM(bxInstruction_c *i)
 
   op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16 &= i->Iw();
-  write_RMW_virtual_word(op1_16);
+  write_RMW_linear_word(op1_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
 
