@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical32.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: logical32.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -33,7 +33,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdGdM(bxInstruction_c *i)
   op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->src());
   op1_32 ^= op2_32;
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 
@@ -78,7 +78,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdIdM(bxInstruction_c *i)
 
   op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 ^= i->Id();
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 
@@ -104,7 +104,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdIdM(bxInstruction_c *i)
 
   op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 |= i->Id();
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 
@@ -128,7 +128,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EdM(bxInstruction_c *i)
 
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 = ~op1_32;
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   BX_NEXT_INSTR(i);
 }
@@ -151,7 +151,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdGdM(bxInstruction_c *i)
   op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->src());
   op1_32 |= op2_32;
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 
@@ -197,7 +197,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdGdM(bxInstruction_c *i)
   op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->src());
   op1_32 &= op2_32;
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 
@@ -242,7 +242,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdIdM(bxInstruction_c *i)
 
   op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 &= i->Id();
-  write_RMW_virtual_dword(op1_32);
+  write_RMW_linear_dword(op1_32);
 
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 
