@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical64.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: logical64.cc 12619 2015-01-26 20:01:25Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -32,10 +32,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EqGqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op2_64 = BX_READ_64BIT_REG(i->src());
   op1_64 ^= op2_64;
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 
@@ -64,7 +64,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GqEqM(bxInstruction_c *i)
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op1_64 = BX_READ_64BIT_REG(i->dst());
-  op2_64 = read_virtual_qword_64(i->seg(), eaddr);
+  op2_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op1_64 ^= op2_64;
 
   BX_WRITE_64BIT_REG(i->dst(), op1_64);
@@ -80,9 +80,9 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EqIdM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op1_64 ^= op2_64;
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 
@@ -108,9 +108,9 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EqIdM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op1_64 |= op2_64;
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 
@@ -134,9 +134,9 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EqM(bxInstruction_c *i)
 {
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op1_64 = ~op1_64;
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   BX_NEXT_INSTR(i);
 }
@@ -156,10 +156,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EqGqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->src());
   op1_64 |= op2_64;
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 
@@ -188,7 +188,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GqEqM(bxInstruction_c *i)
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op1_64 = BX_READ_64BIT_REG(i->dst());
-  op2_64 = read_virtual_qword_64(i->seg(), eaddr);
+  op2_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op1_64 |= op2_64;
 
   BX_WRITE_64BIT_REG(i->dst(), op1_64);
@@ -204,10 +204,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EqGqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op2_64 = BX_READ_64BIT_REG(i->src());
   op1_64 &= op2_64;
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 
@@ -236,7 +236,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GqEqM(bxInstruction_c *i)
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op1_64 = BX_READ_64BIT_REG(i->dst());
-  op2_64 = read_virtual_qword_64(i->seg(), eaddr);
+  op2_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op1_64 &= op2_64;
 
   BX_WRITE_64BIT_REG(i->dst(), op1_64);
@@ -252,9 +252,9 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EqIdM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op1_64 &= op2_64;
-  write_RMW_virtual_qword(op1_64);
+  write_RMW_linear_qword(op1_64);
 
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 
@@ -293,7 +293,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EqGqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op2_64 = BX_READ_64BIT_REG(i->src());
   op1_64 &= op2_64;
 
@@ -321,7 +321,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EqIdM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
   op2_64 = (Bit32s) i->Id();
   op1_64 &= op2_64;
 
