@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: mmx.cc 11330 2012-08-09 13:11:25Z sshwarts $
+// $Id: mmx.cc 12671 2015-02-23 21:17:33Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2002-2012 Stanislav Shwartsman
+//   Copyright (c) 2002-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -32,7 +32,7 @@ void BX_CPU_C::print_state_MMX(void)
 {
   for(int i=0;i<8;i++) {
     BxPackedMmxRegister mm = BX_READ_MMX_REG(i);
-    BX_DEBUG(("MM%d: %08x%08x\n", i, MMXUD1(mm), MMXUD0(mm)));
+    BX_DEBUG(("MM%d: %08x%08x", i, MMXUD1(mm), MMXUD0(mm)));
   }
 }
 
@@ -61,11 +61,11 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSHUFB_PqQq(bxInstruction_c *i)
 
   for(unsigned j=0; j<8; j++)
   {
-    unsigned mask = op2.mmxubyte(j);
+    unsigned mask = op2.ubyte(j);
     if (mask & 0x80)
-      result.mmxubyte(j) = 0;
+      result.ubyte(j) = 0;
     else
-      result.mmxubyte(j) = op1.mmxubyte(mask & 0x7);
+      result.ubyte(j) = op1.ubyte(mask & 0x7);
   }
 
   BX_WRITE_MMX_REG(i->dst(), result);
@@ -179,10 +179,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMADDUBSW_PqQq(bxInstruction_c *i)
 
   for(unsigned j=0; j<4; j++)
   {
-    Bit32s temp = Bit32s(op1.mmxubyte(j*2+0))*Bit32s(op2.mmxsbyte(j*2)) +
-                  Bit32s(op1.mmxubyte(j*2+1))*Bit32s(op2.mmxsbyte(j*2+1));
+    Bit32s temp = Bit32s(op1.ubyte(j*2+0))*Bit32s(op2.sbyte(j*2)) +
+                  Bit32s(op1.ubyte(j*2+1))*Bit32s(op2.sbyte(j*2+1));
 
-    op1.mmx16s(j) = SaturateDwordSToWordS(temp);
+    op1.s16(j) = SaturateDwordSToWordS(temp);
   }
 
   BX_WRITE_MMX_REG(i->dst(), op1);
@@ -295,8 +295,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSIGNB_PqQq(bxInstruction_c *i)
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
   for(unsigned j=0; j<8; j++) {
-    int sign = (op2.mmxsbyte(j) > 0) - (op2.mmxsbyte(j) < 0);
-    op1.mmxsbyte(j) *= sign;
+    int sign = (op2.sbyte(j) > 0) - (op2.sbyte(j) < 0);
+    op1.sbyte(j) *= sign;
   }
 
   BX_WRITE_MMX_REG(i->dst(), op1);
@@ -324,8 +324,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSIGNW_PqQq(bxInstruction_c *i)
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
   for(unsigned j=0; j<4; j++) {
-    int sign = (op2.mmx16s(j) > 0) - (op2.mmx16s(j) < 0);
-    op1.mmx16s(j) *= sign;
+    int sign = (op2.s16(j) > 0) - (op2.s16(j) < 0);
+    op1.s16(j) *= sign;
   }
 
   BX_WRITE_MMX_REG(i->dst(), op1);
@@ -909,8 +909,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_PqEdR(bxInstruction_c *i)
   BX_CPU_THIS_PTR prepareMMX();
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
-  BxPackedMmxRegister op;
-  MMXUQ(op) = (Bit64u) BX_READ_32BIT_REG(i->src());
+  BxPackedMmxRegister op = (Bit64u) BX_READ_32BIT_REG(i->src());
   BX_WRITE_MMX_REG(i->dst(), op);
 #endif
 
@@ -922,10 +921,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_PqEdM(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
 
-  BxPackedMmxRegister op;
-
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-  MMXUQ(op) = (Bit64u) read_virtual_dword(i->seg(), eaddr);
+  BxPackedMmxRegister op = (Bit64u) read_virtual_dword(i->seg(), eaddr);
 
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
@@ -942,9 +939,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_PqEqR(bxInstruction_c *i)
   BX_CPU_THIS_PTR prepareMMX();
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
-  BxPackedMmxRegister op;
-  MMXUQ(op) = BX_READ_64BIT_REG(i->src());
-
+  BxPackedMmxRegister op = BX_READ_64BIT_REG(i->src());
   BX_WRITE_MMX_REG(i->dst(), op);
 
   BX_NEXT_INSTR(i);
@@ -969,10 +964,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVQ_PqQqM(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
 
-  BxPackedMmxRegister op;
-
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-  MMXUQ(op) = read_virtual_qword(i->seg(), eaddr);
+  BxPackedMmxRegister op = read_virtual_qword(i->seg(), eaddr);
 
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
@@ -1003,10 +996,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSHUFW_PqQqIb(bxInstruction_c *i)
 
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
-  MMXUW0(result) = op.mmx16u((order)    & 0x3);
-  MMXUW1(result) = op.mmx16u((order>>2) & 0x3);
-  MMXUW2(result) = op.mmx16u((order>>4) & 0x3);
-  MMXUW3(result) = op.mmx16u((order>>6) & 0x3);
+  MMXUW0(result) = op.u16((order)    & 0x3);
+  MMXUW1(result) = op.u16((order>>2) & 0x3);
+  MMXUW2(result) = op.u16((order>>4) & 0x3);
+  MMXUW3(result) = op.u16((order>>6) & 0x3);
 
   BX_WRITE_MMX_REG(i->dst(), result);
 #endif
@@ -1122,7 +1115,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::EMMS(bxInstruction_c *i)
 }
 
 /* 0F 7E */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_EdPdR(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_EdPqR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -1135,7 +1128,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_EdPdR(bxInstruction_c *i)
   BX_NEXT_INSTR(i);
 }
 
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_EdPdM(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVD_EdPqM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -1207,7 +1200,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PINSRW_PqEwIb(bxInstruction_c *i)
 
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
-  op1.mmx16u(i->Ib() & 0x3) = op2;
+  op1.u16(i->Ib() & 0x3) = op2;
 
   BX_WRITE_MMX_REG(i->dst(), op1);
 #endif
@@ -1216,14 +1209,14 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PINSRW_PqEwIb(bxInstruction_c *i)
 }
 
 /* 0F C5 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRW_GdPqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRW_GdNqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
   BX_CPU_THIS_PTR prepareFPU2MMX(); /* FPU2MMX transition */
 
   BxPackedMmxRegister op = BX_READ_MMX_REG(i->src());
-  Bit32u result = (Bit32u) op.mmx16u(i->Ib() & 0x3);
+  Bit32u result = (Bit32u) op.u16(i->Ib() & 0x3);
 
   BX_WRITE_32BIT_REGZ(i->dst(), result);
 #endif
@@ -1401,7 +1394,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMULLW_PqQq(bxInstruction_c *i)
 }
 
 /* 0F D7 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVMSKB_GdPRq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PMOVMSKB_GdNq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -1748,10 +1741,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAW_PqQq(bxInstruction_c *i)
   }
 
   if(MMXUQ(op2) > 15) {
-    MMXUW0(op1) = (MMXUW0(op1) & 0x8000) ? 0xffff : 0;
-    MMXUW1(op1) = (MMXUW1(op1) & 0x8000) ? 0xffff : 0;
-    MMXUW2(op1) = (MMXUW2(op1) & 0x8000) ? 0xffff : 0;
-    MMXUW3(op1) = (MMXUW3(op1) & 0x8000) ? 0xffff : 0;
+    MMXUW0(op1) = (MMXSW0(op1) < 0) ? 0xffff : 0;
+    MMXUW1(op1) = (MMXSW1(op1) < 0) ? 0xffff : 0;
+    MMXUW2(op1) = (MMXSW2(op1) < 0) ? 0xffff : 0;
+    MMXUW3(op1) = (MMXSW3(op1) < 0) ? 0xffff : 0;
   }
   else {
     Bit8u shift = MMXUB0(op2);
@@ -1793,8 +1786,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAD_PqQq(bxInstruction_c *i)
   }
 
   if(MMXUQ(op2) > 31) {
-    MMXUD0(op1) = (MMXUD0(op1) & 0x80000000) ? 0xffffffff : 0;
-    MMXUD1(op1) = (MMXUD1(op1) & 0x80000000) ? 0xffffffff : 0;
+    MMXUD0(op1) = (MMXSD0(op1) < 0) ? 0xffffffff : 0;
+    MMXUD1(op1) = (MMXSD1(op1) < 0) ? 0xffffffff : 0;
   }
   else {
     Bit8u shift = MMXUB0(op2);
@@ -2374,7 +2367,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSADBW_PqQq(bxInstruction_c *i)
 }
 
 /* 0F F7 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVQ_PqPRq(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVQ_PqNq(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2400,7 +2393,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVQ_PqPRq(bxInstruction_c *i)
   if(MMXUB6(mask) & 0x80) MMXUB6(tmp) = MMXUB6(op);
   if(MMXUB7(mask) & 0x80) MMXUB7(tmp) = MMXUB7(op);
 
-  write_RMW_virtual_qword(MMXUQ(tmp));
+  write_RMW_linear_qword(MMXUQ(tmp));
 #endif
 
   BX_NEXT_INSTR(i);
@@ -2625,7 +2618,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PADDD_PqQq(bxInstruction_c *i)
 }
 
 /* 0F 71 GrpA 010 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLW_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLW_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2650,7 +2643,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLW_PqIb(bxInstruction_c *i)
 }
 
 /* 0F 71 GrpA 100 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAW_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAW_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2664,10 +2657,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAW_PqIb(bxInstruction_c *i)
   }
 
   if(shift > 15) {
-    MMXUW0(op) = (MMXUW0(op) & 0x8000) ? 0xffff : 0;
-    MMXUW1(op) = (MMXUW1(op) & 0x8000) ? 0xffff : 0;
-    MMXUW2(op) = (MMXUW2(op) & 0x8000) ? 0xffff : 0;
-    MMXUW3(op) = (MMXUW3(op) & 0x8000) ? 0xffff : 0;
+    MMXUW0(op) = (MMXSW0(op) < 0) ? 0xffff : 0;
+    MMXUW1(op) = (MMXSW1(op) < 0) ? 0xffff : 0;
+    MMXUW2(op) = (MMXSW2(op) < 0) ? 0xffff : 0;
+    MMXUW3(op) = (MMXSW3(op) < 0) ? 0xffff : 0;
   }
   else {
     MMXUW0(op) = (Bit16u)(MMXSW0(op) >> shift);
@@ -2683,7 +2676,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAW_PqIb(bxInstruction_c *i)
 }
 
 /* 0F 71 GrpA 110 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLW_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLW_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2708,7 +2701,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLW_PqIb(bxInstruction_c *i)
 }
 
 /* 0F 72 GrpA 010 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLD_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLD_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2731,7 +2724,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLD_PqIb(bxInstruction_c *i)
 }
 
 /* 0F 72 GrpA 100 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAD_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAD_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2745,8 +2738,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAD_PqIb(bxInstruction_c *i)
   }
 
   if(shift > 31) {
-    MMXUD0(op) = (MMXUD0(op) & 0x80000000) ? 0xffffffff : 0;
-    MMXUD1(op) = (MMXUD1(op) & 0x80000000) ? 0xffffffff : 0;
+    MMXUD0(op) = (MMXSD0(op) < 0) ? 0xffffffff : 0;
+    MMXUD1(op) = (MMXSD1(op) < 0) ? 0xffffffff : 0;
   }
   else {
     MMXUD0(op) = (Bit32u)(MMXSD0(op) >> shift);
@@ -2760,7 +2753,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRAD_PqIb(bxInstruction_c *i)
 }
 
 /* 0F 72 GrpA 110 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLD_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLD_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2783,7 +2776,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLD_PqIb(bxInstruction_c *i)
 }
 
 /* 0F 73 GrpA 010 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLQ_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLQ_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
@@ -2806,7 +2799,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSRLQ_PqIb(bxInstruction_c *i)
 }
 
 /* 0F 73 GrpA 110 */
-BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLQ_PqIb(bxInstruction_c *i)
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSLLQ_NqIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 5
   BX_CPU_THIS_PTR prepareMMX();
