@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: msr.cc 11680 2013-04-17 19:59:56Z sshwarts $
+// $Id: msr.cc 12671 2015-02-23 21:17:33Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2008-2012 Stanislav Shwartsman
+//   Copyright (c) 2008-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -32,7 +32,7 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
   Bit64u val64 = 0;
 
 #if BX_CPU_LEVEL >= 6
-  if (bx_cpuid_support_x2apic()) {
+  if (is_cpu_extension_supported(BX_ISA_X2APIC)) {
     if (index >= 0x800 && index <= 0xBFF) {
       if (BX_CPU_THIS_PTR msr.apicbase & 0x400)  // X2APIC mode
         return BX_CPU_THIS_PTR lapic.read_x2apic(index, msr);
@@ -46,24 +46,24 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_SYSENTER_CS:
-      if (! bx_cpuid_support_sep()) {
-        BX_ERROR(("RDMSR MSR_SYSENTER_CS: SYSENTER/SYSEXIT feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SYSENTER_SYSEXIT)) {
+        BX_ERROR(("RDMSR MSR_SYSENTER_CS: SYSENTER/SYSEXIT feature not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.sysenter_cs_msr;
       break;
 
     case BX_MSR_SYSENTER_ESP:
-      if (! bx_cpuid_support_sep()) {
-        BX_ERROR(("RDMSR MSR_SYSENTER_ESP: SYSENTER/SYSEXIT feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SYSENTER_SYSEXIT)) {
+        BX_ERROR(("RDMSR MSR_SYSENTER_ESP: SYSENTER/SYSEXIT feature not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.sysenter_esp_msr;
       break;
 
     case BX_MSR_SYSENTER_EIP:
-      if (! bx_cpuid_support_sep()) {
-        BX_ERROR(("RDMSR MSR_SYSENTER_EIP: SYSENTER/SYSEXIT feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SYSENTER_SYSEXIT)) {
+        BX_ERROR(("RDMSR MSR_SYSENTER_EIP: SYSENTER/SYSEXIT feature not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.sysenter_eip_msr;
@@ -72,11 +72,11 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_MTRRCAP:   // read only MSR
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("RDMSR MSR_MTRRCAP: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("RDMSR MSR_MTRRCAP: MTRR is not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = BX_CONST64(0x0000000000000508);
+      val64 = BX_CONST64(0x0000000000000500) | BX_NUM_VARIABLE_RANGE_MTRRS;
       break;
 
     case BX_MSR_MTRRPHYSBASE0:
@@ -95,27 +95,27 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
     case BX_MSR_MTRRPHYSMASK6:
     case BX_MSR_MTRRPHYSBASE7:
     case BX_MSR_MTRRPHYSMASK7:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("RDMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.mtrrphys[index - BX_MSR_MTRRPHYSBASE0];
       break;
 
     case BX_MSR_MTRRFIX64K_00000:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("RDMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = BX_CPU_THIS_PTR msr.mtrrfix64k_00000;
+      val64 = BX_CPU_THIS_PTR msr.mtrrfix64k.u64;
       break;
     case BX_MSR_MTRRFIX16K_80000:
     case BX_MSR_MTRRFIX16K_A0000:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("RDMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = BX_CPU_THIS_PTR msr.mtrrfix16k[index - BX_MSR_MTRRFIX16K_80000];
+      val64 = BX_CPU_THIS_PTR msr.mtrrfix16k[index - BX_MSR_MTRRFIX16K_80000].u64;
       break;
 
     case BX_MSR_MTRRFIX4K_C0000:
@@ -126,27 +126,27 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
     case BX_MSR_MTRRFIX4K_E8000:
     case BX_MSR_MTRRFIX4K_F0000:
     case BX_MSR_MTRRFIX4K_F8000:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("RDMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("RDMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = BX_CPU_THIS_PTR msr.mtrrfix4k[index - BX_MSR_MTRRFIX4K_C0000];
+      val64 = BX_CPU_THIS_PTR msr.mtrrfix4k[index - BX_MSR_MTRRFIX4K_C0000].u64;
       break;
 
     case BX_MSR_MTRR_DEFTYPE:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("RDMSR MSR_MTRR_DEFTYPE: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("RDMSR MSR_MTRR_DEFTYPE: MTRR is not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.mtrr_deftype;
       break;
 
     case BX_MSR_PAT:
-      if (! bx_cpuid_support_pat()) {
-        BX_ERROR(("RDMSR BX_MSR_PAT: PAT is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_PAT)) {
+        BX_ERROR(("RDMSR BX_MSR_PAT: PAT is not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = BX_CPU_THIS_PTR msr.pat;
+      val64 = BX_CPU_THIS_PTR msr.pat.u64;
       break;
 #endif
 
@@ -163,8 +163,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_TSC_DEADLINE:
-      if (! bx_cpuid_support_tsc_deadline()) {
-        BX_ERROR(("RDMSR BX_MSR_TSC_DEADLINE: TSC-Deadline not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_TSC_DEADLINE)) {
+        BX_ERROR(("RDMSR BX_MSR_TSC_DEADLINE: TSC-Deadline not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR lapic.get_tsc_deadline();
@@ -257,8 +257,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 
 #if BX_SUPPORT_SVM
     case BX_SVM_HSAVE_PA_MSR:
-      if (! bx_cpuid_support_svm()) {
-        BX_ERROR(("RDMSR SVM_HSAVE_PA_MSR: SVM support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SVM)) {
+        BX_ERROR(("RDMSR SVM_HSAVE_PA_MSR: SVM support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = BX_CPU_THIS_PTR msr.svm_hsave_pa;
@@ -267,7 +267,7 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 
     case BX_MSR_STAR:
       if ((BX_CPU_THIS_PTR efer_suppmask & BX_EFER_SCE_MASK) == 0) {
-        BX_ERROR(("RDMSR MSR_STAR: SYSCALL/SYSRET support not enabled !"));
+        BX_ERROR(("RDMSR MSR_STAR: SYSCALL/SYSRET support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = MSR_STAR;
@@ -275,59 +275,59 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::rdmsr(Bit32u index, Bit64u *msr)
 
 #if BX_SUPPORT_X86_64
     case BX_MSR_LSTAR:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("RDMSR MSR_LSTAR: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("RDMSR MSR_LSTAR: long mode support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = MSR_LSTAR;
+      val64 = BX_CPU_THIS_PTR msr.lstar;
       break;
 
     case BX_MSR_CSTAR:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("RDMSR MSR_CSTAR: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("RDMSR MSR_CSTAR: long mode support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = MSR_CSTAR;
+      val64 = BX_CPU_THIS_PTR msr.cstar;
       break;
 
     case BX_MSR_FMASK:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("RDMSR MSR_FMASK: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("RDMSR MSR_FMASK: long mode support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = MSR_FMASK;
+      val64 = BX_CPU_THIS_PTR msr.fmask;
       break;
 
     case BX_MSR_FSBASE:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("RDMSR MSR_FSBASE: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("RDMSR MSR_FSBASE: long mode support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = MSR_FSBASE;
       break;
 
     case BX_MSR_GSBASE:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("RDMSR MSR_GSBASE: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("RDMSR MSR_GSBASE: long mode support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = MSR_GSBASE;
       break;
 
     case BX_MSR_KERNELGSBASE:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("RDMSR MSR_KERNELGSBASE: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("RDMSR MSR_KERNELGSBASE: long mode support not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
       val64 = MSR_KERNELGSBASE;
       break;
 
     case BX_MSR_TSC_AUX:
-      if (! bx_cpuid_support_rdtscp()) {
-        BX_ERROR(("RDMSR MSR_TSC_AUX: RTDSCP feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_RDTSCP)) {
+        BX_ERROR(("RDMSR MSR_TSC_AUX: RTDSCP feature not enabled in the cpu model"));
         return handle_unknown_rdmsr(index, msr);
       }
-      val64 = MSR_TSC_AUX;   // 32 bit MSR
+      val64 = BX_CPU_THIS_PTR msr.tsc_aux;   // 32 bit MSR
       break;
 #endif
 
@@ -441,30 +441,22 @@ BX_CPP_INLINE bx_bool isMemTypeValidPAT(unsigned memtype)
   return (memtype == 0x07) /* UC- */ || isMemTypeValidMTRR(memtype);
 }
 
-bx_bool isValidMSR_PAT(Bit64u pat_msr)
+bx_bool isValidMSR_PAT(Bit64u pat_val)
 {
-  if (! isMemTypeValidPAT(pat_msr & 0xFF) ||
-      ! isMemTypeValidPAT((pat_msr >>  8) & 0xFF) || 
-      ! isMemTypeValidPAT((pat_msr >> 16) & 0xFF) || 
-      ! isMemTypeValidPAT((pat_msr >> 24) & 0xFF) ||
-      ! isMemTypeValidPAT((pat_msr >> 32) & 0xFF) ||
-      ! isMemTypeValidPAT((pat_msr >> 40) & 0xFF) || 
-      ! isMemTypeValidPAT((pat_msr >> 48) & 0xFF) || 
-      ! isMemTypeValidPAT(pat_msr >> 56)) return BX_FALSE;
+  // use packed register as 64-bit value with convinient accessors
+  BxPackedRegister pat_msr = pat_val;
+  for (unsigned i=0; i<8; i++)
+    if (! isMemTypeValidPAT(pat_msr.ubyte(i))) return BX_FALSE;
 
   return BX_TRUE;
 }
 
-bx_bool isValidMSR_FixedMTRR(Bit64u fixed_mtrr_msr)
+bx_bool isValidMSR_FixedMTRR(Bit64u fixed_mtrr_val)
 {
-  if (! isMemTypeValidMTRR(fixed_mtrr_msr & 0xFF) ||
-      ! isMemTypeValidMTRR((fixed_mtrr_msr >>  8) & 0xFF) || 
-      ! isMemTypeValidMTRR((fixed_mtrr_msr >> 16) & 0xFF) || 
-      ! isMemTypeValidMTRR((fixed_mtrr_msr >> 24) & 0xFF) ||
-      ! isMemTypeValidMTRR((fixed_mtrr_msr >> 32) & 0xFF) ||
-      ! isMemTypeValidMTRR((fixed_mtrr_msr >> 40) & 0xFF) || 
-      ! isMemTypeValidMTRR((fixed_mtrr_msr >> 48) & 0xFF) || 
-      ! isMemTypeValidMTRR(fixed_mtrr_msr >> 56)) return BX_FALSE;
+  // use packed register as 64-bit value with convinient accessors
+  BxPackedRegister fixed_mtrr_msr = fixed_mtrr_val;
+  for (unsigned i=0; i<8; i++)
+    if (! isMemTypeValidMTRR(fixed_mtrr_msr.ubyte(i))) return BX_FALSE;
 
   return BX_TRUE;
 }
@@ -481,7 +473,7 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
   BX_DEBUG(("WRMSR: write %08x:%08x to MSR %x", val32_hi, val32_lo, index));
 
 #if BX_CPU_LEVEL >= 6
-  if (bx_cpuid_support_x2apic()) {
+  if (is_cpu_extension_supported(BX_ISA_X2APIC)) {
     if (index >= 0x800 && index <= 0xBFF) {
       if (BX_CPU_THIS_PTR msr.apicbase & 0x400)  // X2APIC mode
         return BX_CPU_THIS_PTR lapic.write_x2apic(index, val32_hi, val32_lo);
@@ -493,18 +485,31 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
   switch(index) {
 
+#if BX_SUPPORT_PERFMON
+    case BX_MSR_PERFEVTSEL0:
+    case BX_MSR_PERFEVTSEL1:
+    case BX_MSR_PERFEVTSEL2:
+    case BX_MSR_PERFEVTSEL3:
+    case BX_MSR_PERFEVTSEL4:
+    case BX_MSR_PERFEVTSEL5:
+    case BX_MSR_PERFEVTSEL6:
+    case BX_MSR_PERFEVTSEL7:
+      BX_INFO(("WRMSR: write into IA32_PERFEVTSEL%d MSR: %08x:%08x", index - BX_MSR_PERFEVTSEL0, val32_hi, val32_lo));
+      return handle_unknown_wrmsr(index, val_64);
+#endif
+
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_SYSENTER_CS:
-      if (! bx_cpuid_support_sep()) {
-        BX_ERROR(("WRMSR MSR_SYSENTER_CS: SYSENTER/SYSEXIT feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SYSENTER_SYSEXIT)) {
+        BX_ERROR(("WRMSR MSR_SYSENTER_CS: SYSENTER/SYSEXIT feature not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       BX_CPU_THIS_PTR msr.sysenter_cs_msr = val32_lo;
       break;
 
     case BX_MSR_SYSENTER_ESP:
-      if (! bx_cpuid_support_sep()) {
-        BX_ERROR(("WRMSR MSR_SYSENTER_ESP: SYSENTER/SYSEXIT feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SYSENTER_SYSEXIT)) {
+        BX_ERROR(("WRMSR MSR_SYSENTER_ESP: SYSENTER/SYSEXIT feature not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
 #if BX_SUPPORT_X86_64
@@ -517,8 +522,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_SYSENTER_EIP:
-      if (! bx_cpuid_support_sep()) {
-        BX_ERROR(("WRMSR MSR_SYSENTER_EIP: SYSENTER/SYSEXIT feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SYSENTER_SYSEXIT)) {
+        BX_ERROR(("WRMSR MSR_SYSENTER_EIP: SYSENTER/SYSEXIT feature not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
 #if BX_SUPPORT_X86_64
@@ -533,8 +538,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_MTRRCAP:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("WRMSR MSR_MTRRCAP: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("WRMSR MSR_MTRRCAP: MTRR is not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       BX_ERROR(("WRMSR: MTRRCAP is read only MSR"));
@@ -548,8 +553,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
     case BX_MSR_MTRRPHYSBASE5:
     case BX_MSR_MTRRPHYSBASE6:
     case BX_MSR_MTRRPHYSBASE7:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("WRMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsValidPhyAddr(val_64)) {
@@ -571,8 +576,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
     case BX_MSR_MTRRPHYSMASK5:
     case BX_MSR_MTRRPHYSMASK6:
     case BX_MSR_MTRRPHYSMASK7:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("WRMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsValidPhyAddr(val_64)) {
@@ -588,21 +593,21 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_MTRRFIX64K_00000:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("WRMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! isValidMSR_FixedMTRR(val_64)) {
         BX_ERROR(("WRMSR: attempt to write invalid Memory Type to MSR_MTRRFIX64K_00000 !"));
         return 0;
       }
-      BX_CPU_THIS_PTR msr.mtrrfix64k_00000 = val_64;
+      BX_CPU_THIS_PTR msr.mtrrfix64k = val_64;
       break;
 
     case BX_MSR_MTRRFIX16K_80000:
     case BX_MSR_MTRRFIX16K_A0000:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("WRMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! isValidMSR_FixedMTRR(val_64)) {
@@ -620,8 +625,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
     case BX_MSR_MTRRFIX4K_E8000:
     case BX_MSR_MTRRFIX4K_F0000:
     case BX_MSR_MTRRFIX4K_F8000:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("WRMSR: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("WRMSR: MTRR is not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! isValidMSR_FixedMTRR(val_64)) {
@@ -632,8 +637,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_MTRR_DEFTYPE:
-      if (! bx_cpuid_support_mtrr()) {
-        BX_ERROR(("WRMSR MSR_MTRR_DEFTYPE: MTRR is not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_MTRR)) {
+        BX_ERROR(("WRMSR MSR_MTRR_DEFTYPE: MTRR is not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! isMemTypeValidMTRR(val32_lo & 0xFF)) {
@@ -648,7 +653,7 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_PAT:
-      if (! bx_cpuid_support_pat()) {
+      if (! is_cpu_extension_supported(BX_ISA_PAT)) {
         BX_ERROR(("WRMSR BX_MSR_PAT: PAT is not enabled !"));
         return handle_unknown_wrmsr(index, val_64);
       }
@@ -674,8 +679,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
 #if BX_CPU_LEVEL >= 6
     case BX_MSR_TSC_DEADLINE:
-      if (! bx_cpuid_support_tsc_deadline()) {
-        BX_ERROR(("WRMSR BX_MSR_TSC_DEADLINE: TSC-Deadline not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_TSC_DEADLINE)) {
+        BX_ERROR(("WRMSR BX_MSR_TSC_DEADLINE: TSC-Deadline not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       BX_CPU_THIS_PTR lapic.set_tsc_deadline(val_64);
@@ -722,8 +727,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
 #if BX_SUPPORT_SVM
     case BX_SVM_HSAVE_PA_MSR:
-      if (! bx_cpuid_support_svm()) {
-        BX_ERROR(("WRMSR SVM_HSAVE_PA_MSR: SVM support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_SVM)) {
+        BX_ERROR(("WRMSR SVM_HSAVE_PA_MSR: SVM support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsValidPageAlignedPhyAddr(val_64)) {
@@ -739,7 +744,7 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
     case BX_MSR_STAR:
       if ((BX_CPU_THIS_PTR efer_suppmask & BX_EFER_SCE_MASK) == 0) {
-        BX_ERROR(("WRMSR MSR_STAR: SYSCALL/SYSRET support not enabled !"));
+        BX_ERROR(("WRMSR MSR_STAR: SYSCALL/SYSRET support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       MSR_STAR = val_64;
@@ -747,40 +752,40 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
 
 #if BX_SUPPORT_X86_64
     case BX_MSR_LSTAR:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("WRMSR MSR_LSTAR: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("WRMSR MSR_LSTAR: long mode support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsCanonical(val_64)) {
         BX_ERROR(("WRMSR: attempt to write non-canonical value to MSR_LSTAR !"));
         return 0;
       }
-      MSR_LSTAR = val_64;
+      BX_CPU_THIS_PTR msr.lstar = val_64;
       break;
 
     case BX_MSR_CSTAR:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("WRMSR MSR_CSTAR: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("WRMSR MSR_CSTAR: long mode support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsCanonical(val_64)) {
         BX_ERROR(("WRMSR: attempt to write non-canonical value to MSR_CSTAR !"));
         return 0;
       }
-      MSR_CSTAR = val_64;
+      BX_CPU_THIS_PTR msr.cstar = val_64;
       break;
 
     case BX_MSR_FMASK:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("WRMSR MSR_FMASK: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("WRMSR MSR_FMASK: long mode support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
-      MSR_FMASK = (Bit32u) val_64;
+      BX_CPU_THIS_PTR msr.fmask = (Bit32u) val_64;
       break;
 
     case BX_MSR_FSBASE:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("WRMSR MSR_FSBASE: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("WRMSR MSR_FSBASE: long mode support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsCanonical(val_64)) {
@@ -791,8 +796,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_GSBASE:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("WRMSR MSR_GSBASE: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("WRMSR MSR_GSBASE: long mode support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsCanonical(val_64)) {
@@ -803,8 +808,8 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_KERNELGSBASE:
-      if (! bx_cpuid_support_x86_64()) {
-        BX_ERROR(("WRMSR MSR_KERNELGSBASE: long mode support not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_LONG_MODE)) {
+        BX_ERROR(("WRMSR MSR_KERNELGSBASE: long mode support not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
       if (! IsCanonical(val_64)) {
@@ -815,11 +820,11 @@ bx_bool BX_CPP_AttrRegparmN(2) BX_CPU_C::wrmsr(Bit32u index, Bit64u val_64)
       break;
 
     case BX_MSR_TSC_AUX:
-      if (! bx_cpuid_support_rdtscp()) {
-        BX_ERROR(("WRMSR MSR_TSC_AUX: RTDSCP feature not enabled !"));
+      if (! is_cpu_extension_supported(BX_ISA_RDTSCP)) {
+        BX_ERROR(("WRMSR MSR_TSC_AUX: RTDSCP feature not enabled in the cpu model"));
         return handle_unknown_wrmsr(index, val_64);
       }
-      MSR_TSC_AUX = val32_lo;
+      BX_CPU_THIS_PTR msr.tsc_aux = val32_lo;
       break;
 #endif  // #if BX_SUPPORT_X86_64
 
@@ -872,7 +877,7 @@ bx_bool BX_CPU_C::relocate_apic(Bit64u val_64)
    * [M:63]  Reserved
    */
 
-#define BX_MSR_APICBASE_RESERVED_BITS (0x2ff | (bx_cpuid_support_x2apic() ? 0 : 0x400))
+#define BX_MSR_APICBASE_RESERVED_BITS (0x2ff | (is_cpu_extension_supported(BX_ISA_X2APIC) ? 0 : 0x400))
 
   if (BX_CPU_THIS_PTR msr.apicbase & 0x800) {
     Bit32u val32_hi = GET32H(val_64), val32_lo = GET32L(val_64);
@@ -887,7 +892,7 @@ bx_bool BX_CPU_C::relocate_apic(Bit64u val_64)
     }
 
 #if BX_CPU_LEVEL >= 6
-    if (bx_cpuid_support_x2apic()) {
+    if (is_cpu_extension_supported(BX_ISA_X2APIC)) {
       unsigned apic_state = (BX_CPU_THIS_PTR msr.apicbase >> 10) & 3;
       unsigned new_state = (val32_lo >> 10) & 3;
 
