@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift16.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: shift16.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -40,25 +40,28 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EwGwM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
+  Bit32u op1_16 = (Bit32u) read_RMW_virtual_word(i->seg(), eaddr);
 
   if (count) {
-    Bit16u op2_16 = BX_READ_16BIT_REG(i->src());
+    Bit32u op2_16 = (Bit32u) BX_READ_16BIT_REG(i->src());
 
     /* count < 32, since only lower 5 bits used */
-    temp_32 = ((Bit32u)(op1_16) << 16) | (op2_16); // double formed by op1:op2
+    temp_32 = (op1_16 << 16) | (op2_16); // double formed by op1:op2
     result_32 = temp_32 << count;
 
     // hack to act like x86 SHLD when count > 16
     if (count > 16) {
-      // when count > 16 actually shifting op1:op2:op2 << count,
+      // for Pentium processor, when count > 16, actually shifting op1:op2:op2 << count,
       // it is the same as shifting op2:op2 by count-16
+      // For P6 and later (CPU_LEVEL >= 6), when count > 16, actually shifting op1:op2:op1 << count,
+      // which is the same as shifting op2:op1 by count-16
+      // The behavior is undefined so both ways are correct, we prefer P6 way of implementation
       result_32 |= (op1_16 << (count - 16));
     }
 
     Bit16u result_16 = (Bit16u)(result_32 >> 16);
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
@@ -85,17 +88,20 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EwGwR(bxInstruction_c *i)
   count &= 0x1f; // use only 5 LSB's
 
   if (count) {
-    Bit16u op1_16 = BX_READ_16BIT_REG(i->dst());
-    Bit16u op2_16 = BX_READ_16BIT_REG(i->src());
+    Bit32u op1_16 = (Bit32u) BX_READ_16BIT_REG(i->dst());
+    Bit32u op2_16 = (Bit32u) BX_READ_16BIT_REG(i->src());
 
     /* count < 32, since only lower 5 bits used */
-    temp_32 = ((Bit32u)(op1_16) << 16) | (op2_16); // double formed by op1:op2
+    temp_32 = (op1_16 << 16) | (op2_16); // double formed by op1:op2
     result_32 = temp_32 << count;
 
     // hack to act like x86 SHLD when count > 16
     if (count > 16) {
-      // when count > 16 actually shifting op1:op2:op2 << count,
+      // for Pentium processor, when count > 16, actually shifting op1:op2:op2 << count,
       // it is the same as shifting op2:op2 by count-16
+      // For P6 and later (CPU_LEVEL >= 6), when count > 16, actually shifting op1:op2:op1 << count,
+      // which is the same as shifting op2:op1 by count-16
+      // The behavior is undefined so both ways are correct, we prefer P6 way of implementation
       result_32 |= (op1_16 << (count - 16));
     }
 
@@ -128,10 +134,10 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
+  Bit32u op1_16 = (Bit32u) read_RMW_virtual_word(i->seg(), eaddr);
 
   if (count) {
-    Bit16u op2_16 = BX_READ_16BIT_REG(i->src());
+    Bit32u op2_16 = (Bit32u) BX_READ_16BIT_REG(i->src());
 
     /* count < 32, since only lower 5 bits used */
     temp_32 = (op2_16 << 16) | op1_16; // double formed by op2:op1
@@ -139,14 +145,17 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwM(bxInstruction_c *i)
 
     // hack to act like x86 SHRD when count > 16
     if (count > 16) {
-      // when count > 16 actually shifting op2:op2:op1 >> count,
+      // for Pentium processor, when count > 16, actually shifting op2:op2:op1 >> count,
       // it is the same as shifting op2:op2 by count-16
+      // For P6 and later (CPU_LEVEL >= 6), when count > 16, actually shifting op1:op2:op1 >> count,
+      // which is the same as shifting op1:op2 by count-16
+      // The behavior is undefined so both ways are correct, we prefer P6 way of implementation
       result_32 |= (op1_16 << (32 - count));
     }
 
     Bit16u result_16 = (Bit16u) result_32;
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
@@ -172,8 +181,8 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwR(bxInstruction_c *i)
   count &= 0x1f; /* use only 5 LSB's */
 
   if (count) {
-    Bit16u op1_16 = BX_READ_16BIT_REG(i->dst());
-    Bit16u op2_16 = BX_READ_16BIT_REG(i->src());
+    Bit32u op1_16 = (Bit32u) BX_READ_16BIT_REG(i->dst());
+    Bit32u op2_16 = (Bit32u) BX_READ_16BIT_REG(i->src());
 
     /* count < 32, since only lower 5 bits used */
     temp_32 = (op2_16 << 16) | op1_16; // double formed by op2:op1
@@ -181,8 +190,11 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwR(bxInstruction_c *i)
 
     // hack to act like x86 SHRD when count > 16
     if (count > 16) {
-      // when count > 16 actually shifting op2:op2:op1 >> count,
+      // for Pentium processor, when count > 16, actually shifting op2:op2:op1 >> count,
       // it is the same as shifting op2:op2 by count-16
+      // For P6 and later (CPU_LEVEL >= 6), when count > 16, actually shifting op1:op2:op1 >> count,
+      // which is the same as shifting op1:op2 by count-16
+      // The behavior is undefined so both ways are correct, we prefer P6 way of implementation
       result_32 |= (op1_16 << (32 - count));
     }
 
@@ -227,7 +239,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EwM(bxInstruction_c *i)
 
     Bit16u result_16 = (op1_16 << count) | (op1_16 >> (16 - count));
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     bit0  = (result_16 & 0x1);
     bit15 = (result_16 >> 15);
@@ -301,7 +313,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EwM(bxInstruction_c *i)
 
     Bit16u result_16 = (op1_16 >> count) | (op1_16 << (16 - count));
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     bit14 = (result_16 >> 14) & 1;
     bit15 = (result_16 >> 15) & 1;
@@ -377,7 +389,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EwM(bxInstruction_c *i)
                   (op1_16 >> (17 - count));
     }
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     cf = (op1_16 >> (16 - count)) & 0x1;
     of = cf ^ (result_16 >> 15); // of = cf ^ result15
@@ -444,7 +456,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EwM(bxInstruction_c *i)
     Bit16u result_16 = (op1_16 >> count) | (getB_CF() << (16 - count)) |
                        (op1_16 << (17 - count));
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     cf = (op1_16 >> (count - 1)) & 0x1;
     of = ((Bit16u)((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result15 ^ result14
@@ -509,7 +521,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EwM(bxInstruction_c *i)
       result_16 = 0;
     }
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     SET_FLAGS_OSZAPC_LOGIC_16(result_16);
     SET_FLAGS_OxxxxC(of, cf);
@@ -571,7 +583,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EwM(bxInstruction_c *i)
   if (count) {
     Bit16u result_16 = (op1_16 >> count);
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
 
     cf = (op1_16 >> (count - 1)) & 0x1;
     // note, that of == result15 if count == 1 and
@@ -638,7 +650,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EwM(bxInstruction_c *i)
     /* signed overflow cannot happen in SAR instruction */
     SET_FLAGS_OxxxxC(0, cf);
 
-    write_RMW_virtual_word(result_16);
+    write_RMW_linear_word(result_16);
   }
 
   BX_NEXT_INSTR(i);
