@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift32.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: shift32.cc 12613 2015-01-25 20:55:10Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -45,7 +45,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EdGdM(bxInstruction_c *i)
 
     Bit32u result_32 = (op1_32 << count) | (op2_32 >> (32 - count));
 
-    write_RMW_virtual_dword(result_32);
+    write_RMW_linear_dword(result_32);
 
     SET_FLAGS_OSZAPC_LOGIC_32(result_32);
 
@@ -112,7 +112,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EdGdM(bxInstruction_c *i)
 
     Bit32u result_32 = (op2_32 << (32 - count)) | (op1_32 >> count);
 
-    write_RMW_virtual_dword(result_32);
+    write_RMW_linear_dword(result_32);
 
     SET_FLAGS_OSZAPC_LOGIC_32(result_32);
 
@@ -176,7 +176,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EdM(bxInstruction_c *i)
   if (count) {
     Bit32u result_32 = (op1_32 << count) | (op1_32 >> (32 - count));
 
-    write_RMW_virtual_dword(result_32);
+    write_RMW_linear_dword(result_32);
 
     unsigned bit0  = (result_32 & 0x1);
     unsigned bit31 = (result_32 >> 31);
@@ -236,7 +236,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EdM(bxInstruction_c *i)
   if (count) {
     Bit32u result_32 = (op1_32 >> count) | (op1_32 << (32 - count));
 
-    write_RMW_virtual_dword(result_32);
+    write_RMW_linear_dword(result_32);
 
     bit31 = (result_32 >> 31) & 1;
     bit30 = (result_32 >> 30) & 1;
@@ -305,7 +305,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EdM(bxInstruction_c *i)
                 (op1_32 >> (33 - count));
   }
 
-  write_RMW_virtual_dword(result_32);
+  write_RMW_linear_dword(result_32);
 
   cf = (op1_32 >> (32 - count)) & 0x1;
   of = cf ^ (result_32 >> 31); // of = cf ^ result31
@@ -379,7 +379,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EdM(bxInstruction_c *i)
                 (op1_32 << (33 - count));
   }
 
-  write_RMW_virtual_dword(result_32);
+  write_RMW_linear_dword(result_32);
 
   cf = (op1_32 >> (count - 1)) & 0x1;
   of = ((result_32 << 1) ^ result_32) >> 31; // of = result30 ^ result31
@@ -445,7 +445,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EdM(bxInstruction_c *i)
     /* count < 32, since only lower 5 bits used */
     Bit32u result_32 = (op1_32 << count);
 
-    write_RMW_virtual_dword(result_32);
+    write_RMW_linear_dword(result_32);
 
     cf = (op1_32 >> (32 - count)) & 0x1;
     of = cf ^ (result_32 >> 31);
@@ -506,7 +506,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EdM(bxInstruction_c *i)
   if (count) {
     Bit32u result_32 = (op1_32 >> count);
 
-    write_RMW_virtual_dword(result_32);
+    write_RMW_linear_dword(result_32);
 
     unsigned cf = (op1_32 >> (count - 1)) & 0x1;
     // note, that of == result31 if count == 1 and
@@ -570,7 +570,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EdM(bxInstruction_c *i)
     /* count < 32, since only lower 5 bits used */
     Bit32u result_32 = ((Bit32s) op1_32) >> count;
 
-    write_RMW_virtual_dword(result_32);
+    write_RMW_linear_dword(result_32);
 
     SET_FLAGS_OSZAPC_LOGIC_32(result_32);
     unsigned cf = (op1_32 >> (count - 1)) & 1;
