@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift64.cc 11313 2012-08-05 13:52:40Z sshwarts $
+// $Id: shift64.cc 12619 2015-01-26 20:01:25Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -35,7 +35,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqM(bxInstruction_c *i)
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_SHLD_EqGq)
     count = CL;
@@ -49,7 +49,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EqGqM(bxInstruction_c *i)
 
     result_64 = (op1_64 << count) | (op2_64 >> (64 - count));
 
-    write_RMW_virtual_qword(result_64);
+    write_RMW_linear_qword(result_64);
 
     SET_FLAGS_OSZAPC_LOGIC_64(result_64);
 
@@ -101,7 +101,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqM(bxInstruction_c *i)
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_SHRD_EqGq)
     count = CL;
@@ -115,7 +115,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EqGqM(bxInstruction_c *i)
 
     result_64 = (op2_64 << (64 - count)) | (op1_64 >> count);
 
-    write_RMW_virtual_qword(result_64);
+    write_RMW_linear_qword(result_64);
 
     SET_FLAGS_OSZAPC_LOGIC_64(result_64);
 
@@ -164,7 +164,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_ROL_Eq)
     count = CL;
@@ -176,7 +176,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_EqM(bxInstruction_c *i)
   if (count) {
     Bit64u result_64 = (op1_64 << count) | (op1_64 >> (64 - count));
 
-    write_RMW_virtual_qword(result_64);
+    write_RMW_linear_qword(result_64);
 
     unsigned bit0  = (result_64 & 0x1);
     unsigned bit63 = (result_64 >> 63);
@@ -218,7 +218,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_ROR_Eq)
     count = CL;
@@ -230,7 +230,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_EqM(bxInstruction_c *i)
   if (count) {
     Bit64u result_64 = (op1_64 >> count) | (op1_64 << (64 - count));
 
-    write_RMW_virtual_qword(result_64);
+    write_RMW_linear_qword(result_64);
 
     unsigned bit63 = (result_64 >> 63) & 1;
     unsigned bit62 = (result_64 >> 62) & 1;
@@ -274,7 +274,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_RCL_Eq)
     count = CL;
@@ -295,7 +295,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_EqM(bxInstruction_c *i)
                 (op1_64 >> (65 - count));
   }
 
-  write_RMW_virtual_qword(result_64);
+  write_RMW_linear_qword(result_64);
 
   cf = (op1_64 >> (64 - count)) & 0x1;
   of = cf ^ (result_64 >> 63); // of = cf ^ result63
@@ -348,7 +348,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_RCR_Eq)
     count = CL;
@@ -369,7 +369,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_EqM(bxInstruction_c *i)
                 (op1_64 << (65 - count));
   }
 
-  write_RMW_virtual_qword(result_64);
+  write_RMW_linear_qword(result_64);
 
   cf = (op1_64 >> (count - 1)) & 0x1;
   of = ((result_64 << 1) ^ result_64) >> 63;
@@ -420,7 +420,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_SHL_Eq)
     count = CL;
@@ -436,7 +436,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_EqM(bxInstruction_c *i)
     unsigned cf = (op1_64 >> (64 - count)) & 0x1;
     unsigned of = cf ^ (result_64 >> 63);
 
-    write_RMW_virtual_qword(result_64);
+    write_RMW_linear_qword(result_64);
 
     SET_FLAGS_OSZAPC_LOGIC_64(result_64);
     SET_FLAGS_OxxxxC(of, cf);
@@ -479,7 +479,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_SHR_Eq)
     count = CL;
@@ -491,7 +491,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_EqM(bxInstruction_c *i)
   if (count) {
     Bit64u result_64 = (op1_64 >> count);
 
-    write_RMW_virtual_qword(result_64);
+    write_RMW_linear_qword(result_64);
 
     unsigned cf = (op1_64 >> (count - 1)) & 0x1;
     // note, that of == result63 if count == 1 and
@@ -539,7 +539,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqM(bxInstruction_c *i)
 
   bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  Bit64u op1_64 = read_RMW_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
 
   if (i->getIaOpcode() == BX_IA_SAR_Eq)
     count = CL;
@@ -552,7 +552,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_EqM(bxInstruction_c *i)
     /* count < 64, since only lower 6 bits used */
     Bit64u result_64 = ((Bit64s) op1_64) >> count;
 
-    write_RMW_virtual_qword(result_64);
+    write_RMW_linear_qword(result_64);
 
     SET_FLAGS_OSZAPC_LOGIC_64(result_64);
     unsigned cf = (op1_64 >> (count - 1)) & 1;
