@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////6////////////////////////
-// $Id: sse.cc 11323 2012-08-07 14:38:43Z sshwarts $
+// $Id: sse.cc 12519 2014-10-22 18:24:33Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2003-2012 Stanislav Shwartsman
+//   Copyright (c) 2003-2014 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -46,36 +46,36 @@
     BX_NEXT_INSTR(i);                                                                      \
   }
 
-SSE_2OP(PHADDW_VdqWdqR, sse_phaddw)
-SSE_2OP(PHADDSW_VdqWdqR, sse_phaddsw)
-SSE_2OP(PHADDD_VdqWdqR, sse_phaddd)
-SSE_2OP(PHSUBW_VdqWdqR, sse_phsubw)
-SSE_2OP(PHSUBSW_VdqWdqR, sse_phsubsw)
-SSE_2OP(PHSUBD_VdqWdqR, sse_phsubd)
+SSE_2OP(PHADDW_VdqWdqR, xmm_phaddw)
+SSE_2OP(PHADDSW_VdqWdqR, xmm_phaddsw)
+SSE_2OP(PHADDD_VdqWdqR, xmm_phaddd)
+SSE_2OP(PHSUBW_VdqWdqR, xmm_phsubw)
+SSE_2OP(PHSUBSW_VdqWdqR, xmm_phsubsw)
+SSE_2OP(PHSUBD_VdqWdqR, xmm_phsubd)
 
-SSE_2OP(PSIGNB_VdqWdqR, sse_psignb)
-SSE_2OP(PSIGNW_VdqWdqR, sse_psignw)
-SSE_2OP(PSIGND_VdqWdqR, sse_psignd)
+SSE_2OP(PSIGNB_VdqWdqR, xmm_psignb)
+SSE_2OP(PSIGNW_VdqWdqR, xmm_psignw)
+SSE_2OP(PSIGND_VdqWdqR, xmm_psignd)
 
-SSE_2OP(PCMPEQQ_VdqWdqR, sse_pcmpeqq)
-SSE_2OP(PCMPGTQ_VdqWdqR, sse_pcmpgtq)
+SSE_2OP(PCMPEQQ_VdqWdqR, xmm_pcmpeqq)
+SSE_2OP(PCMPGTQ_VdqWdqR, xmm_pcmpgtq)
 
-SSE_2OP(PMINSB_VdqWdqR, sse_pminsb)
-SSE_2OP(PMINSD_VdqWdqR, sse_pminsd)
-SSE_2OP(PMINUW_VdqWdqR, sse_pminuw)
-SSE_2OP(PMINUD_VdqWdqR, sse_pminud)
-SSE_2OP(PMAXSB_VdqWdqR, sse_pmaxsb)
-SSE_2OP(PMAXSD_VdqWdqR, sse_pmaxsd)
-SSE_2OP(PMAXUW_VdqWdqR, sse_pmaxuw)
-SSE_2OP(PMAXUD_VdqWdqR, sse_pmaxud)
+SSE_2OP(PMINSB_VdqWdqR, xmm_pminsb)
+SSE_2OP(PMINSD_VdqWdqR, xmm_pminsd)
+SSE_2OP(PMINUW_VdqWdqR, xmm_pminuw)
+SSE_2OP(PMINUD_VdqWdqR, xmm_pminud)
+SSE_2OP(PMAXSB_VdqWdqR, xmm_pmaxsb)
+SSE_2OP(PMAXSD_VdqWdqR, xmm_pmaxsd)
+SSE_2OP(PMAXUW_VdqWdqR, xmm_pmaxuw)
+SSE_2OP(PMAXUD_VdqWdqR, xmm_pmaxud)
 
-SSE_2OP(PACKUSDW_VdqWdqR, sse_packusdw)
+SSE_2OP(PACKUSDW_VdqWdqR, xmm_packusdw)
 
-SSE_2OP(PMULLD_VdqWdqR, sse_pmulld)
-SSE_2OP(PMULDQ_VdqWdqR, sse_pmuldq)
-SSE_2OP(PMULHRSW_VdqWdqR, sse_pmulhrsw)
+SSE_2OP(PMULLD_VdqWdqR, xmm_pmulld)
+SSE_2OP(PMULDQ_VdqWdqR, xmm_pmuldq)
+SSE_2OP(PMULHRSW_VdqWdqR, xmm_pmulhrsw)
 
-SSE_2OP(PMADDUBSW_VdqWdqR, sse_pmaddubsw)
+SSE_2OP(PMADDUBSW_VdqWdqR, xmm_pmaddubsw)
 
 #endif // BX_CPU_LEVEL >= 6
 
@@ -95,65 +95,65 @@ SSE_2OP(PMADDUBSW_VdqWdqR, sse_pmaddubsw)
 
 #endif
 
-SSE_2OP_CPU_LEVEL6(PMINUB_VdqWdqR, sse_pminub)
-SSE_2OP_CPU_LEVEL6(PMINSW_VdqWdqR, sse_pminsw)
-SSE_2OP_CPU_LEVEL6(PMAXUB_VdqWdqR, sse_pmaxub)
-SSE_2OP_CPU_LEVEL6(PMAXSW_VdqWdqR, sse_pmaxsw)
+SSE_2OP_CPU_LEVEL6(PMINUB_VdqWdqR, xmm_pminub)
+SSE_2OP_CPU_LEVEL6(PMINSW_VdqWdqR, xmm_pminsw)
+SSE_2OP_CPU_LEVEL6(PMAXUB_VdqWdqR, xmm_pmaxub)
+SSE_2OP_CPU_LEVEL6(PMAXSW_VdqWdqR, xmm_pmaxsw)
 
-SSE_2OP_CPU_LEVEL6(PAVGB_VdqWdqR, sse_pavgb)
-SSE_2OP_CPU_LEVEL6(PAVGW_VdqWdqR, sse_pavgw)
+SSE_2OP_CPU_LEVEL6(PAVGB_VdqWdqR, xmm_pavgb)
+SSE_2OP_CPU_LEVEL6(PAVGW_VdqWdqR, xmm_pavgw)
 
-SSE_2OP_CPU_LEVEL6(PCMPEQB_VdqWdqR, sse_pcmpeqb)
-SSE_2OP_CPU_LEVEL6(PCMPEQW_VdqWdqR, sse_pcmpeqw)
-SSE_2OP_CPU_LEVEL6(PCMPEQD_VdqWdqR, sse_pcmpeqd)
-SSE_2OP_CPU_LEVEL6(PCMPGTB_VdqWdqR, sse_pcmpgtb)
-SSE_2OP_CPU_LEVEL6(PCMPGTW_VdqWdqR, sse_pcmpgtw)
-SSE_2OP_CPU_LEVEL6(PCMPGTD_VdqWdqR, sse_pcmpgtd)
+SSE_2OP_CPU_LEVEL6(PCMPEQB_VdqWdqR, xmm_pcmpeqb)
+SSE_2OP_CPU_LEVEL6(PCMPEQW_VdqWdqR, xmm_pcmpeqw)
+SSE_2OP_CPU_LEVEL6(PCMPEQD_VdqWdqR, xmm_pcmpeqd)
+SSE_2OP_CPU_LEVEL6(PCMPGTB_VdqWdqR, xmm_pcmpgtb)
+SSE_2OP_CPU_LEVEL6(PCMPGTW_VdqWdqR, xmm_pcmpgtw)
+SSE_2OP_CPU_LEVEL6(PCMPGTD_VdqWdqR, xmm_pcmpgtd)
 
-SSE_2OP_CPU_LEVEL6(ANDPS_VpsWpsR, sse_andps)
-SSE_2OP_CPU_LEVEL6(ANDNPS_VpsWpsR, sse_andnps)
-SSE_2OP_CPU_LEVEL6(ORPS_VpsWpsR, sse_orps)
-SSE_2OP_CPU_LEVEL6(XORPS_VpsWpsR, sse_xorps)
+SSE_2OP_CPU_LEVEL6(ANDPS_VpsWpsR, xmm_andps)
+SSE_2OP_CPU_LEVEL6(ANDNPS_VpsWpsR, xmm_andnps)
+SSE_2OP_CPU_LEVEL6(ORPS_VpsWpsR, xmm_orps)
+SSE_2OP_CPU_LEVEL6(XORPS_VpsWpsR, xmm_xorps)
 
-SSE_2OP_CPU_LEVEL6(PSUBB_VdqWdqR, sse_psubb)
-SSE_2OP_CPU_LEVEL6(PSUBW_VdqWdqR, sse_psubw)
-SSE_2OP_CPU_LEVEL6(PSUBD_VdqWdqR, sse_psubd)
-SSE_2OP_CPU_LEVEL6(PSUBQ_VdqWdqR, sse_psubq)
-SSE_2OP_CPU_LEVEL6(PADDB_VdqWdqR, sse_paddb)
-SSE_2OP_CPU_LEVEL6(PADDW_VdqWdqR, sse_paddw)
-SSE_2OP_CPU_LEVEL6(PADDD_VdqWdqR, sse_paddd)
-SSE_2OP_CPU_LEVEL6(PADDQ_VdqWdqR, sse_paddq)
+SSE_2OP_CPU_LEVEL6(PSUBB_VdqWdqR, xmm_psubb)
+SSE_2OP_CPU_LEVEL6(PSUBW_VdqWdqR, xmm_psubw)
+SSE_2OP_CPU_LEVEL6(PSUBD_VdqWdqR, xmm_psubd)
+SSE_2OP_CPU_LEVEL6(PSUBQ_VdqWdqR, xmm_psubq)
+SSE_2OP_CPU_LEVEL6(PADDB_VdqWdqR, xmm_paddb)
+SSE_2OP_CPU_LEVEL6(PADDW_VdqWdqR, xmm_paddw)
+SSE_2OP_CPU_LEVEL6(PADDD_VdqWdqR, xmm_paddd)
+SSE_2OP_CPU_LEVEL6(PADDQ_VdqWdqR, xmm_paddq)
 
-SSE_2OP_CPU_LEVEL6(PSUBSB_VdqWdqR, sse_psubsb)
-SSE_2OP_CPU_LEVEL6(PSUBUSB_VdqWdqR, sse_psubusb)
-SSE_2OP_CPU_LEVEL6(PSUBSW_VdqWdqR, sse_psubsw)
-SSE_2OP_CPU_LEVEL6(PSUBUSW_VdqWdqR, sse_psubusw)
-SSE_2OP_CPU_LEVEL6(PADDSB_VdqWdqR, sse_paddsb)
-SSE_2OP_CPU_LEVEL6(PADDUSB_VdqWdqR, sse_paddusb)
-SSE_2OP_CPU_LEVEL6(PADDSW_VdqWdqR, sse_paddsw)
-SSE_2OP_CPU_LEVEL6(PADDUSW_VdqWdqR, sse_paddusw)
+SSE_2OP_CPU_LEVEL6(PSUBSB_VdqWdqR, xmm_psubsb)
+SSE_2OP_CPU_LEVEL6(PSUBUSB_VdqWdqR, xmm_psubusb)
+SSE_2OP_CPU_LEVEL6(PSUBSW_VdqWdqR, xmm_psubsw)
+SSE_2OP_CPU_LEVEL6(PSUBUSW_VdqWdqR, xmm_psubusw)
+SSE_2OP_CPU_LEVEL6(PADDSB_VdqWdqR, xmm_paddsb)
+SSE_2OP_CPU_LEVEL6(PADDUSB_VdqWdqR, xmm_paddusb)
+SSE_2OP_CPU_LEVEL6(PADDSW_VdqWdqR, xmm_paddsw)
+SSE_2OP_CPU_LEVEL6(PADDUSW_VdqWdqR, xmm_paddusw)
 
-SSE_2OP_CPU_LEVEL6(PACKUSWB_VdqWdqR, sse_packuswb)
-SSE_2OP_CPU_LEVEL6(PACKSSWB_VdqWdqR, sse_packsswb)
-SSE_2OP_CPU_LEVEL6(PACKSSDW_VdqWdqR, sse_packssdw)
+SSE_2OP_CPU_LEVEL6(PACKUSWB_VdqWdqR, xmm_packuswb)
+SSE_2OP_CPU_LEVEL6(PACKSSWB_VdqWdqR, xmm_packsswb)
+SSE_2OP_CPU_LEVEL6(PACKSSDW_VdqWdqR, xmm_packssdw)
 
-SSE_2OP_CPU_LEVEL6(UNPCKLPS_VpsWpsR, sse_unpcklps)
-SSE_2OP_CPU_LEVEL6(UNPCKHPS_VpsWpsR, sse_unpckhps)
-SSE_2OP_CPU_LEVEL6(PUNPCKLQDQ_VdqWdqR, sse_unpcklpd)
-SSE_2OP_CPU_LEVEL6(PUNPCKHQDQ_VdqWdqR, sse_unpckhpd)
+SSE_2OP_CPU_LEVEL6(UNPCKLPS_VpsWpsR, xmm_unpcklps)
+SSE_2OP_CPU_LEVEL6(UNPCKHPS_VpsWpsR, xmm_unpckhps)
+SSE_2OP_CPU_LEVEL6(PUNPCKLQDQ_VdqWdqR, xmm_unpcklpd)
+SSE_2OP_CPU_LEVEL6(PUNPCKHQDQ_VdqWdqR, xmm_unpckhpd)
 
-SSE_2OP_CPU_LEVEL6(PUNPCKLBW_VdqWdqR, sse_punpcklbw)
-SSE_2OP_CPU_LEVEL6(PUNPCKLWD_VdqWdqR, sse_punpcklwd)
-SSE_2OP_CPU_LEVEL6(PUNPCKHBW_VdqWdqR, sse_punpckhbw)
-SSE_2OP_CPU_LEVEL6(PUNPCKHWD_VdqWdqR, sse_punpckhwd)
+SSE_2OP_CPU_LEVEL6(PUNPCKLBW_VdqWdqR, xmm_punpcklbw)
+SSE_2OP_CPU_LEVEL6(PUNPCKLWD_VdqWdqR, xmm_punpcklwd)
+SSE_2OP_CPU_LEVEL6(PUNPCKHBW_VdqWdqR, xmm_punpckhbw)
+SSE_2OP_CPU_LEVEL6(PUNPCKHWD_VdqWdqR, xmm_punpckhwd)
 
-SSE_2OP_CPU_LEVEL6(PMULLW_VdqWdqR, sse_pmullw)
-SSE_2OP_CPU_LEVEL6(PMULHW_VdqWdqR, sse_pmulhw)
-SSE_2OP_CPU_LEVEL6(PMULHUW_VdqWdqR, sse_pmulhuw)
-SSE_2OP_CPU_LEVEL6(PMULUDQ_VdqWdqR, sse_pmuludq)
-SSE_2OP_CPU_LEVEL6(PMADDWD_VdqWdqR, sse_pmaddwd)
+SSE_2OP_CPU_LEVEL6(PMULLW_VdqWdqR, xmm_pmullw)
+SSE_2OP_CPU_LEVEL6(PMULHW_VdqWdqR, xmm_pmulhw)
+SSE_2OP_CPU_LEVEL6(PMULHUW_VdqWdqR, xmm_pmulhuw)
+SSE_2OP_CPU_LEVEL6(PMULUDQ_VdqWdqR, xmm_pmuludq)
+SSE_2OP_CPU_LEVEL6(PMADDWD_VdqWdqR, xmm_pmaddwd)
 
-SSE_2OP_CPU_LEVEL6(PSADBW_VdqWdqR, sse_psadbw)
+SSE_2OP_CPU_LEVEL6(PSADBW_VdqWdqR, xmm_psadbw)
 
 #if BX_CPU_LEVEL >= 6
 
@@ -168,16 +168,16 @@ SSE_2OP_CPU_LEVEL6(PSADBW_VdqWdqR, sse_psadbw)
     BX_NEXT_INSTR(i);                                                                      \
   }
 
-SSE_1OP(PABSB_VdqWdqR, sse_pabsb)
-SSE_1OP(PABSW_VdqWdqR, sse_pabsw)
-SSE_1OP(PABSD_VdqWdqR, sse_pabsd)
+SSE_1OP(PABSB_VdqWdqR, xmm_pabsb)
+SSE_1OP(PABSW_VdqWdqR, xmm_pabsw)
+SSE_1OP(PABSD_VdqWdqR, xmm_pabsd)
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSHUFB_VdqWdqR(bxInstruction_c *i)
 {
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst());
   BxPackedXmmRegister op2 = BX_READ_XMM_REG(i->src()), result;
 
-  sse_pshufb(&result, &op1, &op2);
+  xmm_pshufb(&result, &op1, &op2);
 
   BX_WRITE_XMM_REG(i->dst(), result);
 
@@ -186,21 +186,21 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSHUFB_VdqWdqR(bxInstruction_c *i)
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PBLENDVB_VdqWdqR(bxInstruction_c *i)
 {
-  sse_pblendvb(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), &BX_XMM_REG(0));
+  xmm_pblendvb(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), &BX_XMM_REG(0));
 
   BX_NEXT_INSTR(i);
 }
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BLENDVPS_VpsWpsR(bxInstruction_c *i)
 {
-  sse_blendvps(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), &BX_XMM_REG(0));
+  xmm_blendvps(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), &BX_XMM_REG(0));
 
   BX_NEXT_INSTR(i);
 }
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BLENDVPD_VpdWpdR(bxInstruction_c *i)
 {
-  sse_blendvpd(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), &BX_XMM_REG(0));
+  xmm_blendvpd(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), &BX_XMM_REG(0));
 
   BX_NEXT_INSTR(i);
 }
@@ -208,15 +208,14 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BLENDVPD_VpdWpdR(bxInstruction_c *
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PTEST_VdqWdqR(bxInstruction_c *i)
 {
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst()), op2 = BX_READ_XMM_REG(i->src());
-  unsigned result = 0;
+
+  clearEFlagsOSZAPC();
 
   if ((op2.xmm64u(0) &  op1.xmm64u(0)) == 0 &&
-      (op2.xmm64u(1) &  op1.xmm64u(1)) == 0) result |= EFlagsZFMask;
+      (op2.xmm64u(1) &  op1.xmm64u(1)) == 0) assert_ZF();
 
   if ((op2.xmm64u(0) & ~op1.xmm64u(0)) == 0 &&
-      (op2.xmm64u(1) & ~op1.xmm64u(1)) == 0) result |= EFlagsCFMask;
-
-  setEFlagsOSZAPC(result);
+      (op2.xmm64u(1) & ~op1.xmm64u(1)) == 0) assert_CF();
 
   BX_NEXT_INSTR(i);
 }
@@ -243,21 +242,21 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PHMINPOSUW_VdqWdqR(bxInstruction_c
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BLENDPS_VpsWpsIbR(bxInstruction_c *i)
 {
-  sse_blendps(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), i->Ib());
+  xmm_blendps(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), i->Ib());
 
   BX_NEXT_INSTR(i);
 }
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::BLENDPD_VpdWpdIbR(bxInstruction_c *i)
 {
-  sse_blendpd(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), i->Ib());
+  xmm_blendpd(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), i->Ib());
 
   BX_NEXT_INSTR(i);
 }
 
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PBLENDW_VdqWdqIbR(bxInstruction_c *i)
 {
-  sse_pblendw(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), i->Ib());
+  xmm_pblendw(&BX_XMM_REG(i->dst()), &BX_XMM_REG(i->src()), i->Ib());
 
   BX_NEXT_INSTR(i);
 }
@@ -322,6 +321,18 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRD_EdVdqIbR(bxInstruction_c *i
   BX_NEXT_INSTR(i);
 }
 
+#if BX_SUPPORT_X86_64
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRQ_EqVdqIbR(bxInstruction_c *i)
+{
+  BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
+
+  Bit64u result = op.xmm64u(i->Ib() & 1);
+  BX_WRITE_64BIT_REG(i->dst(), result);
+
+  BX_NEXT_INSTR(i);
+}
+#endif
+
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRD_EdVdqIbM(bxInstruction_c *i)
 {
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
@@ -332,7 +343,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRD_EdVdqIbM(bxInstruction_c *i
   if (i->os64L())  /* 64 bit operand size mode */
   {
      Bit64u result = op.xmm64u(i->Ib() & 1);
-     write_virtual_qword_64(i->seg(), eaddr, result);
+     write_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr), result);
   }
   else
 #endif
@@ -344,6 +355,19 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRD_EdVdqIbM(bxInstruction_c *i
   BX_NEXT_INSTR(i);
 }
 
+#if BX_SUPPORT_X86_64
+BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PEXTRQ_EqVdqIbM(bxInstruction_c *i)
+{
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
+  Bit64u result = op.xmm64u(i->Ib() & 1);
+  write_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr), result);
+
+  BX_NEXT_INSTR(i);
+}
+#endif
+
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::EXTRACTPS_EdVpsIbR(bxInstruction_c *i)
 {
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->src());
@@ -368,9 +392,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PINSRB_VdqHdqEbIbR(bxInstruction_c
 {
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->src1());
 
-  Bit8u op2 = (Bit8u) BX_READ_16BIT_REG(i->src2()); // won't allow reading of AH/CH/BH/DH
-
-  op1.xmmubyte(i->Ib() & 0xF) = op2;
+  op1.xmmubyte(i->Ib() & 0xF) = BX_READ_8BIT_REGL(i->src2()); // won't allow reading of AH/CH/BH/DH
 
   BX_WRITE_XMM_REGZ(i->dst(), op1, i->getVL());
 
@@ -445,7 +467,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PINSRD_VdqHdqEdIbM(bxInstruction_c
 
 #if BX_SUPPORT_X86_64
   if (i->os64L()) {  /* 64 bit operand size mode */
-    Bit64u op2 = read_virtual_qword_64(i->seg(), eaddr);
+    Bit64u op2 = read_linear_qword(i->seg(), get_laddr64(i->seg(), eaddr));
     op1.xmm64u(i->Ib() & 1) = op2;
   }
   else
@@ -465,7 +487,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::MPSADBW_VdqWdqIbR(bxInstruction_c
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst());
   BxPackedXmmRegister op2 = BX_READ_XMM_REG(i->src()), result;
 
-  sse_mpsadbw(&result, &op1, &op2, i->Ib() & 0x7);
+  xmm_mpsadbw(&result, &op1, &op2, i->Ib() & 0x7);
 
   BX_WRITE_XMM_REG(i->dst(), result);
 
@@ -479,7 +501,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSHUFD_VdqWdqIbR(bxInstruction_c *
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->src()), result;
 
-  sse_shufps(&result, &op, &op, i->Ib());
+  xmm_shufps(&result, &op, &op, i->Ib());
 
   BX_WRITE_XMM_REG(i->dst(), result);
 #endif
@@ -492,7 +514,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSHUFHW_VdqWdqIbR(bxInstruction_c
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->src()), result;
 
-  sse_pshufhw(&result, &op, i->Ib());
+  xmm_pshufhw(&result, &op, i->Ib());
 
   BX_WRITE_XMM_REG(i->dst(), result);
 #endif
@@ -505,7 +527,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::PSHUFLW_VdqWdqIbR(bxInstruction_c
 #if BX_CPU_LEVEL >= 6
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->src()), result;
 
-  sse_pshuflw(&result, &op, i->Ib());
+  xmm_pshuflw(&result, &op, i->Ib());
 
   BX_WRITE_XMM_REG(i->dst(), result);
 #endif
@@ -545,7 +567,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHUFPS_VpsWpsIbR(bxInstruction_c *
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst());
   BxPackedXmmRegister op2 = BX_READ_XMM_REG(i->src()), result;
 
-  sse_shufps(&result, &op1, &op2, i->Ib());
+  xmm_shufps(&result, &op1, &op2, i->Ib());
 
   BX_WRITE_XMM_REG(i->dst(), result);
 #endif
@@ -559,7 +581,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHUFPD_VpdWpdIbR(bxInstruction_c *
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->dst());
   BxPackedXmmRegister op2 = BX_READ_XMM_REG(i->src()), result;
 
-  sse_shufpd(&result, &op1, &op2, i->Ib());
+  xmm_shufpd(&result, &op1, &op2, i->Ib());
 
   BX_WRITE_XMM_REG(i->dst(), result);
 #endif
@@ -593,14 +615,14 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::SHUFPD_VpdWpdIbR(bxInstruction_c *
 
 #endif
 
-SSE_PSHIFT_CPU_LEVEL6(PSRLW_VdqWdqR, sse_psrlw);
-SSE_PSHIFT_CPU_LEVEL6(PSRLD_VdqWdqR, sse_psrld);
-SSE_PSHIFT_CPU_LEVEL6(PSRLQ_VdqWdqR, sse_psrlq);
-SSE_PSHIFT_CPU_LEVEL6(PSRAW_VdqWdqR, sse_psraw);
-SSE_PSHIFT_CPU_LEVEL6(PSRAD_VdqWdqR, sse_psrad);
-SSE_PSHIFT_CPU_LEVEL6(PSLLW_VdqWdqR, sse_psllw);
-SSE_PSHIFT_CPU_LEVEL6(PSLLD_VdqWdqR, sse_pslld);
-SSE_PSHIFT_CPU_LEVEL6(PSLLQ_VdqWdqR, sse_psllq);
+SSE_PSHIFT_CPU_LEVEL6(PSRLW_VdqWdqR, xmm_psrlw);
+SSE_PSHIFT_CPU_LEVEL6(PSRLD_VdqWdqR, xmm_psrld);
+SSE_PSHIFT_CPU_LEVEL6(PSRLQ_VdqWdqR, xmm_psrlq);
+SSE_PSHIFT_CPU_LEVEL6(PSRAW_VdqWdqR, xmm_psraw);
+SSE_PSHIFT_CPU_LEVEL6(PSRAD_VdqWdqR, xmm_psrad);
+SSE_PSHIFT_CPU_LEVEL6(PSLLW_VdqWdqR, xmm_psllw);
+SSE_PSHIFT_CPU_LEVEL6(PSLLD_VdqWdqR, xmm_pslld);
+SSE_PSHIFT_CPU_LEVEL6(PSLLQ_VdqWdqR, xmm_psllq);
 
 #if BX_CPU_LEVEL >= 6
 
@@ -624,24 +646,24 @@ SSE_PSHIFT_CPU_LEVEL6(PSLLQ_VdqWdqR, sse_psllq);
 
 #endif
 
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLW_UdqIb, sse_psrlw);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLD_UdqIb, sse_psrld);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLQ_UdqIb, sse_psrlq);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSRAW_UdqIb, sse_psraw);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSRAD_UdqIb, sse_psrad);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLW_UdqIb, sse_psllw);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLD_UdqIb, sse_pslld);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLQ_UdqIb, sse_psllq);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLW_UdqIb, xmm_psrlw);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLD_UdqIb, xmm_psrld);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLQ_UdqIb, xmm_psrlq);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSRAW_UdqIb, xmm_psraw);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSRAD_UdqIb, xmm_psrad);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLW_UdqIb, xmm_psllw);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLD_UdqIb, xmm_pslld);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLQ_UdqIb, xmm_psllq);
 
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLDQ_UdqIb, sse_psrldq);
-SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLDQ_UdqIb, sse_pslldq);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSRLDQ_UdqIb, xmm_psrldq);
+SSE_PSHIFT_IMM_CPU_LEVEL6(PSLLDQ_UdqIb, xmm_pslldq);
 
 /* ************************ */
 /* SSE4A (AMD) INSTRUCTIONS */
 /* ************************ */
 
 #if BX_CPU_LEVEL >= 6
-BX_CPP_INLINE Bit64u sse_extrq(Bit64u src, unsigned shift, unsigned len)
+BX_CPP_INLINE Bit64u xmm_extrq(Bit64u src, unsigned shift, unsigned len)
 {
   len   &= 0x3f;
   shift &= 0x3f;
@@ -655,7 +677,7 @@ BX_CPP_INLINE Bit64u sse_extrq(Bit64u src, unsigned shift, unsigned len)
   return src;
 }
 
-BX_CPP_INLINE Bit64u sse_insertq(Bit64u dest, Bit64u src, unsigned shift, unsigned len)
+BX_CPP_INLINE Bit64u xmm_insertq(Bit64u dest, Bit64u src, unsigned shift, unsigned len)
 {
   Bit64u mask;
 
@@ -675,7 +697,7 @@ BX_CPP_INLINE Bit64u sse_insertq(Bit64u dest, Bit64u src, unsigned shift, unsign
 BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::EXTRQ_UdqIbIb(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), sse_extrq(BX_READ_XMM_REG_LO_QWORD(i->dst()), i->Ib2(), i->Ib()));
+  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), xmm_extrq(BX_READ_XMM_REG_LO_QWORD(i->dst()), i->Ib2(), i->Ib()));
 #endif
   
   BX_NEXT_INSTR(i);
@@ -686,7 +708,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::EXTRQ_VdqUq(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 6
   Bit16u ctrl = BX_READ_XMM_REG_LO_WORD(i->src());
 
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), sse_extrq(BX_READ_XMM_REG_LO_QWORD(i->dst()), ctrl >> 8, ctrl));
+  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), xmm_extrq(BX_READ_XMM_REG_LO_QWORD(i->dst()), ctrl >> 8, ctrl));
 #endif
 
   BX_NEXT_INSTR(i);
@@ -697,7 +719,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::INSERTQ_VdqUqIbIb(bxInstruction_c
 #if BX_CPU_LEVEL >= 6
   Bit64u dst = BX_READ_XMM_REG_LO_QWORD(i->dst()), src = BX_READ_XMM_REG_LO_QWORD(i->src());
 
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), sse_insertq(dst, src, i->Ib2(), i->Ib()));
+  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), xmm_insertq(dst, src, i->Ib2(), i->Ib()));
 #endif
 
   BX_NEXT_INSTR(i);
@@ -710,7 +732,7 @@ BX_INSF_TYPE BX_CPP_AttrRegparmN(1) BX_CPU_C::INSERTQ_VdqUdq(bxInstruction_c *i)
 
   Bit64u dst = BX_READ_XMM_REG_LO_QWORD(i->dst());
 
-  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), sse_insertq(dst, src.xmm64u(0), src.xmmubyte(9), src.xmmubyte(8)));
+  BX_WRITE_XMM_REG_LO_QWORD(i->dst(), xmm_insertq(dst, src.xmm64u(0), src.xmmubyte(9), src.xmmubyte(8)));
 #endif
 
   BX_NEXT_INSTR(i);
