@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: stack.cc 11580 2013-01-19 20:45:03Z sshwarts $
+// $Id: stack.cc 12667 2015-02-22 21:26:26Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2012-2013 Stanislav Shwartsman
+//   Copyright (c) 2012-2015 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -26,18 +26,22 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#include "cpustats.h"
+
 void BX_CPP_AttrRegparmN(2) BX_CPU_C::stackPrefetch(bx_address offset, unsigned len)
 {
   bx_address laddr;
   unsigned pageOffset;
 
+  INC_STACK_PREFETCH_STAT(stackPrefetch);
+
   BX_CPU_THIS_PTR espHostPtr = 0; // initialize with NULL pointer
   BX_CPU_THIS_PTR espPageWindowSize = 0;
 
   len--;
 
 #if BX_SUPPORT_X86_64
-  if (long64_mode()) {
+  if (long64_mode() || (BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.valid & SegAccessWOK4G)) {
     laddr = offset;
     pageOffset = PAGE_OFFSET(offset);
 
@@ -51,7 +55,7 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::stackPrefetch(bx_address offset, unsigned
   else
 #endif
   {
-    laddr = get_laddr32(BX_SEG_REG_SS, offset);
+    laddr = get_laddr32(BX_SEG_REG_SS, (Bit32u) offset);
     pageOffset = PAGE_OFFSET(laddr);
     if (pageOffset + len >= 4096) // don't care for page split accesses
       return;
@@ -113,6 +117,9 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::stackPrefetch(bx_address offset, unsigned
       BX_CPU_THIS_PTR espPageBias = (bx_address) pageOffset - offset;
       BX_CPU_THIS_PTR pAddrStackPage = tlbEntry->ppf;
       BX_CPU_THIS_PTR espHostPtr = (Bit8u*) tlbEntry->hostPageAddr;
+#if BX_SUPPORT_MEMTYPE
+      BX_CPU_THIS_PTR espPageMemtype = tlbEntry->get_memtype();
+#endif
     }
   }
 
@@ -134,7 +141,8 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::stack_write_byte(bx_address offset, Bit8u
   if (BX_CPU_THIS_PTR espHostPtr) {
     Bit8u *hostPageAddr = (Bit8u*)(BX_CPU_THIS_PTR espHostPtr + espBiased);
     bx_phy_address pAddr = BX_CPU_THIS_PTR pAddrStackPage + espBiased;
-    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 1, CPL, BX_WRITE, (Bit8u*) &data);
+    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 1,
+                                MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_WRITE, (Bit8u*) &data);
     pageWriteStampTable.decWriteStamp(pAddr, 1);
     *hostPageAddr = data;
   }
@@ -161,7 +169,8 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::stack_write_word(bx_address offset, Bit16u
       exception(BX_AC_EXCEPTION, 0);
     }
 #endif
-    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 2, CPL, BX_WRITE, (Bit8u*) &data);
+    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 2,
+                                MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_WRITE, (Bit8u*) &data);
     pageWriteStampTable.decWriteStamp(pAddr, 2);
     WriteHostWordToLittleEndian(hostPageAddr, data);
   }
@@ -188,7 +197,8 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::stack_write_dword(bx_address offset, Bit32
       exception(BX_AC_EXCEPTION, 0);
     }
 #endif
-    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 4, CPL, BX_WRITE, (Bit8u*) &data);
+    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 4,
+                                MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_WRITE, (Bit8u*) &data);
     pageWriteStampTable.decWriteStamp(pAddr, 4);
     WriteHostDWordToLittleEndian(hostPageAddr, data);
   }
@@ -215,7 +225,8 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::stack_write_qword(bx_address offset, Bit64
       exception(BX_AC_EXCEPTION, 0);
     }
 #endif
-    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 8, CPL, BX_WRITE, (Bit8u*) &data);
+    BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset), pAddr, 8,
+                                MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_WRITE, (Bit8u*) &data);
     pageWriteStampTable.decWriteStamp(pAddr, 8);
     WriteHostQWordToLittleEndian(hostPageAddr, data);
   }
@@ -237,7 +248,8 @@ Bit8u BX_CPP_AttrRegparmN(1) BX_CPU_C::stack_read_byte(bx_address offset)
     Bit8u *hostPageAddr = (Bit8u*)(BX_CPU_THIS_PTR espHostPtr + espBiased), data;
     data = *hostPageAddr;
     BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset),
-        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 1, CPL, BX_READ, (Bit8u*) &data);
+        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 1,
+         MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_READ, (Bit8u*) &data);
     return data;
   }
   else {
@@ -267,7 +279,8 @@ Bit16u BX_CPP_AttrRegparmN(1) BX_CPU_C::stack_read_word(bx_address offset)
 #endif
     ReadHostWordFromLittleEndian(hostPageAddr, data);
     BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset),
-        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 2, CPL, BX_READ, (Bit8u*) &data);
+        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 2,
+         MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_READ, (Bit8u*) &data);
     return data;
   }
   else {
@@ -297,7 +310,8 @@ Bit32u BX_CPP_AttrRegparmN(1) BX_CPU_C::stack_read_dword(bx_address offset)
 #endif
     ReadHostDWordFromLittleEndian(hostPageAddr, data);
     BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset),
-        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 4, CPL, BX_READ, (Bit8u*) &data);
+        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 4,
+         MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_READ, (Bit8u*) &data);
     return data;
   }
   else {
@@ -327,7 +341,8 @@ Bit64u BX_CPP_AttrRegparmN(1) BX_CPU_C::stack_read_qword(bx_address offset)
 #endif
     ReadHostQWordFromLittleEndian(hostPageAddr, data);
     BX_NOTIFY_LIN_MEMORY_ACCESS(get_laddr(BX_SEG_REG_SS, offset),
-        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 8, CPL, BX_READ, (Bit8u*) &data);
+        (BX_CPU_THIS_PTR pAddrStackPage + espBiased), 8,
+         MEMTYPE(BX_CPU_THIS_PTR espPageMemtype), BX_READ, (Bit8u*) &data);
     return data;
   }
   else {
