@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: tasking.cc 11299 2012-07-26 16:03:26Z sshwarts $
+// $Id: tasking.cc 12392 2014-07-03 06:40:42Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -129,7 +129,7 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
 
   // Discard any traps and inhibits for new context; traps will
   // resume upon return.
-  BX_CPU_THIS_PTR debug_trap = 0;
+  BX_CPU_THIS_PTR debug_trap &= ~BX_DEBUG_SINGLE_STEP_BIT;
   BX_CPU_THIS_PTR inhibit_mask = 0;
 
   // STEP 1: The following checks are made before calling task_switch(),
@@ -196,8 +196,8 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
   // used in the task switch are paged in.
   if (BX_CPU_THIS_PTR cr0.get_PG())
   {
-    translate_linear(nbase32, 0, BX_READ);  // old TSS
-    translate_linear(nbase32 + new_TSS_max, 0, BX_READ);
+    translate_linear(BX_TLB_ENTRY_OF(nbase32), nbase32, 0, BX_READ);  // old TSS
+    translate_linear(BX_TLB_ENTRY_OF(nbase32 + new_TSS_max), nbase32 + new_TSS_max, 0, BX_READ);
 
     // ??? Humm, we check the new TSS region with READ above,
     // but sometimes we need to write the link field in that
@@ -208,8 +208,8 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
 
     if (source == BX_TASK_FROM_CALL || source == BX_TASK_FROM_INT)
     {
-      translate_linear(nbase32,     0, BX_WRITE);
-      translate_linear(nbase32 + 1, 0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(nbase32),     nbase32,     0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(nbase32 + 1), nbase32 + 1, 0, BX_WRITE);
     }
   }
 
@@ -222,9 +222,9 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
   if (source == BX_TASK_FROM_JUMP || source == BX_TASK_FROM_IRET) {
     // Bit is cleared
     Bit32u laddr = (Bit32u) BX_CPU_THIS_PTR gdtr.base + (BX_CPU_THIS_PTR tr.selector.index<<3) + 4;
-    access_read_linear(laddr, 4, 0, BX_RW, &temp32);
+    access_read_linear(laddr, 4, 0, BX_RW, 0x0, &temp32);
     temp32 &= ~0x200;
-    access_write_linear(laddr, 4, 0, &temp32);
+    access_write_linear(laddr, 4, 0, 0x0, &temp32);
   }
 
   // STEP 4: If the task switch was initiated with an IRET instruction,
@@ -250,8 +250,10 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
   if (BX_CPU_THIS_PTR tr.cache.type <= 3) {
     // check that we won't page fault while writing
     if (BX_CPU_THIS_PTR cr0.get_PG()) {
-      translate_linear(Bit32u(obase32 + 14), 0, BX_WRITE);
-      translate_linear(Bit32u(obase32 + 41), 0, BX_WRITE);
+      Bit32u start = Bit32u(obase32 + 14), end = Bit32u(obase32 + 41);
+
+      translate_linear(BX_TLB_ENTRY_OF(start), start, 0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(end),   end,   0, BX_WRITE);
     }
 
     system_write_word(Bit32u(obase32 + 14), IP);
@@ -277,8 +279,10 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
   else {
     // check that we won't page fault while writing
     if (BX_CPU_THIS_PTR cr0.get_PG()) {
-      translate_linear(Bit32u(obase32 + 0x20), 0, BX_WRITE);
-      translate_linear(Bit32u(obase32 + 0x5d), 0, BX_WRITE);
+      Bit32u start = Bit32u(obase32 + 0x20), end = Bit32u(obase32 + 0x5d);
+
+      translate_linear(BX_TLB_ENTRY_OF(start), start, 0, BX_WRITE);
+      translate_linear(BX_TLB_ENTRY_OF(end),   end,   0, BX_WRITE);
     }
 
     system_write_dword(Bit32u(obase32 + 0x20), EIP);
@@ -387,9 +391,9 @@ void BX_CPU_C::task_switch(bxInstruction_c *i, bx_selector_t *tss_selector,
   {
     // set the new task's busy bit
     Bit32u laddr = (Bit32u)(BX_CPU_THIS_PTR gdtr.base) + (tss_selector->index<<3) + 4;
-    access_read_linear(laddr, 4, 0, BX_RW, &dword2);
+    access_read_linear(laddr, 4, 0, BX_RW, 0x0, &dword2);
     dword2 |= 0x200;
-    access_write_linear(laddr, 4, 0, &dword2);
+    access_write_linear(laddr, 4, 0, 0x0, &dword2);
   }
 
   //
