@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: extplugin.h 10966 2012-01-10 17:45:18Z vruppert $
+// $Id: extplugin.h 12366 2014-06-08 08:40:08Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 // extplugin.h
@@ -17,11 +17,11 @@
 #ifndef __EXTPLUGIN_H
 #define __EXTPLUGIN_H
 
-#if BX_PLUGINS
+#if BX_PLUGINS && !defined(WIN32)
 #if BX_HAVE_LTDL
 #include <ltdl.h>
-#elif !defined(_MSC_VER)
-#include "ltdl.h"
+#else
+#include "ltdl-bochs.h"
 #endif
 #endif
 
@@ -35,15 +35,15 @@ enum plugintype_t {
 
 #define MAX_ARGC 10
 
-typedef int (*plugin_init_t)(struct _plugin_t *plugin, plugintype_t type, int argc, char *argv[]);
-typedef void (*plugin_fini_t)(void);
+typedef int (CDECL *plugin_init_t)(struct _plugin_t *plugin, plugintype_t type, int argc, char *argv[]);
+typedef void (CDECL *plugin_fini_t)(void);
 
 typedef struct _plugin_t
 {
     plugintype_t type;
     int  initialized;
 #if BX_PLUGINS
-#if defined(_MSC_VER)
+#if defined(WIN32)
     HINSTANCE handle;
 #else
     lt_dlhandle handle;
