@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: gui.h 11622 2013-02-12 21:08:35Z vruppert $
+// $Id: gui.h 12404 2014-07-08 14:30:27Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2012  The Bochs Project
+//  Copyright (C) 2002-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -156,6 +156,7 @@ public:
   static void toggle_mouse_enable(void);
   bx_bool mouse_toggle_check(Bit32u key, bx_bool pressed);
   const char* get_toggle_info(void);
+  bx_bool parse_user_shortcut(const char *val);
 #if BX_DEBUGGER && BX_DEBUGGER_GUI
   void init_debug_dialog(void);
   void close_debug_dialog(void);
@@ -244,6 +245,9 @@ protected:
   Bit8u toggle_method;
   Bit32u toggle_keystate;
   char mouse_toggle_text[20];
+  // userbutton shortcut
+  Bit32u user_shortcut[4];
+  int user_shortcut_len;
   // gui dialog capabilities
   Bit32u dialog_caps;
 };
@@ -464,11 +468,11 @@ virtual void graphics_tile_update_in_place(unsigned x, unsigned y,          \
 //   static bx_sdl_gui_c *theGui;
 
 #define IMPLEMENT_GUI_PLUGIN_CODE(gui_name)                           \
-  int lib##gui_name##_LTX_plugin_init(plugin_t *plugin,               \
+  int CDECL lib##gui_name##_LTX_plugin_init(plugin_t *plugin,         \
           plugintype_t type, int argc, char *argv[]) {                \
     genlog->info("installing %s module as the Bochs GUI", #gui_name); \
     theGui = new bx_##gui_name##_gui_c ();                            \
     bx_gui = theGui;                                                  \
     return(0); /* Success */                                          \
   }                                                                   \
-  void lib##gui_name##_LTX_plugin_fini(void) { }
+  void CDECL lib##gui_name##_LTX_plugin_fini(void) { }
