@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: macintosh.cc 11501 2012-10-14 18:29:44Z vruppert $
+// $Id: macintosh.cc 12081 2013-12-29 12:56:52Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2012  The Bochs Project
+//  Copyright (C) 2001-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -396,7 +396,7 @@ void CreateWindows(void)
 
 void bx_macintosh_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 {
-  put("MGUI");
+  put("MACGUI");
   InitToolbox();
 
   //SouixWin = FrontWindow();
