@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: paramtree.cc 11634 2013-02-17 08:27:43Z vruppert $
+// $Id: paramtree.cc 12684 2015-03-13 21:28:40Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2010-2013  The Bochs Project
+//  Copyright (C) 2010-2015  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -50,6 +50,7 @@ bx_param_c::bx_param_c(Bit32u id, const char *param_name, const char *param_desc
   this->long_text_format = default_text_format;
   this->runtime_param = 0;
   this->enabled = 1;
+  this->options = 0;
   // dependent_list must be initialized before the set(),
   // because set calls update_dependents().
   dependent_list = NULL;
@@ -72,6 +73,7 @@ bx_param_c::bx_param_c(Bit32u id, const char *param_name, const char *param_labe
   this->long_text_format = default_text_format;
   this->runtime_param = 0;
   this->enabled = 1;
+  this->options = 0;
   // dependent_list must be initialized before the set(),
   // because set calls update_dependents().
   dependent_list = NULL;
@@ -293,7 +295,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p64bit = ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x"FMT_LL"x";
+    this->text_format = "0x" FMT_LL "x";
   }
 }
 
@@ -312,7 +314,7 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   val.p64bit = (Bit64s*) ptr_to_real_val;
   if (base == BASE_HEX) {
     this->base = base;
-    this->text_format = "0x"FMT_LL"x";
+    this->text_format = "0x" FMT_LL "x";
   }
 }
 
@@ -431,6 +433,19 @@ bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
   }
 }
 
+// Double (floating point)
+bx_shadow_num_c::bx_shadow_num_c(bx_param_c *parent,
+    const char *name,
+    double *ptr_to_real_val)
+: bx_param_num_c(parent, name, NULL, NULL, BX_MIN_BIT64U, BX_MAX_BIT64U, 0, 1)
+{
+  this->varsize = 64;
+  this->lowbit = 0;
+  this->mask = BX_MAX_BIT64U;
+  val.pdouble = ptr_to_real_val;
+  this->base = BASE_DOUBLE;
+}
+
 Bit64s bx_shadow_num_c::get64()
 {
   Bit64u current = 0;
@@ -538,9 +553,9 @@ Bit64s bx_shadow_bool_c::get64()
 void bx_shadow_bool_c::set(Bit64s newval)
 {
   // only change the bitnum bit
-  Bit64s tmp = (newval&1) << bitnum;
-  *(val.pbool) &= ~tmp;
-  *(val.pbool) |= tmp;
+  Bit64s mask = 1 << bitnum;
+  *(val.pbool) &= ~mask;
+  *(val.pbool) |= ((newval & 1) << bitnum);
   if (handler) {
     // the handler can override the new value and/or perform some side effect
     (*handler)(this, 1, newval&1);
@@ -809,7 +824,11 @@ void bx_param_string_c::set_initial_val(const char *buf)
 
 bx_bool bx_param_string_c::isempty()
 {
-  return ((strlen(val) == 0) || !strcmp(val, "none"));
+  if (options & RAW_BYTES) {
+    return (memcmp(val, initial_val, maxsize) == 0);
+  } else {
+    return ((strlen(val) == 0) || !strcmp(val, "none"));
+  }
 }
 
 int bx_param_string_c::sprint(char *buf, int len, bx_bool dquotes)
@@ -1023,6 +1042,9 @@ void bx_list_c::add(bx_param_c *param)
       temp = temp->next;
     temp->next = item;
   }
+  if (runtime_param) {
+    param->set_runtime_param(1);
+  }
   size++;
 }
 
@@ -1101,3 +1123,15 @@ void bx_list_c::remove(const char *name)
     }
   }
 }
+
+void bx_list_c::set_runtime_param(int val)
+{
+  bx_listitem_t *item;
+
+  runtime_param = val;
+  if (runtime_param) {
+    for (item = list; item; item = item->next) {
+      item->param->set_runtime_param(1);
+    }
+  }
+}
