@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: textconfig.cc 11127 2012-04-06 13:15:27Z vruppert $
+// $Id: textconfig.cc 12594 2015-01-07 16:17:40Z sshwarts $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2009  The Bochs Project
+//  Copyright (C) 2002-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -318,20 +318,16 @@ static const char *runtime_menu_prompt =
 "---------------------\n"
 "1. Floppy disk 0: %s\n"
 "2. Floppy disk 1: %s\n"
-"3. 1st CDROM: %s\n"
-"4. 2nd CDROM: %s\n"
-"5. 3rd CDROM: %s\n"
-"6. 4th CDROM: %s\n"
-"7. (not implemented)\n"
-"8. Log options for all devices\n"
-"9. Log options for individual devices\n"
-"10. Instruction tracing: off (doesn't exist yet)\n"
-"11. USB runtime options\n"
-"12. Misc runtime options\n"
-"13. Continue simulation\n"
-"14. Quit now\n"
+"3. CDROM runtime options\n"
+"4. Log options for all devices\n"
+"5. Log options for individual devices\n"
+"6. USB runtime options\n"
+"7. Misc runtime options\n"
+"8. Save configuration\n"
+"9. Continue simulation\n"
+"10. Quit now\n"
 "\n"
-"Please choose one:  [13] ";
+"Please choose one:  [9] ";
 
 static const char *plugin_ctrl_prompt =
 "\n-----------------------\n"
@@ -353,7 +349,6 @@ static const char *plugin_ctrl_prompt =
 void build_runtime_options_prompt(const char *format, char *buf, int size)
 {
   bx_list_c *floppyop;
-  bx_list_c *cdromop = NULL;
   char pname[80];
   char buffer[6][128];
 
@@ -365,24 +360,12 @@ void build_runtime_options_prompt(const char *format, char *buf, int size)
     else {
       sprintf(buffer[i], "%s, size=%s, %s", SIM->get_param_string("path", floppyop)->getptr(),
         SIM->get_param_enum("type", floppyop)->get_selected(),
-        SIM->get_param_bool("status", floppyop)->get() ? "inserted":"ejected");
+        SIM->get_param_enum("status", floppyop)->get_selected());
       if (!SIM->get_param_string("path", floppyop)->getptr()[0]) strcpy(buffer[i], "none");
     }
   }
 
-  // 4 cdroms supported at run time
-  int device;
-  for (Bit8u cdrom=0; cdrom<4; cdrom++) {
-    if (!SIM->get_cdrom_options(cdrom, &cdromop, &device) || !SIM->get_param_bool("present", cdromop)->get())
-      sprintf(buffer[2+cdrom], "(not present)");
-    else
-      sprintf(buffer[2+cdrom], "(%s on ata%d) %s, %s",
-        device&1?"slave":"master", device/2, SIM->get_param_string("path", cdromop)->getptr(),
-        (SIM->get_param_bool("status", cdromop)->get()) ? "inserted" : "ejected");
-    }
-
-  snprintf(buf, size, format, buffer[0], buffer[1], buffer[2],
-           buffer[3], buffer[4], buffer[5]);
+  snprintf(buf, size, format, buffer[0], buffer[1]);
 }
 
 int do_menu(const char *pname)
@@ -507,8 +490,6 @@ int bx_config_interface(int menu)
         break;
       case BX_CI_RUNTIME:
         {
-          bx_list_c *cdromop = NULL;
-          char pname[80];
           char prompt[1024];
           build_runtime_options_prompt(runtime_menu_prompt, prompt, 1024);
           if (ask_uint(prompt, "", 1, BX_CI_RT_QUIT, BX_CI_RT_CONT, &choice, 10) < 0) return -1;
@@ -519,26 +500,14 @@ int bx_config_interface(int menu)
             case BX_CI_RT_FLOPPYB:
               if (SIM->get_param_enum(BXPN_FLOPPYB_DEVTYPE)->get() != BX_FDD_NONE) do_menu(BXPN_FLOPPYB);
               break;
-            case BX_CI_RT_CDROM1:
-            case BX_CI_RT_CDROM2:
-            case BX_CI_RT_CDROM3:
-            case BX_CI_RT_CDROM4:
-              int device;
-              if (SIM->get_cdrom_options(choice - BX_CI_RT_CDROM1, &cdromop, &device) && SIM->get_param_bool("present", cdromop)->get()) {
-                cdromop->get_param_path(pname, 80);
-                do_menu(pname);
-              }
-              break;
-            case BX_CI_RT_IPS:
-              // not implemented yet because I would have to mess with
-              // resetting timers and pits and everything on the fly.
-              // askparam(BXPN_IPS);
-              break;
+            case BX_CI_RT_CDROM: do_menu(BXPN_MENU_RUNTIME_CDROM); break;
             case BX_CI_RT_LOGOPTS1: bx_log_options(0); break;
             case BX_CI_RT_LOGOPTS2: bx_log_options(1); break;
-            case BX_CI_RT_INST_TR: NOT_IMPLEMENTED(choice); break;
             case BX_CI_RT_USB: do_menu(BXPN_MENU_RUNTIME_USB); break;
             case BX_CI_RT_MISC: do_menu(BXPN_MENU_RUNTIME_MISC); break;
+            case BX_CI_RT_SAVE_CFG:
+              bx_write_rc(NULL);
+              break;
             case BX_CI_RT_CONT:
               SIM->update_runtime_options();
               fprintf(stderr, "Continuing simulation\n");
@@ -573,7 +542,7 @@ static void bx_print_log_action_table()
   fprintf(stderr, "----  ---------  ---------  ---------  ----------  ----------\n");
   int i, j, imax=SIM->get_n_log_modules();
   for (i=0; i<imax; i++) {
-    if (strcmp(SIM->get_prefix(i), "[     ]")) {
+    if (strcmp(SIM->get_prefix(i), BX_NULL_PREFIX)) {
       fprintf(stderr, "%3d.  %s ", i, SIM->get_prefix(i));
       for (j=0; j<SIM->get_max_log_level(); j++) {
         fprintf(stderr, "%10s ", SIM->get_action_name(SIM->get_log_action(i, j)));
@@ -589,7 +558,6 @@ static int log_level_n_choices_normal = 4;
 
 void bx_log_options(int individual)
 {
-  SIM->apply_log_actions_by_device(); // settings from bochsrc
   if (individual) {
     int done = 0;
     while (!done) {
@@ -607,7 +575,14 @@ void bx_log_options(int individual)
         // don't show the no change choice (choices=3)
         if (ask_menu(prompt, "", log_level_n_choices_normal, log_level_choices, default_action, &action)<0)
           return;
-        SIM->set_log_action(id, level, action);
+        // the exclude expression allows some choices not being available if they
+        // don't make any sense.  For example, it would be stupid to ignore a panic.
+        if (!BX_LOG_OPTS_EXCLUDE(level, action)) {
+          SIM->set_log_action(id, level, action);
+        } else {
+          fprintf(stderr, "Event type '%s' does not support log action '%s'.\n",
+                  SIM->get_log_level_name(level), log_level_choices[action]);
+        }
       }
     }
   } else {
@@ -615,14 +590,19 @@ void bx_log_options(int individual)
     bx_print_log_action_table();
     for (int level=0; level<SIM->get_max_log_level(); level++) {
       char prompt[1024];
-      int action, default_action = 3;  // default to no change
+      int action, default_action = 4;  // default to no change
       sprintf(prompt, "Enter action for %s event on all devices: [no change] ", SIM->get_log_level_name(level));
       // do show the no change choice (choices=4)
       if (ask_menu(prompt, "", log_level_n_choices_normal+1, log_level_choices, default_action, &action)<0)
         return;
-      if (action < 3) {
-        SIM->set_default_log_action(level, action);
-        SIM->set_log_action(-1, level, action);
+      if (action < log_level_n_choices_normal) {
+        if  (!BX_LOG_OPTS_EXCLUDE(level, action)) {
+          SIM->set_default_log_action(level, action);
+          SIM->set_log_action(-1, level, action);
+        } else {
+          fprintf(stderr, "Event type '%s' does not support log action '%s'.\n",
+                  SIM->get_log_level_name(level), log_level_choices[action]);
+        }
       }
     }
   }
@@ -651,6 +631,7 @@ int bx_write_rc(char *rc)
       strcpy(oldrc, "none");
   } else {
     strncpy(oldrc, rc, CI_PATH_LENGTH);
+    oldrc[sizeof(oldrc) - 1] = '\0';
   }
   while (1) {
     if (ask_string("Save configuration to what file?  To cancel, type 'none'.\n[%s] ", oldrc, newrc) < 0) return -1;
@@ -772,11 +753,12 @@ ask:
     return event;
   case BX_ASYNC_EVT_REFRESH:
   case BX_ASYNC_EVT_DBG_MSG:
+  case BX_ASYNC_EVT_LOG_MSG:
     // The text mode interface does not use these events, so just ignore
     // them.
     return event;
   default:
-    fprintf(stderr, "Control panel: notify callback called with event type %04x\n", event->type);
+    fprintf(stderr, "textconfig: notify callback called with event type %04x\n", event->type);
     return event;
   }
   assert(0); // switch statement should return
@@ -839,24 +821,9 @@ void bx_param_enum_c::text_print(FILE *fp)
 
 void bx_param_string_c::text_print(FILE *fp)
 {
-  char *value = getptr();
-  int opts = options;
-  if (opts & RAW_BYTES) {
-    char buffer[1024];
-    buffer[0] = 0;
-    char sep_string[2];
-    sep_string[0] = separator;
-    sep_string[1] = 0;
-    for (int i=0; i<maxsize; i++) {
-      char eachbyte[16];
-      sprintf(eachbyte, "%s%02x", (i>0)?sep_string : "", (unsigned int)0xff&val[i]);
-      strncat(buffer, eachbyte, sizeof(eachbyte));
-    }
-    if (strlen(buffer) > sizeof(buffer)-4) {
-      assert(0); // raw byte print buffer is probably overflowing. increase the max or make it dynamic
-    }
-    value = buffer;
-  }
+  char value[1024];
+
+  this->sprint(value, 1024, 0);
   if (get_format()) {
     fprintf(fp, get_format(), value);
   } else {
@@ -999,7 +966,8 @@ int bx_param_string_c::text_ask(FILE *fpin, FILE *fpout)
     int opts = options;
     char buffer2[1024];
     strcpy(buffer2, buffer);
-    if (status == 1 && opts & RAW_BYTES) {
+    if (opts & RAW_BYTES) {
+      if (status == 0) return 0;
       // copy raw hex into buffer
       status = parse_raw_bytes(buffer, buffer2, maxsize, separator);
       if (status < 0) {
