@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: textconfig.h 10209 2011-02-24 22:05:47Z sshwarts $
+// $Id: textconfig.h 12050 2013-12-20 17:53:00Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2009  The Bochs Project
+//  Copyright (C) 2009-2013  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -26,11 +26,6 @@ enum {
   BX_CI_INIT,
   BX_CI_START_MENU,
   BX_CI_START_OPTS,
-  BX_CI_START_OPTS_MEM,
-  BX_CI_START_OPTS_INTERFACE,
-  BX_CI_START_OPTS_DISK,
-  BX_CI_START_OPTS_SOUND,
-  BX_CI_START_OPTS_MISC,
   BX_CI_START_SIMULATION,
   BX_CI_RUNTIME,
   BX_CI_N_MENUS
@@ -39,16 +34,12 @@ enum {
 enum {
   BX_CI_RT_FLOPPYA = 1,
   BX_CI_RT_FLOPPYB,
-  BX_CI_RT_CDROM1,
-  BX_CI_RT_CDROM2,
-  BX_CI_RT_CDROM3,
-  BX_CI_RT_CDROM4,
-  BX_CI_RT_IPS,
+  BX_CI_RT_CDROM,
   BX_CI_RT_LOGOPTS1,
   BX_CI_RT_LOGOPTS2,
-  BX_CI_RT_INST_TR,
   BX_CI_RT_USB,
   BX_CI_RT_MISC,
+  BX_CI_RT_SAVE_CFG,
   BX_CI_RT_CONT,
   BX_CI_RT_QUIT
 };
