@@ -1,8 +1,24 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wx.cc 11370 2012-08-26 12:32:10Z vruppert $
+// $Id: wx.cc 12714 2015-04-11 10:21:03Z vruppert $
 /////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2012  The Bochs Project
+//  Copyright (C) 2002-2015  The Bochs Project
+//
+//  This library is free software; you can redistribute it and/or
+//  modify it under the terms of the GNU Lesser General Public
+//  License as published by the Free Software Foundation; either
+//  version 2 of the License, or (at your option) any later version.
+//
+//  This library is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//  Lesser General Public License for more details.
+//
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+//
+/////////////////////////////////////////////////////////////////
 //
 // wxWidgets VGA display for Bochs.  wx.cc implements a custom
 // wxPanel called a MyPanel, which has methods to display
@@ -10,22 +26,7 @@
 // is instantiated within a MyFrame created by wxmain.cc, but
 // this is not a requirement.
 //
-// The separation between wxmain.cc and wx.cc is as follows:
-// - wxmain.cc implements a Bochs configuration interface (CI),
-//   which is the wxWidgets equivalent of textconfig.cc. wxmain creates
-//   a frame with several menus and a toolbar, and allows the user to
-//   choose the machine configuration and start the simulation.  Note
-//   that wxmain.cc does NOT include bochs.h.  All interactions
-//   between the CI and the simulator are through the siminterface
-//   object.
-// - wx.cc implements a VGA display screen using wxWidgets.  It is
-//   is the wxWidgets equivalent of x.cc, win32.cc, macos.cc, etc.
-//   wx.cc includes bochs.h and has access to all Bochs devices.
-//   The VGA panel accepts only paint, key, and mouse events.  As it
-//   receives events, it builds BxEvents and places them into a
-//   thread-safe BxEvent queue.  The simulation thread periodically
-//   processes events from the BxEvent queue (bx_wx_gui_c::handle_events)
-//   and notifies the appropriate emulated I/O device.
+// See the top of the source file wxmain.cc for more information.
 //
 /////////////////////////////////////////////////////////////////
 
@@ -50,6 +51,7 @@
 #endif
 #include <wx/image.h>
 #include <wx/clipbrd.h>
+#include <wx/display.h>
 
 //#include "gui/icon_bochs.h"
 #include "osdep.h"
@@ -64,10 +66,11 @@
 //////////////////////////////////////////////////////////////
 class bx_wx_gui_c : public bx_gui_c {
 public:
-  bx_wx_gui_c (void) {}
+  bx_wx_gui_c(void) {}
   DECLARE_GUI_VIRTUAL_METHODS()
   DECLARE_GUI_NEW_VIRTUAL_METHODS()
-  void statusbar_setitem_specific(int element, bx_bool active, bx_bool w);
+  virtual void statusbar_setitem_specific(int element, bx_bool active, bx_bool w);
+  virtual void get_capabilities(Bit16u *xres, Bit16u *yres, Bit16u *bpp);
   virtual void set_mouse_mode_absxy(bx_bool mode);
 #if BX_SHOW_IPS
   void show_ips(Bit32u ips_count);
@@ -78,8 +81,9 @@ public:
 // plugin code
 static bx_wx_gui_c *theGui = NULL;
 
-void MyPanel::OnPluginInit () {
-  theGui = new bx_wx_gui_c ();
+void MyPanel::OnPluginInit()
+{
+  theGui = new bx_wx_gui_c();
   bx_gui = theGui;
 }
 
@@ -91,6 +95,7 @@ void MyPanel::OnPluginInit () {
 // The bits to be displayed on the VGA screen are stored in wxScreen.
 // wxScreen is an array (size=width*height*3) of RGB values.  Each
 // pixel is represented by three bytes, one for red, green, and blue.
+static wxRect wx_maxres;
 static char *wxScreen = NULL;
 wxCriticalSection wxScreen_lock;
 static long wxScreenX = 0;
@@ -106,7 +111,7 @@ static unsigned long wxFontY = 0;
 static unsigned int text_rows=25, text_cols=80;
 static Bit8u h_panning = 0, v_panning = 0;
 static Bit16u line_compare = 1023;
-static unsigned vga_bpp=8;
+static unsigned disp_bpp = 8;
 static struct {
   unsigned char red;
   unsigned char green;
@@ -114,7 +119,7 @@ static struct {
 } wxBochsPalette[256];
 wxCriticalSection event_thread_lock;
 BxEvent event_queue[MAX_EVENTS];
-unsigned long num_events = 0;
+unsigned long num_events;
 static bx_bool mouse_captured = 0;
 static bx_bool wx_hide_ips = 0;
 #if defined (wxHAS_RAW_KEY_CODES) && defined(__WXGTK__)
@@ -136,6 +141,7 @@ BEGIN_EVENT_TABLE(MyPanel, wxPanel)
   EVT_TIMER(-1, MyPanel::OnTimer)
   EVT_PAINT(MyPanel::OnPaint)
   EVT_MOUSE_EVENTS(MyPanel::OnMouse)
+  EVT_KILL_FOCUS(MyPanel::OnKillFocus)
 END_EVENT_TABLE()
 
 MyPanel::MyPanel(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
@@ -150,7 +156,7 @@ MyPanel::MyPanel(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSi
   thePanel = this;
 }
 
-MyPanel::~MyPanel ()
+MyPanel::~MyPanel()
 {
   delete blankCursor;
   thePanel = NULL;
@@ -199,22 +205,23 @@ void MyPanel::OnPaint(wxPaintEvent& WXUNUSED(event))
   needRefresh = false;
 }
 
-void MyPanel::ToggleMouse (bool fromToolbar)
+void MyPanel::ToggleMouse(bool fromToolbar)
 {
   static bool first_enable = true;
   bx_param_bool_c *enable = SIM->get_param_bool(BXPN_MOUSE_ENABLED);
   bool en = ! enable->get();
-  bool is_main_thread = wxThread::IsMain ();
-  bool needmutex = !is_main_thread && SIM->is_sim_thread ();
+  bool is_main_thread = wxThread::IsMain();
+  bool needmutex = !is_main_thread && SIM->is_sim_thread();
   if (needmutex) wxMutexGuiEnter();
   if (fromToolbar && first_enable && en) {
     // only show this help if you click on the toolbar.  If they already
     // know the shortcut, don't annoy them with the message.
-    wxString msg = wxT(
+    wxString msg;
+    msg.Printf(wxT(
       "You have enabled the mouse in Bochs, so now your mouse actions will\n"
       "be sent into the simulator.  The usual mouse cursor will be trapped\n"
-      "inside the Bochs window until you press a CTRL key + the middle button\n"
-      "to turn mouse capture off.");
+      "inside the Bochs window until you press %s\n"
+      "to turn mouse capture off."), theGui->get_toggle_info());
     wxMessageBox(msg, wxT("Mouse Capture Enabled"), wxOK | wxICON_INFORMATION);
     first_enable = false;
   }
@@ -242,18 +249,23 @@ void MyPanel::ToggleMouse (bool fromToolbar)
 void MyPanel::OnMouse(wxMouseEvent& event)
 {
   long x,y;
-  event.GetPosition (&x, &y);
+  int wheel = 0;
+
+  if (theFrame->GetSimThread() == NULL)
+    return;
+
+  event.GetPosition(&x, &y);
   IFDBG_MOUSE (
-    if (event.IsButton ()) {
-      wxLogDebug (wxT ("mouse button event at %d,%d", x, y));
-    } else if (event.Entering ()) {
-      wxLogDebug (wxT ("mouse entering at %d,%d", x, y));
-    } else if (event.Leaving ()) {
-      wxLogDebug (wxT ("mouse leaving at %d,%d", x, y));
-    } else if (event.Moving() || event.Dragging ()) {
-      wxLogDebug (wxT ("mouse moved to %d,%d", x, y));
+    if (event.IsButton()) {
+      wxLogDebug(wxT("mouse button event at %d,%d", x, y));
+    } else if (event.Entering()) {
+      wxLogDebug(wxT("mouse entering at %d,%d", x, y));
+    } else if (event.Leaving()) {
+      wxLogDebug(wxT("mouse leaving at %d,%d", x, y));
+    } else if (event.Moving() || event.Dragging()) {
+      wxLogDebug(wxT("mouse moved to %d,%d", x, y));
     } else {
-      wxLogDebug (wxT ("other mouse event at %d,%d", x, y));
+      wxLogDebug(wxT("other mouse event at %d,%d", x, y));
     }
   )
 
@@ -274,7 +286,10 @@ void MyPanel::OnMouse(wxMouseEvent& event)
   buttons  = event.LeftIsDown()  ? 1 : 0;
   buttons |= event.RightIsDown() ? 2 : 0;
   buttons |= event.MiddleIsDown() ? 4 : 0;
-  if (x==mouseSavedX && y==mouseSavedY && !event.IsButton ()) {
+  if (event.GetWheelRotation() != 0) {
+    wheel = event.GetWheelRotation() / event.GetWheelDelta();
+  }
+  if ((x == mouseSavedX) && (y == mouseSavedY) && !event.IsButton() && (wheel == 0)) {
     // nothing happened.  This could have been generated by the WarpPointer.
     return;
   } else {
@@ -291,7 +306,7 @@ void MyPanel::OnMouse(wxMouseEvent& event)
         event_queue[num_events].u.mouse.dx = dx;
         event_queue[num_events].u.mouse.dy = -dy;
       }
-      // TODO: handle mouse wheel
+      event_queue[num_events].u.mouse.dz = wheel;
       event_queue[num_events].u.mouse.buttons = buttons;
       num_events++;
       mouseSavedX = x;
@@ -312,18 +327,30 @@ void MyPanel::OnMouse(wxMouseEvent& event)
   // will move the cursor to (mouseSavedX, mouseSavedY).
 }
 
-void MyPanel::MyRefresh ()
+void MyPanel::OnKillFocus(wxFocusEvent& event)
 {
-  IFDBG_VGA (wxLogDebug (wxT ("set needRefresh=true")));
+  // Send the request to release all keys as a key event
+  wxCriticalSectionLocker lock(event_thread_lock);
+  if (num_events < MAX_EVENTS) {
+    event_queue[num_events].type = BX_ASYNC_EVT_KEY;
+    event_queue[num_events].u.key.bx_key = BX_KEY_NBKEYS | BX_KEY_RELEASED;
+    event_queue[num_events].u.key.raw_scancode = false;
+    num_events++;
+  }
+}
+
+void MyPanel::MyRefresh()
+{
+  IFDBG_VGA(wxLogDebug(wxT("set needRefresh=true")));
   needRefresh = true;
 }
 
 void MyPanel::OnKeyDown(wxKeyEvent& event)
 {
   wxCriticalSectionLocker lock(event_thread_lock);
-  if(num_events < MAX_EVENTS) {
+  if (num_events < MAX_EVENTS) {
     event_queue[num_events].type = BX_ASYNC_EVT_KEY;
-    fillBxKeyEvent (event, event_queue[num_events].u.key, false);
+    fillBxKeyEvent(event, event_queue[num_events].u.key, false);
     num_events++;
   }
 }
@@ -332,9 +359,9 @@ void MyPanel::OnKeyDown(wxKeyEvent& event)
 void MyPanel::OnKeyUp(wxKeyEvent& event)
 {
   wxCriticalSectionLocker lock(event_thread_lock);
-  if(num_events < MAX_EVENTS) {
+  if (num_events < MAX_EVENTS) {
     event_queue[num_events].type = BX_ASYNC_EVT_KEY;
-    fillBxKeyEvent (event, event_queue[num_events].u.key, true);
+    fillBxKeyEvent(event, event_queue[num_events].u.key, true);
     num_events++;
   }
 }
@@ -778,6 +805,9 @@ bx_bool MyPanel::fillBxKeyEvent(wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool rele
   Bit32u key = wxev.m_keyCode;
   bx_bool mouse_toggle = 0;
 
+  if (theFrame->GetSimThread() == NULL)
+    return false;
+
   if (key == WXK_CONTROL) {
     mouse_toggle = bx_gui->mouse_toggle_check(BX_MT_KEY_CTRL, !release);
   } else if (key == WXK_ALT) {
@@ -823,8 +853,8 @@ bx_bool MyPanel::fillBxKeyEvent(wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool rele
     case WXK_ALT:                  bx_key = BX_KEY_ALT_L;        break;
     case WXK_MENU:                 bx_key = BX_KEY_MENU;         break;
     case WXK_PAUSE:                bx_key = BX_KEY_PAUSE;        break;
-    case WXK_PRIOR:                bx_key = BX_KEY_PAGE_UP;      break;
-    case WXK_NEXT:                 bx_key = BX_KEY_PAGE_DOWN;    break;
+    case WXK_PAGEUP:               bx_key = BX_KEY_PAGE_UP;      break;
+    case WXK_PAGEDOWN:             bx_key = BX_KEY_PAGE_DOWN;    break;
     case WXK_END:                  bx_key = BX_KEY_END;          break;
     case WXK_HOME:                 bx_key = BX_KEY_HOME;         break;
     case WXK_LEFT:                 bx_key = BX_KEY_LEFT;         break;
@@ -868,14 +898,8 @@ bx_bool MyPanel::fillBxKeyEvent(wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool rele
     case WXK_NUMPAD_UP:            bx_key = BX_KEY_KP_UP;        break;
     case WXK_NUMPAD_RIGHT:         bx_key = BX_KEY_KP_RIGHT;     break;
     case WXK_NUMPAD_DOWN:          bx_key = BX_KEY_KP_DOWN;      break;
-    case WXK_NUMPAD_PRIOR:         bx_key = BX_KEY_KP_PAGE_UP;   break;
-#if WXK_NUMPAD_PAGEUP != WXK_NUMPAD_PRIOR
     case WXK_NUMPAD_PAGEUP:        bx_key = BX_KEY_KP_PAGE_UP;   break;
-#endif
-    case WXK_NUMPAD_NEXT:          bx_key = BX_KEY_KP_PAGE_DOWN; break;
-#if WXK_NUMPAD_PAGEDOWN != WXK_NUMPAD_NEXT
     case WXK_NUMPAD_PAGEDOWN:      bx_key = BX_KEY_KP_PAGE_DOWN; break;
-#endif
     case WXK_NUMPAD_END:           bx_key = BX_KEY_KP_END;       break;
     case WXK_NUMPAD_BEGIN:         bx_key = BX_KEY_KP_HOME;      break;
     case WXK_NUMPAD_INSERT:        bx_key = BX_KEY_KP_INSERT;    break;
@@ -917,6 +941,20 @@ bx_bool MyPanel::fillBxKeyEvent(wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool rele
   return true;
 }
 
+#if defined(WIN32) && BX_DEBUGGER && BX_DEBUGGER_GUI
+DWORD WINAPI DebugGuiThread(LPVOID)
+{
+  MSG msg;
+
+  bx_gui->init_debug_dialog();
+  while (GetMessage(&msg, NULL, 0, 0)) {
+    TranslateMessage(&msg);
+    DispatchMessage(&msg);
+  }
+  return 0;
+}
+#endif
+
 //////////////////////////////////////////////////////////////
 // fill in methods of bx_gui
 //////////////////////////////////////////////////////////////
@@ -925,8 +963,11 @@ void bx_wx_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 {
   int b,i,j;
   unsigned char fc, vc;
+  wxDisplay display;
 
   put("WX");
+  wx_maxres = display.GetGeometry();
+  info("Current display dimensions %d x %d", wx_maxres.GetWidth(), wx_maxres.GetHeight());
   if (SIM->get_param_bool(BXPN_PRIVATE_COLORMAP)->get()) {
     BX_INFO(("private_colormap option ignored."));
   }
@@ -988,6 +1029,25 @@ void bx_wx_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
     }
   }
 
+#if BX_DEBUGGER && BX_DEBUGGER_GUI
+#ifdef WIN32
+  // on Windows the debugger gui must run in a separate thread
+  DWORD threadID;
+  CreateThread(NULL, 0, DebugGuiThread, NULL, 0, &threadID);
+#else
+  wxMutexGuiEnter();
+  init_debug_dialog();
+  wxMutexGuiLeave();
+#endif
+#endif
+
+  wxString msg;
+  msg.Printf(wxT("Enable mouse capture\nThere is also a shortcut for this: %s."),
+             theGui->get_toggle_info());
+  theFrame->SetToolBarHelp(ID_Toolbar_Mouse_en, msg);
+
+  num_events = 0;
+
   new_gfx_api = 1;
   dialog_caps = BX_GUI_DLG_USER | BX_GUI_DLG_SNAPSHOT | BX_GUI_DLG_SAVE_RESTORE;
 }
@@ -1000,113 +1060,128 @@ void bx_wx_gui_c::specific_init(int argc, char **argv, unsigned headerbar_y)
 
 void bx_wx_gui_c::handle_events(void)
 {
-  wxCriticalSectionLocker lock(event_thread_lock);
-  Bit32u bx_key = 0;
-  for(unsigned int i = 0; i < num_events; i++) {
-    switch(event_queue[i].type) {
-      case BX_ASYNC_EVT_TOOLBAR:
-        switch (event_queue[i].u.toolbar.button) {
-          case BX_TOOLBAR_FLOPPYA: floppyA_handler(); break;
-          case BX_TOOLBAR_FLOPPYB: floppyB_handler(); break;
-          case BX_TOOLBAR_CDROM1: cdrom1_handler(); break;
-          case BX_TOOLBAR_RESET: reset_handler(); break;
-          case BX_TOOLBAR_POWER: power_handler(); break;
-          case BX_TOOLBAR_SAVE_RESTORE: save_restore_handler(); break;
-          case BX_TOOLBAR_COPY: copy_handler(); break;
-          case BX_TOOLBAR_PASTE: paste_handler(); break;
-          case BX_TOOLBAR_SNAPSHOT: snapshot_handler(); break;
-          case BX_TOOLBAR_CONFIG: config_handler(); break;
-          case BX_TOOLBAR_MOUSE_EN: thePanel->ToggleMouse(true); break;
-          case BX_TOOLBAR_USER: userbutton_handler(); break;
-          default:
-            wxLogDebug (wxT ("unknown toolbar id %d"), event_queue[i].u.toolbar.button);
-        }
-        break;
-      case BX_ASYNC_EVT_KEY:
-        bx_key = event_queue[i].u.key.bx_key;
-        if (event_queue[i].u.key.raw_scancode) {
-          // event contains raw scancodes: convert to BX_KEY values first
-          bx_bool released = ((bx_key & 0x80) > 0);
-          if (bx_key & 0xFF00) { // for extended keys
-            switch (bx_key & 0x7f) {
-              case 0x1C:
-                bx_key = BX_KEY_KP_ENTER;
-                break;
-              case 0x1D:
-                bx_key = BX_KEY_CTRL_R;
-                break;
-              case 0x35:
-                bx_key = BX_KEY_KP_DIVIDE;
-                break;
-              case 0x38:
-                // This makes the "AltGr" key on European keyboards work
-                DEV_kbd_gen_scancode(BX_KEY_CTRL_L | BX_KEY_RELEASED);
-                bx_key = BX_KEY_ALT_R;
-                break;
-              case 0x45:
-                bx_key = BX_KEY_NUM_LOCK;
-                break;
-              case 0x47:
-                bx_key = BX_KEY_HOME;
-                break;
-              case 0x48:
-                bx_key = BX_KEY_UP;
-                break;
-              case 0x49:
-                bx_key = BX_KEY_PAGE_UP;
-                break;
-              case 0x4B:
-                bx_key = BX_KEY_LEFT;
-                break;
-              case 0x4D:
-                bx_key = BX_KEY_RIGHT;
-                break;
-              case 0x4F:
-                bx_key = BX_KEY_END;
-                break;
-              case 0x50:
-                bx_key = BX_KEY_DOWN;
-                break;
-              case 0x51:
-                bx_key = BX_KEY_PAGE_DOWN;
-                break;
-              case 0x52:
-                bx_key = BX_KEY_INSERT;
-                break;
-              case 0x53:
-                bx_key = BX_KEY_DELETE;
-                break;
-              case 0x5B:
-                bx_key = BX_KEY_WIN_L;
-                break;
-              case 0x5C:
-                bx_key = BX_KEY_WIN_R;
-                break;
-              case 0x5D:
-                bx_key = BX_KEY_MENU;
-                break;
-            }
-          } else {
-            bx_key = wxMSW_to_bx_key[bx_key & 0x7f];
+  unsigned tb_button = 0;
+  { // critical section start
+    wxCriticalSectionLocker lock(event_thread_lock);
+    Bit32u bx_key = 0;
+    for(unsigned int i = 0; i < num_events; i++) {
+      switch(event_queue[i].type) {
+        case BX_ASYNC_EVT_TOOLBAR:
+          switch (event_queue[i].u.toolbar.button) {
+            case BX_TOOLBAR_RESET: reset_handler(); break;
+            case BX_TOOLBAR_POWER: tb_button = 1; break;
+            case BX_TOOLBAR_SAVE_RESTORE: tb_button = 2; break;
+            case BX_TOOLBAR_COPY: copy_handler(); break;
+            case BX_TOOLBAR_PASTE: paste_handler(); break;
+            case BX_TOOLBAR_SNAPSHOT: snapshot_handler(); break;
+            case BX_TOOLBAR_USER: tb_button = 3; break;
+            default:
+              wxLogDebug (wxT ("unknown toolbar id %d"), event_queue[i].u.toolbar.button);
           }
-          if (released) bx_key |= BX_KEY_RELEASED;
-        }
-        // event contains BX_KEY_* codes: use gen_scancode
-        IFDBG_KEY (wxLogDebug (wxT ("sending key event 0x%02x", bx_key)));
-        DEV_kbd_gen_scancode(bx_key);
-        break;
-      case BX_ASYNC_EVT_MOUSE:
-        DEV_mouse_motion(
-            event_queue[i].u.mouse.dx,
-            event_queue[i].u.mouse.dy,
-            0,
-            event_queue[i].u.mouse.buttons, wxMouseModeAbsXY);
-        break;
-      default:
-        wxLogError (wxT ("handle_events received unhandled event type %d in queue"), (int)event_queue[i].type);
+          break;
+        case BX_ASYNC_EVT_KEY:
+          bx_key = event_queue[i].u.key.bx_key;
+          if (event_queue[i].u.key.raw_scancode) {
+            // event contains raw scancodes: convert to BX_KEY values first
+            bx_bool released = ((bx_key & 0x80) > 0);
+            if (bx_key & 0xFF00) { // for extended keys
+              switch (bx_key & 0x7f) {
+                case 0x1C:
+                  bx_key = BX_KEY_KP_ENTER;
+                  break;
+                case 0x1D:
+                  bx_key = BX_KEY_CTRL_R;
+                  break;
+                case 0x35:
+                  bx_key = BX_KEY_KP_DIVIDE;
+                  break;
+                case 0x38:
+                  // This makes the "AltGr" key on European keyboards work
+                  DEV_kbd_gen_scancode(BX_KEY_CTRL_L | BX_KEY_RELEASED);
+                  bx_key = BX_KEY_ALT_R;
+                  break;
+                case 0x45:
+                  bx_key = BX_KEY_NUM_LOCK;
+                  break;
+                case 0x47:
+                  bx_key = BX_KEY_HOME;
+                  break;
+                case 0x48:
+                  bx_key = BX_KEY_UP;
+                  break;
+                case 0x49:
+                  bx_key = BX_KEY_PAGE_UP;
+                  break;
+                case 0x4B:
+                  bx_key = BX_KEY_LEFT;
+                  break;
+                case 0x4D:
+                  bx_key = BX_KEY_RIGHT;
+                  break;
+                case 0x4F:
+                  bx_key = BX_KEY_END;
+                  break;
+                case 0x50:
+                  bx_key = BX_KEY_DOWN;
+                  break;
+                case 0x51:
+                  bx_key = BX_KEY_PAGE_DOWN;
+                  break;
+                case 0x52:
+                  bx_key = BX_KEY_INSERT;
+                  break;
+                case 0x53:
+                  bx_key = BX_KEY_DELETE;
+                  break;
+                case 0x5B:
+                  bx_key = BX_KEY_WIN_L;
+                  break;
+                case 0x5C:
+                  bx_key = BX_KEY_WIN_R;
+                  break;
+                case 0x5D:
+                  bx_key = BX_KEY_MENU;
+                  break;
+              }
+            } else {
+              bx_key = wxMSW_to_bx_key[bx_key & 0x7f];
+            }
+            if (released) bx_key |= BX_KEY_RELEASED;
+          }
+          if ((bx_key & 0xff) < BX_KEY_NBKEYS) {
+            // event contains BX_KEY_* codes: use gen_scancode
+            IFDBG_KEY(wxLogDebug(wxT("sending key event 0x%02x", bx_key)));
+            DEV_kbd_gen_scancode(bx_key);
+          } else {
+            // value BX_KEY_NBKEYS forces sending release for all pressed keys
+            DEV_kbd_release_keys();
+          }
+          break;
+        case BX_ASYNC_EVT_MOUSE:
+          DEV_mouse_motion(
+              event_queue[i].u.mouse.dx,
+              event_queue[i].u.mouse.dy,
+              event_queue[i].u.mouse.dz,
+              event_queue[i].u.mouse.buttons, wxMouseModeAbsXY);
+          break;
+        default:
+          wxLogError (wxT ("handle_events received unhandled event type %d in queue"), (int)event_queue[i].type);
+      }
+      if (tb_button == 1) break;
     }
+    num_events = 0;
+  } // critical section end
+  // These button handlers must be placed outside of the critical section
+  if (tb_button == 1) {
+    // power_handler() never returns.
+    power_handler();
+  } else if (tb_button == 2) {
+    // save_restore_handler() calls a dialog.
+    save_restore_handler();
+  } else if (tb_button == 3) {
+    // userbutton_handler() also calls a dialog.
+    userbutton_handler();
   }
-  num_events = 0;
 }
 
 void bx_wx_gui_c::statusbar_setitem_specific(int element, bx_bool active, bx_bool w)
@@ -1160,7 +1235,7 @@ static void UpdateScreen(unsigned char *newBits, int x, int y, int width, int he
   wxCriticalSectionLocker lock(wxScreen_lock);
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::UpdateScreen got lock. wxScreen=%p", wxScreen)));
   if(wxScreen != NULL) {
-    switch (vga_bpp) {
+    switch (disp_bpp) {
       case 8: /* 8 bpp */
         for(int i = 0; i < height; i++) {
           char *pwxScreen = &wxScreen[(y * wxScreenX * 3) + (x * 3)];
@@ -1176,7 +1251,7 @@ static void UpdateScreen(unsigned char *newBits, int x, int y, int width, int he
         }
         break;
       default:
-        BX_PANIC(("%u bpp modes handled by new graphics API", vga_bpp));
+        BX_PANIC(("%u bpp modes handled by new graphics API", disp_bpp));
         return;
     }
   } else {
@@ -1423,7 +1498,7 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
 // returns: 0=no screen update needed (color map change has direct effect)
 //          1=screen update needed (redraw using current colormap)
 
-bx_bool bx_wx_gui_c::palette_change(unsigned index, unsigned red, unsigned green, unsigned blue)
+bx_bool bx_wx_gui_c::palette_change(Bit8u index, Bit8u red, Bit8u green, Bit8u blue)
 {
   IFDBG_VGA(wxLogDebug (wxT ("palette_change")));
   wxBochsPalette[index].red = red;
@@ -1457,13 +1532,6 @@ void bx_wx_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 
 bx_svga_tileinfo_t *bx_wx_gui_c::graphics_tile_info(bx_svga_tileinfo_t *info)
 {
-  if (!info) {
-    info = (bx_svga_tileinfo_t *)malloc(sizeof(bx_svga_tileinfo_t));
-    if (!info) {
-      return NULL;
-    }
-  }
-
   info->bpp = 24;
   info->pitch = wxScreenX * 3;
   info->red_shift = 8;
@@ -1528,18 +1596,23 @@ void bx_wx_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, uns
   BX_INFO (("dimension update x=%d y=%d fontheight=%d fontwidth=%d bpp=%d", x, y, fheight, fwidth, bpp));
   if ((bpp == 8) || (bpp == 15) || (bpp == 16) || (bpp == 24) || (bpp == 32)) {
     if (bpp == 32) BX_INFO(("wxWidgets ignores bit 24..31 in 32bpp mode"));
-    vga_bpp = bpp;
-  }
-  else
-  {
+    disp_bpp = guest_bpp = bpp;
+  } else {
     BX_PANIC(("%d bpp graphics mode not supported", bpp));
   }
-  if (fheight > 0) {
+  guest_textmode = (fheight > 0);
+  guest_xres = x;
+  guest_yres = y;
+  if (guest_textmode) {
     wxFontX = fwidth;
     wxFontY = fheight;
     text_cols = x / wxFontX;
     text_rows = y / wxFontY;
   }
+  if (((int)x > wx_maxres.GetWidth()) || ((int)y > wx_maxres.GetHeight())) {
+    BX_PANIC(("dimension_update(): resolution of out of display bounds"));
+    return;
+  }
   wxScreenX = x;
   wxScreenY = y;
   wxScreen = (char *)realloc(wxScreen, wxScreenX * wxScreenY * 3);
@@ -1645,6 +1718,15 @@ bx_wx_gui_c::replace_bitmap(unsigned hbar_id, unsigned bmap_id)
 void bx_wx_gui_c::exit(void)
 {
   clear_screen();
+  if (mouse_captured) {
+    thePanel->ToggleMouse(false);
+    mouse_captured = 0;
+  }
+#if BX_DEBUGGER && BX_DEBUGGER_GUI
+  wxMutexGuiEnter();
+  close_debug_dialog();
+  wxMutexGuiLeave();
+#endif
 }
 
 void bx_wx_gui_c::mouse_enabled_changed_specific(bx_bool val)
@@ -1692,6 +1774,16 @@ int bx_wx_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
   return ret;
 }
 
+void bx_wx_gui_c::get_capabilities(Bit16u *xres, Bit16u *yres, Bit16u *bpp)
+{
+  wxDisplay display;
+
+  wx_maxres = display.GetGeometry();
+  *xres = wx_maxres.GetWidth();
+  *yres = wx_maxres.GetHeight();
+  *bpp = 32;
+}
+
 void bx_wx_gui_c::set_mouse_mode_absxy(bx_bool mode)
 {
   wxMouseModeAbsXY = mode;
