@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: biosdev.cc 11058 2012-02-23 17:16:35Z vruppert $
+// $Id: biosdev.cc 12366 2014-06-08 08:40:08Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2009  The Bochs Project
+//  Copyright (C) 2002-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -42,17 +42,17 @@
 
 bx_biosdev_c *theBiosDevice = NULL;
 
-logfunctions  *bioslog;
+#define bioslog theBiosDevice
 logfunctions  *vgabioslog;
 
-int libbiosdev_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+int CDECL libbiosdev_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
   theBiosDevice = new bx_biosdev_c();
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theBiosDevice, BX_PLUGIN_BIOSDEV);
   return(0); // Success
 }
 
-void libbiosdev_LTX_plugin_fini(void)
+void CDECL libbiosdev_LTX_plugin_fini(void)
 {
   delete theBiosDevice;
 }
@@ -61,8 +61,7 @@ bx_biosdev_c::bx_biosdev_c(void)
 {
   memset(&s, 0, sizeof(s));
 
-  bioslog = new logfunctions();
-  bioslog->put("BIOS");
+  put("biosdev", "BIOS");
 
   vgabioslog = new logfunctions();
   vgabioslog->put("vgabios", "VBIOS");
@@ -71,10 +70,6 @@ bx_biosdev_c::bx_biosdev_c(void)
 bx_biosdev_c::~bx_biosdev_c(void)
 {
   bioslog->ldebug("Exit");
-  if (bioslog != NULL) {
-    delete bioslog;
-    bioslog = NULL;
-  }
   if (vgabioslog != NULL) {
     delete vgabioslog;
     vgabioslog = NULL;
