@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vgacore.cc 11608 2013-02-02 16:02:08Z vruppert $
+// $Id: vgacore.cc 12550 2014-11-15 10:22:27Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2013  The Bochs Project
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -259,17 +259,18 @@ void bx_vgacore_c::init_iohandlers(bx_read_handler_t f_read, bx_write_handler_t
 
 void bx_vgacore_c::init_systemtimer(bx_timer_handler_t f_timer, param_event_handler f_param)
 {
+  const bx_bool realtime = SIM->get_param_bool(BXPN_VGA_REALTIME)->get();
   bx_param_num_c *vga_update_freq = SIM->get_param_num(BXPN_VGA_UPDATE_FREQUENCY);
   BX_VGA_THIS update_interval = (Bit32u)(1000000 / vga_update_freq->get());
-  BX_INFO(("interval=%u", BX_VGA_THIS update_interval));
+  BX_INFO(("interval=%u, mode=%s", BX_VGA_THIS update_interval, realtime ? "realtime":"standard"));
   if (BX_VGA_THIS timer_id == BX_NULL_TIMER_HANDLE) {
     BX_VGA_THIS timer_id = bx_virt_timer.register_timer(this, f_timer,
-       BX_VGA_THIS update_interval, 1, 1, "vga");
+       BX_VGA_THIS update_interval, 1, 1, realtime, "vga");
     vga_update_freq->set_handler(f_param);
-    vga_update_freq->set_runtime_param(1);
   }
-  if (BX_VGA_THIS update_interval < 300000) {
-    BX_VGA_THIS s.blink_counter = 300000 / (unsigned)BX_VGA_THIS update_interval;
+  // VGA text mode cursor blink frequency 1.875 Hz
+  if (BX_VGA_THIS update_interval < 266666) {
+    BX_VGA_THIS s.blink_counter = 266666 / (unsigned)BX_VGA_THIS update_interval;
   } else {
     BX_VGA_THIS s.blink_counter = 1;
   }
@@ -399,7 +400,7 @@ void bx_vgacore_c::determine_screen_dimensions(unsigned *piHeight, unsigned *piW
 {
   int ai[0x20];
   int i,h,v;
-  for (i = 0 ; i < 0x20 ; i++)
+  for (i = 0 ; i < 0x19 ; i++)
    ai[i] = BX_VGA_THIS s.CRTC.reg[i];
 
   h = (ai[1] + 1) * 8;
