@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ioapic.cc 11553 2012-11-25 19:06:03Z vruppert $
+// $Id: ioapic.cc 12366 2014-06-08 08:40:08Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2012  The Bochs Project
+//  Copyright (C) 2002-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,7 @@
 
 bx_ioapic_c *theIOAPIC = NULL;
 
-int libioapic_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+int CDECL libioapic_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
   theIOAPIC = new bx_ioapic_c();
   bx_devices.pluginIOAPIC = theIOAPIC;
@@ -42,7 +42,7 @@ int libioapic_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, cha
   return(0); // Success
 }
 
-void libioapic_LTX_plugin_fini(void)
+void CDECL libioapic_LTX_plugin_fini(void)
 {
   bx_devices.pluginIOAPIC = &bx_devices.stubIOAPIC;
   delete theIOAPIC;
@@ -122,7 +122,7 @@ void bx_io_redirect_entry_t::register_state(bx_param_c *parent)
 bx_ioapic_c::bx_ioapic_c(): enabled(0), base_addr(BX_IOAPIC_BASE_ADDR)
 {
   set_id(BX_IOAPIC_DEFAULT_ID);
-  put("ioapic", "IOAP");
+  put("IOAPIC");
 }
 
 bx_ioapic_c::~bx_ioapic_c()
