@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: iodev.h 11553 2012-11-25 19:06:03Z vruppert $
+// $Id: iodev.h 12681 2015-03-06 22:54:30Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2002-2012  The Bochs Project
+//  Copyright (C) 2001-2015  The Bochs Project
 //
 //  I/O port handlers API Copyright (C) 2003 by Frank Cornelis
 //
@@ -77,14 +77,18 @@ class BOCHSAPI bx_devmodel_c : public logfunctions {
 #endif
 };
 
+// forward declarations
+class bx_list_c;
+class device_image_t;
+class cdrom_base_c;
+class bx_soundlow_waveout_c;
+class bx_soundlow_wavein_c;
+class bx_soundlow_midiout_c;
+
 //////////////////////////////////////////////////////////////////////
 // declare stubs for PCI devices
 //////////////////////////////////////////////////////////////////////
 
-class bx_list_c;
-class device_image_t;
-class LOWLEVEL_CDROM;
-
 // the best should be deriving of bx_pci_device_stub_c from bx_devmodel_c
 // but it make serious problems for cirrus_svga device
 class BOCHSAPI bx_pci_device_stub_c {
@@ -100,8 +104,8 @@ public:
 
   virtual void pci_write_handler(Bit8u address, Bit32u value, unsigned io_len) {}
 
+  void init_pci_conf(Bit16u vid, Bit16u did, Bit8u rev, Bit32u classc, Bit8u headt);
   void register_pci_state(bx_list_c *list);
-
   void load_pci_rom(const char *path);
 
 protected:
@@ -131,6 +135,9 @@ public:
   virtual void paste_bytes(Bit8u *data, Bit32s length) {
     STUBFUNC(keyboard, paste_bytes);
   }
+  virtual void release_keys(void) {
+    STUBFUNC(keyboard, release_keys);
+  }
 };
 
 class BOCHSAPI bx_hard_drive_stub_c : public bx_devmodel_c {
@@ -277,6 +284,7 @@ public:
   virtual bx_bool bmdma_present(void) {
     return 0;
   }
+  virtual void bmdma_start_transfer(Bit8u channel) {}
   virtual void bmdma_set_irq(Bit8u channel) {}
 };
 
@@ -338,24 +346,22 @@ public:
   virtual device_image_t* init_image(Bit8u image_mode, Bit64u disk_size, const char *journal) {
     STUBFUNC(hdimage_ctl, init_image); return NULL;
   }
-#ifdef LOWLEVEL_CDROM
-  virtual LOWLEVEL_CDROM* init_cdrom(const char *dev) {
+  virtual cdrom_base_c* init_cdrom(const char *dev) {
     STUBFUNC(hdimage_ctl, init_cdrom); return NULL;
   }
-#endif
 };
 
 #if BX_SUPPORT_SOUNDLOW
 class BOCHSAPI bx_soundmod_ctl_stub_c : public bx_devmodel_c {
 public:
-  virtual void* init_module(const char *type, logfunctions *dev) {
-    STUBFUNC(soundmod_ctl, init_module); return NULL;
+  virtual bx_soundlow_waveout_c* get_waveout(bx_bool using_file) {
+    STUBFUNC(soundmod_ctl, get_waveout); return NULL;
   }
-  virtual bx_bool beep_on(float frequency) {
-    return 0;
+  virtual bx_soundlow_wavein_c* get_wavein() {
+    STUBFUNC(soundmod_ctl, get_wavein); return NULL;
   }
-  virtual bx_bool beep_off() {
-    return 0;
+  virtual bx_soundlow_midiout_c* get_midiout(bx_bool using_file) {
+    STUBFUNC(soundmod_ctl, get_midiout); return NULL;
   }
 };
 #endif
