@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pcidev.cc 10961 2012-01-09 17:15:03Z vruppert $
+// $Id: pcidev.cc 12366 2014-06-08 08:40:08Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 
 /*
@@ -69,7 +69,7 @@ void pcidev_init_options(void)
   pcidid->set_format("0x%04x");
   pcidid->set_long_format("PCI Device ID: 0x%04x");
   pcidev->set_options(pcidev->SHOW_PARENT | pcidev->USE_BOX_TITLE);
-  bx_list_c *deplist = ((bx_param_bool_c*)SIM->get_param(BXPN_I440FX_SUPPORT))->get_dependent_list();
+  bx_list_c *deplist = ((bx_param_bool_c*)SIM->get_param(BXPN_PCI_ENABLED))->get_dependent_list();
   deplist->add(pcidev);
   deplist->add(pcivid);
   deplist->add(pcidid);
@@ -114,7 +114,7 @@ Bit32s pcidev_options_save(FILE *fp)
 
 // device plugin entry points
 
-int libpcidev_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+int CDECL libpcidev_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
   thePciDevAdapter = new bx_pcidev_c();
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, thePciDevAdapter, BX_PLUGIN_PCIDEV);
@@ -125,7 +125,7 @@ int libpcidev_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, cha
   return 0; // Success
 }
 
-void libpcidev_LTX_plugin_fini(void)
+void CDECL libpcidev_LTX_plugin_fini(void)
 {
   SIM->unregister_addon_option("pcidev");
   bx_list_c *menu = (bx_list_c*)SIM->get_param("network");
@@ -137,7 +137,7 @@ void libpcidev_LTX_plugin_fini(void)
 
 bx_pcidev_c::bx_pcidev_c()
 {
-  put("pcidev", "PCI2H");
+  put("PCIDEV");
 }
 
 bx_pcidev_c::~bx_pcidev_c()
