@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: unmapped.cc 10886 2011-12-29 19:51:54Z vruppert $
+// $Id: unmapped.cc 12366 2014-06-08 08:40:08Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2009  The Bochs Project
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -31,21 +31,21 @@
 
 bx_unmapped_c *theUnmappedDevice = NULL;
 
-int libunmapped_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+int CDECL libunmapped_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
   theUnmappedDevice = new bx_unmapped_c();
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theUnmappedDevice, BX_PLUGIN_UNMAPPED);
   return(0); // Success
 }
 
-void libunmapped_LTX_plugin_fini(void)
+void CDECL libunmapped_LTX_plugin_fini(void)
 {
   delete theUnmappedDevice;
 }
 
 bx_unmapped_c::bx_unmapped_c(void)
 {
-  put("unmapped", "UNMP");
+  put("unmapped", "UNMAP");
 }
 
 bx_unmapped_c::~bx_unmapped_c(void)
