@@ -1,12 +1,12 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: usb_msd.cc 11591 2013-01-25 17:56:40Z vruppert $
+// $Id: usb_msd.cc 12494 2014-09-29 17:48:30Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
 //  USB mass storage device support (ported from QEMU)
 //
 //  Copyright (c) 2006 CodeSourcery.
 //  Written by Paul Brook
-//  Copyright (C) 2009-2012  The Bochs Project
+//  Copyright (C) 2009-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -67,6 +67,7 @@ struct usb_msd_csw {
 #define MassStorageReset  0xff
 #define GetMaxLun         0xfe
 
+// Low-, Full-, and High-speed
 static const Bit8u bx_msd_dev_descriptor[] = {
   0x12,       /*  u8 bLength; */
   0x01,       /*  u8 bDescriptorType; Device */
@@ -88,6 +89,7 @@ static const Bit8u bx_msd_dev_descriptor[] = {
   0x01        /*  u8  bNumConfigurations; */
 };
 
+// Low-, Full-, and High-speed
 static const Bit8u bx_msd_config_descriptor[] = {
 
   /* one configuration */
@@ -132,8 +134,112 @@ static const Bit8u bx_msd_config_descriptor[] = {
   0x00        /*  u8  ep_bInterval; */
 };
 
-static int cdrom_count = 0;
+// Super-speed
+static const Bit8u bx_msd_dev_descriptor3[] = {
+  0x12,       /*  u8 bLength; */
+  0x01,       /*  u8 bDescriptorType; Device */
+  0x00, 0x03, /*  u16 bcdUSB; v3.0 */
 
+  0x00,       /*  u8  bDeviceClass; */
+  0x00,       /*  u8  bDeviceSubClass; */
+  0x00,       /*  u8  bDeviceProtocol; */
+  0x09,       /*  u8  bMaxPacketSize0; 2^^9 = 512 Bytes */
+
+  /* Vendor and product id are arbitrary.  */
+  0x00, 0x00, /*  u16 idVendor; */
+  0x00, 0x00, /*  u16 idProduct; */
+  0x00, 0x01, /*  u16 bcdDevice */
+
+  0x01,       /*  u8  iManufacturer; */
+  0x02,       /*  u8  iProduct; */
+  0x03,       /*  u8  iSerialNumber; */
+  0x01        /*  u8  bNumConfigurations; */
+};
+
+static const Bit8u bx_msd_config_descriptor3[] = {
+  /* one configuration */
+  0x09,       /*  u8  bLength; */
+  0x02,       /*  u8  bDescriptorType; Configuration */
+  0x2C, 0x00, /*  u16 wTotalLength; */
+  0x01,       /*  u8  bNumInterfaces; (1) */
+  0x01,       /*  u8  bConfigurationValue; */
+  0x00,       /*  u8  iConfiguration; */
+  0x80,       /*  u8  bmAttributes;
+                        Bit 7: must be set,
+                            6: Self-powered,
+                            5: Remote wakeup,
+                            4..0: resvd */
+  0x3F,       /*  u8  MaxPower; */
+
+  /* one interface */
+  0x09,       /*  u8  if_bLength; */
+  0x04,       /*  u8  if_bDescriptorType; Interface */
+  0x00,       /*  u8  if_bInterfaceNumber; */
+  0x00,       /*  u8  if_bAlternateSetting; */
+  0x02,       /*  u8  if_bNumEndpoints; */
+  0x08,       /*  u8  if_bInterfaceClass; MASS STORAGE */
+  0x06,       /*  u8  if_bInterfaceSubClass; SCSI */
+  0x50,       /*  u8  if_bInterfaceProtocol; Bulk Only */
+  0x00,       /*  u8  if_iInterface; */
+
+  /* Bulk-In endpoint */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x81,       /*  u8  ep_bEndpointAddress; IN Endpoint 1 */
+  0x02,       /*  u8  ep_bmAttributes; Bulk */
+  0x00, 0x04, /*  u16 ep_wMaxPacketSize; */
+  0x00,       /*  u8  ep_bInterval; */
+
+  /* Bulk-In companion descriptor */
+  0x06,       /*  u8  epc_bLength; */
+  0x30,       /*  u8  epc_bDescriptorType; Endpoint Companion */
+  0x0F,       /*  u8  epc_bMaxPerBurst; */
+  0x00,       /*  u8  epc_bmAttributes; */
+  0x00, 0x00, /*  u16 epc_reserved; */
+
+  /* Bulk-Out endpoint */
+  0x07,       /*  u8  ep_bLength; */
+  0x05,       /*  u8  ep_bDescriptorType; Endpoint */
+  0x02,       /*  u8  ep_bEndpointAddress; OUT Endpoint 2 */
+  0x02,       /*  u8  ep_bmAttributes; Bulk */
+  0x00, 0x04, /*  u16 ep_wMaxPacketSize; */
+  0x00,       /*  u8  ep_bInterval; */
+
+  /* Bulk-Out companion descriptor */
+  0x06,       /*  u8  epc_bLength; */
+  0x30,       /*  u8  epc_bDescriptorType; Endpoint Companion */
+  0x0F,       /*  u8  epc_bMaxPerBurst; */
+  0x00,       /*  u8  epc_bmAttributes; */
+  0x00, 0x00 /*  u16  epc_reserved; */
+};
+
+// BOS Descriptor
+static const Bit8u bx_msd_bos_descriptor3[] = {
+  /* stub */
+  0x05,       /*  u8  bos_bLength; */
+  0x0F,       /*  u8  bos_bDescriptorType; BOS */
+  0x16, 0x00, /* u16  bos_wTotalLength; */
+  0x02,       /*  u8  bos_bNumCapEntries; BOS */
+
+  /* USB 2.0 Extention */
+  0x07,       /*  u8  bss_bLength; */
+  0x10,       /*  u8  bss_bType; Device Cap */
+  0x02,       /*  u8  bss_bCapType; USB 2.0 Ext */
+  0x02, 0x00, /* u32  bss_bmAttributes; */
+  0x00, 0x00,
+
+  /* USB 3.0 */
+  0x0A,       /*  u8  bss_bLength; */
+  0x10,       /*  u8  bss_bType; Device Cap */
+  0x03,       /*  u8  bss_bCapType; USB 3.0 */
+  0x00,       /*  u8  bss_bmAttributes; */
+  0x0E, 0x00, /* u16  bss_bmSupSpeeds; */
+  0x01,       /*  u8  bss_bSupFunct; */
+  0x0A,       /*  u8  bss_bU1DevExitLat; */
+  0x20, 0x00  /* u16  bss_wU2DevExitLat; */
+};
+
+static int usb_cdrom_count = 0;
 
 usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
 {
@@ -145,9 +251,7 @@ usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
   bx_param_enum_c *status;
 
   d.type = type;
-  d.maxspeed = USB_SPEED_FULL;
-  d.speed = d.maxspeed;
-  // TODO: d.max_packet_size = ???
+  d.maxspeed = USB_SPEED_SUPER;
   memset((void*)&s, 0, sizeof(s));
   if (d.type == USB_DEV_TYPE_DISK) {
     strcpy(d.devname, "BOCHS USB HARDDRIVE");
@@ -162,21 +266,20 @@ usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
       s.fname = filename+strlen(ptr1)+1;
     }
     s.journal[0] = 0;
+    s.size = 0;
   } else if (d.type == USB_DEV_TYPE_CDROM) {
     strcpy(d.devname, "BOCHS USB CDROM");
     s.fname = filename;
     // config options
     bx_list_c *usb_rt = (bx_list_c*)SIM->get_param(BXPN_MENU_RUNTIME_USB);
-    sprintf(pname, "cdrom%d", ++cdrom_count);
-    sprintf(label, "USB CD-ROM #%d Configuration", cdrom_count);
+    sprintf(pname, "cdrom%d", ++usb_cdrom_count);
+    sprintf(label, "USB CD-ROM #%d Configuration", usb_cdrom_count);
     s.config = new bx_list_c(usb_rt, pname, label);
     s.config->set_options(bx_list_c::SERIES_ASK | bx_list_c::USE_BOX_TITLE);
-    s.config->set_runtime_param(1);
     s.config->set_device_param(this);
     path = new bx_param_string_c(s.config, "path", "Path", "", "", BX_PATHNAME_LEN);
     path->set(s.fname);
     path->set_handler(cd_param_string_handler);
-    path->set_runtime_param(1);
     status = new bx_param_enum_c(s.config,
       "status",
       "Status",
@@ -185,15 +288,14 @@ usb_msd_device_c::usb_msd_device_c(usbdev_type type, const char *filename)
       BX_INSERTED,
       BX_EJECTED);
     status->set_handler(cd_param_handler);
-    status->set_runtime_param(1);
     status->set_ask_format("Is the device inserted or ejected? [%s] ");
-#if BX_WITH_WX
-    bx_list_c *usb = (bx_list_c*)SIM->get_param("ports.usb");
-    usb->add(s.config);
-#endif
+    if (SIM->is_wx_selected()) {
+      bx_list_c *usb = (bx_list_c*)SIM->get_param("ports.usb");
+      usb->add(s.config);
+    }
   }
 
-  put("usb_msd", "USBMS");
+  put("usb_msd", "USBMSD");
 }
 
 usb_msd_device_c::~usb_msd_device_c(void)
@@ -202,24 +304,47 @@ usb_msd_device_c::~usb_msd_device_c(void)
     delete s.scsi_dev;
   if (s.hdimage != NULL) {
     delete s.hdimage;
-#ifdef LOWLEVEL_CDROM
   } else if (s.cdrom != NULL) {
     delete s.cdrom;
-#if BX_WITH_WX
-    bx_list_c *usb = (bx_list_c*)SIM->get_param("ports.usb");
-    usb->remove(s.config->get_name());
-#endif
+    if (SIM->is_wx_selected()) {
+      bx_list_c *usb = (bx_list_c*)SIM->get_param("ports.usb");
+      usb->remove(s.config->get_name());
+    }
     bx_list_c *usb_rt = (bx_list_c*)SIM->get_param(BXPN_MENU_RUNTIME_USB);
     usb_rt->remove(s.config->get_name());
-#endif
   }
 }
 
 bx_bool usb_msd_device_c::set_option(const char *option)
 {
+  char *suffix;
+
   if (!strncmp(option, "journal:", 8)) {
-    strcpy(s.journal, option+8);
-    return 1;
+    if (d.type == USB_DEV_TYPE_DISK) {
+      strcpy(s.journal, option+8);
+      return 1;
+    } else {
+      BX_ERROR(("Option 'journal' is only valid for USB disks"));
+    }
+  } else if (!strncmp(option, "size:", 5)) {
+    if ((d.type == USB_DEV_TYPE_DISK) && (s.image_mode == BX_HDIMAGE_MODE_VVFAT)) {
+      s.size = (int)strtol(option+5, &suffix, 10);
+      if (!strcmp(suffix, "G")) {
+        s.size <<= 10;
+      } else if (strcmp(suffix, "M")) {
+        BX_ERROR(("Unknown VVFAT disk size suffix '%s' - using default", suffix));
+        s.size = 0;
+        return 0;
+      }
+      if ((s.size < 128) || (s.size >= 131072)) {
+        BX_ERROR(("Invalid VVFAT disk size value - using default"));
+        s.size = 0;
+        return 0;
+      }
+      return 1;
+    } else {
+      BX_ERROR(("Option 'size' is only valid for USB VVFAT disks"));
+    }
   }
   return 0;
 }
@@ -228,6 +353,12 @@ bx_bool usb_msd_device_c::init()
 {
   if (d.type == USB_DEV_TYPE_DISK) {
     s.hdimage = DEV_hdimage_init_image(s.image_mode, 0, s.journal);
+    if (s.image_mode == BX_HDIMAGE_MODE_VVFAT) {
+      Bit64u hdsize = ((Bit64u)s.size) << 20;
+      s.hdimage->cylinders = (Bit64u)(hdsize/16.0/63.0/512.0);
+      s.hdimage->heads = 16;
+      s.hdimage->spt = 63;
+    }
     if (s.hdimage->open(s.fname) < 0) {
       BX_ERROR(("could not open hard drive image file '%s'", s.fname));
       return 0;
@@ -236,19 +367,13 @@ bx_bool usb_msd_device_c::init()
     }
     sprintf(s.info_txt, "USB HD: path='%s', mode='%s'", s.fname, hdimage_mode_names[s.image_mode]);
   } else if (d.type == USB_DEV_TYPE_CDROM) {
-#ifdef LOWLEVEL_CDROM
     s.cdrom = DEV_hdimage_init_cdrom(s.fname);
-    if (!s.cdrom->insert_cdrom()) {
-      BX_ERROR(("could not open cdrom image file '%s'", s.fname));
-      return 0;
+    s.scsi_dev = new scsi_device_t(s.cdrom, 0, usb_msd_command_complete, (void*)this);
+    if (set_inserted(1)) {
+      sprintf(s.info_txt, "USB CD: path='%s'", s.fname);
     } else {
-      s.scsi_dev = new scsi_device_t(s.cdrom, 0, usb_msd_command_complete, (void*)this);
+      sprintf(s.info_txt, "USB CD: media not present");
     }
-    sprintf(s.info_txt, "USB CD: path='%s'", s.fname);
-#else
-    BX_PANIC(("missing LOWLEVEL_CDROM support"));
-    return 0;
-#endif
   }
   s.scsi_dev->register_state(s.sr_list, "scsidev");
   s.mode = USB_MSDM_CBW;
@@ -289,42 +414,66 @@ int usb_msd_device_c::handle_control(int request, int value, int index, int leng
   switch (request) {
     case DeviceRequest | USB_REQ_GET_STATUS:
     case EndpointRequest | USB_REQ_GET_STATUS:
+      BX_DEBUG(("USB_REQ_GET_STATUS:"));
       data[0] = (1 << USB_DEVICE_SELF_POWERED) |
         (d.remote_wakeup << USB_DEVICE_REMOTE_WAKEUP);
       data[1] = 0x00;
       ret = 2;
       break;
     case DeviceOutRequest | USB_REQ_CLEAR_FEATURE:
+      BX_DEBUG(("USB_REQ_CLEAR_FEATURE:"));
       if (value == USB_DEVICE_REMOTE_WAKEUP) {
         d.remote_wakeup = 0;
       } else {
+        BX_DEBUG(("USB_REQ_CLEAR_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
         goto fail;
       }
       ret = 0;
       break;
     case DeviceOutRequest | USB_REQ_SET_FEATURE:
-      if (value == USB_DEVICE_REMOTE_WAKEUP) {
-        d.remote_wakeup = 1;
-      } else {
+      BX_DEBUG(("USB_REQ_SET_FEATURE:"));
+      switch (value) {
+        case USB_DEVICE_REMOTE_WAKEUP:
+          d.remote_wakeup = 1;
+          break;
+        case USB_DEVICE_U1_ENABLE:
+        case USB_DEVICE_U2_ENABLE:
+          break;
+        default:
+        BX_DEBUG(("USB_REQ_SET_FEATURE: Not handled: %i %i %i %i", request, value, index, length ));
         goto fail;
       }
       ret = 0;
       break;
     case DeviceOutRequest | USB_REQ_SET_ADDRESS:
+      BX_DEBUG(("USB_REQ_SET_ADDRESS:"));
       d.addr = value;
       ret = 0;
       break;
     case DeviceRequest | USB_REQ_GET_DESCRIPTOR:
-      switch(value >> 8) {
+      switch (value >> 8) {
         case USB_DT_DEVICE:
-          memcpy(data, bx_msd_dev_descriptor, sizeof(bx_msd_dev_descriptor));
-          ret = sizeof(bx_msd_dev_descriptor);
+          BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: Device"));
+          if (get_speed() == USB_SPEED_SUPER) {
+            memcpy(data, bx_msd_dev_descriptor3, sizeof(bx_msd_dev_descriptor3));
+            ret = sizeof(bx_msd_dev_descriptor3);
+          } else {
+            memcpy(data, bx_msd_dev_descriptor, sizeof(bx_msd_dev_descriptor));
+            ret = sizeof(bx_msd_dev_descriptor);
+          }
           break;
         case USB_DT_CONFIG:
-          memcpy(data, bx_msd_config_descriptor, sizeof(bx_msd_config_descriptor));
-          ret = sizeof(bx_msd_config_descriptor);
+          BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: Config"));
+          if (get_speed() == USB_SPEED_SUPER) {
+            memcpy(data, bx_msd_config_descriptor3, sizeof(bx_msd_config_descriptor3));
+            ret = sizeof(bx_msd_config_descriptor3);
+          } else {
+            memcpy(data, bx_msd_config_descriptor, sizeof(bx_msd_config_descriptor));
+            ret = sizeof(bx_msd_config_descriptor);
+          }
           break;
         case USB_DT_STRING:
+          BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: String"));
           switch(value & 0xff) {
             case 0:
               // language IDs
@@ -350,12 +499,17 @@ int usb_msd_device_c::handle_control(int request, int value, int index, int leng
               // serial number
               ret = set_usb_string(data, s.scsi_dev->get_serial_number());
               break;
+            case 0xEE:
+              // Microsoft OS Descriptor check
+              // We don't support this check, so fail
+              goto fail;
             default:
               BX_ERROR(("USB MSD handle_control: unknown string descriptor 0x%02x", value & 0xff));
               goto fail;
           }
           break;
         case USB_DT_DEVICE_QUALIFIER:
+          BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: Device Qualifier"));
           // device qualifier
           data[0] = 10;
           data[1] = USB_DT_DEVICE_QUALIFIER;
@@ -364,40 +518,60 @@ int usb_msd_device_c::handle_control(int request, int value, int index, int leng
           data[9] = 0;
           ret = 10;
           break;
+        case USB_DT_BIN_DEV_OBJ_STORE:
+          BX_DEBUG(("USB_REQ_GET_DESCRIPTOR: BOS"));
+          if (get_speed() == USB_SPEED_SUPER) {
+            memcpy(data, bx_msd_bos_descriptor3, sizeof(bx_msd_bos_descriptor3));
+            ret = sizeof(bx_msd_bos_descriptor3);
+          } else
+            goto fail;
+          break;
         default:
           BX_ERROR(("USB MSD handle_control: unknown descriptor type 0x%02x", value >> 8));
           goto fail;
       }
       break;
     case DeviceRequest | USB_REQ_GET_CONFIGURATION:
+      BX_DEBUG(("USB_REQ_GET_CONFIGURATION:"));
       data[0] = 1;
       ret = 1;
       break;
     case DeviceOutRequest | USB_REQ_SET_CONFIGURATION:
+      BX_DEBUG(("USB_REQ_SET_CONFIGURATION:"));
       ret = 0;
       break;
     case DeviceRequest | USB_REQ_GET_INTERFACE:
+      BX_DEBUG(("USB_REQ_GET_INFTERFACE:"));
       data[0] = 0;
       ret = 1;
       break;
     case DeviceOutRequest | USB_REQ_SET_INTERFACE:
     case InterfaceOutRequest | USB_REQ_SET_INTERFACE:
+      BX_DEBUG(("USB_REQ_SET_INFTERFACE:"));
       ret = 0;
       break;
     case EndpointOutRequest | USB_REQ_CLEAR_FEATURE:
+      BX_DEBUG(("USB_REQ_CLEAR_FEATURE:"));
       if (value == 0 && index != 0x81) { /* clear ep halt */
         goto fail;
       }
       ret = 0;
       break;
-      // Class specific requests
+    case DeviceOutRequest | USB_REQ_SET_SEL:
+      // Set U1 and U2 System Exit Latency
+      BX_DEBUG(("SET_SEL (U1 and U2):"));
+      ret = 0;
+      break;
+    // Class specific requests
     case InterfaceOutClassRequest | MassStorageReset:
     case MassStorageReset:
+      BX_DEBUG(("MASS STORAGE RESET:"));
       s.mode = USB_MSDM_CBW;
       ret = 0;
       break;
     case InterfaceInClassRequest | GetMaxLun:
     case GetMaxLun:
+      BX_DEBUG(("MASS STORAGE: GET MAX LUN"));
       data[0] = 0;
       ret = 1;
       break;
@@ -433,7 +607,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
           }
           memcpy(&cbw, data, 31);
           if (dtoh32(cbw.sig) != 0x43425355) {
-            BX_ERROR(("bad signature %08x", dtoh32(cbw.sig)));
+            BX_ERROR(("bad signature %08X", dtoh32(cbw.sig)));
             goto fail;
           }
           BX_DEBUG(("command on LUN %d", cbw.lun));
@@ -446,7 +620,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
           } else {
             s.mode = USB_MSDM_DATAOUT;
           }
-          BX_DEBUG(("command tag 0x%x flags %08x len %d data %d",
+          BX_DEBUG(("command tag 0x%X flags %08X len %d data %d",
                    s.tag, cbw.flags, cbw.cmd_len, s.data_len));
           s.residue = 0;
           s.scsi_dev->scsi_send_command(s.tag, cbw.cmd, cbw.lun);
@@ -487,7 +661,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
           break;
 
         default:
-          BX_ERROR(("USB MSD handle_data: unexpected mode at USB_TOKEN_OUT"));
+          BX_ERROR(("USB MSD handle_data: unexpected mode at USB_TOKEN_OUT: (0x%02X)", s.mode));
           goto fail;
       }
       break;
@@ -552,7 +726,7 @@ int usb_msd_device_c::handle_data(USBPacket *p)
           break;
 
         default:
-          BX_ERROR(("USB MSD handle_data: unexpected mode at USB_TOKEN_IN"));
+          BX_ERROR(("USB MSD handle_data: unexpected mode at USB_TOKEN_IN: (0x%02X)", s.mode));
           goto fail;
       }
       if (ret > 0) usb_dump_packet(data, ret);
@@ -659,24 +833,21 @@ void usb_msd_device_c::cancel_packet(USBPacket *p)
   s.scsi_len = 0;
 }
 
-void usb_msd_device_c::set_inserted(bx_bool value)
+bx_bool usb_msd_device_c::set_inserted(bx_bool value)
 {
-#ifdef LOWLEVEL_CDROM
   const char *path;
 
   if (value) {
     path = SIM->get_param_string("path", s.config)->getptr();
     if (!s.cdrom->insert_cdrom(path)) {
-      SIM->get_param_bool("status", s.config)->set(0);
-      return;
+      SIM->get_param_enum("status", s.config)->set(BX_EJECTED);
+      return 0;
     }
   } else {
     s.cdrom->eject_cdrom();
   }
   s.scsi_dev->set_inserted(value);
-#else
-  SIM->get_param_bool("status", s.config)->set(0);
-#endif
+  return value;
 }
 
 bx_bool usb_msd_device_c::get_inserted()
@@ -703,7 +874,7 @@ const char *usb_msd_device_c::cd_param_string_handler(bx_param_string_c *param,
           param->set("none");
         }
       } else {
-        SIM->get_param_bool("status", param->get_parent())->set(0);
+        SIM->get_param_enum("status", param->get_parent())->set(BX_EJECTED);
       }
     } else {
       BX_PANIC(("cd_param_string_handler: cdrom not found"));
