@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: main.cc 11370 2012-08-26 12:32:10Z vruppert $
+// $Id: main.cc 12628 2015-02-01 11:46:55Z vruppert $
 /////////////////////////////////////////////////////////////////////////
 //
-//  Copyright (C) 2001-2011  The Bochs Project
+//  Copyright (C) 2001-2014  The Bochs Project
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,7 @@
 #include "bxversion.h"
 #include "param_names.h"
 #include "gui/textconfig.h"
-#if BX_USE_TEXTCONFIG && defined(WIN32) && (BX_WITH_WIN32 || BX_WITH_SDL)
+#if BX_USE_WIN32CONFIG
 #include "gui/win32dialog.h"
 #endif
 #include "cpu/cpu.h"
@@ -32,7 +32,7 @@
 #include <locale.h>
 #endif
 
-#if BX_WITH_SDL
+#if BX_WITH_SDL || BX_WITH_SDL2
 // since SDL redefines main() to SDL_main(), we must include SDL.h so that the
 // C language prototype is found.  Otherwise SDL_main() will get its name
 // mangled and not match what the SDL library is expecting.
@@ -58,19 +58,12 @@ extern "C" {
 bx_bool bx_gui_sighandler = 0;
 #endif
 
-// some prototypes from iodev/
-// I want to stay away from including iodev/iodev.h here
-Bit32u bx_unmapped_io_read_handler(Bit32u address, unsigned io_len);
-void   bx_unmapped_io_write_handler(Bit32u address, Bit32u value,
-                                    unsigned io_len);
-
+int  bx_init_main(int argc, char *argv[]);
 void bx_init_hardware(void);
 void bx_init_options(void);
 void bx_init_bx_dbg(void);
 
 static const char *divider = "========================================================================";
-static logfunctions thePluginLog;
-logfunctions *pluginlog = &thePluginLog;
 
 bx_startup_flags_t bx_startup_flags;
 bx_bool bx_user_quit;
@@ -190,10 +183,10 @@ static void carbonFatalDialog(const char *error, const char *exposition)
 #endif
 
 #if BX_DEBUGGER
-void print_tree(bx_param_c *node, int level)
+void print_tree(bx_param_c *node, int level, bx_bool xml)
 {
   int i;
-  char tmpstr[BX_PATHNAME_LEN], tmpbyte[4];
+  char tmpstr[BX_PATHNAME_LEN];
 
   for (i=0; i<level; i++)
     dbg_printf("  ");
@@ -201,51 +194,108 @@ void print_tree(bx_param_c *node, int level)
       dbg_printf("NULL pointer\n");
       return;
   }
+
+  if (xml)
+    dbg_printf("<%s>", node->get_name());
+  else
+    dbg_printf("%s = ", node->get_name());
+
   switch (node->get_type()) {
     case BXT_PARAM_NUM:
       if (((bx_param_num_c*)node)->get_base() == BASE_DEC) {
-        dbg_printf("%s = " FMT_LL "d (number)\n", node->get_name(), ((bx_param_num_c*)node)->get64());
+        dbg_printf("" FMT_LL "d", ((bx_param_num_c*)node)->get64());
+        if (! xml) dbg_printf(" (number)");
       } else {
-        dbg_printf("%s = 0x" FMT_LL "x (hex number)\n", node->get_name(), ((bx_param_num_c*)node)->get64());
+        dbg_printf("0x" FMT_LL "x", ((bx_param_num_c*)node)->get64());
+        if (! xml) dbg_printf(" (hex number)");
       }
       break;
     case BXT_PARAM_BOOL:
-      dbg_printf("%s = %s (boolean)\n", node->get_name(), ((bx_param_bool_c*)node)->get()?"true":"false");
+      dbg_printf("%s", ((bx_param_bool_c*)node)->get()?"true":"false");
+      if (! xml) dbg_printf(" (boolean)");
       break;
     case BXT_PARAM_ENUM:
-      dbg_printf("%s = '%s' (enum)\n", node->get_name(), ((bx_param_enum_c*)node)->get_selected());
+      dbg_printf("'%s'", ((bx_param_enum_c*)node)->get_selected());
+      if (! xml) dbg_printf(" (enum)");
       break;
     case BXT_PARAM_STRING:
+      ((bx_param_string_c*)node)->sprint(tmpstr, BX_PATHNAME_LEN, 0);
       if (((bx_param_string_c*)node)->get_options() & bx_param_string_c::RAW_BYTES) {
-        tmpstr[0] = 0;
-        for (i = 0; i < ((bx_param_string_c*)node)->get_maxsize(); i++) {
-          if (i > 0) {
-            tmpbyte[0] = ((bx_param_string_c*)node)->get_separator();
-            tmpbyte[1] = 0;
-            strcat(tmpstr, tmpbyte);
-          }
-          sprintf(tmpbyte, "%02x", (Bit8u)((bx_param_string_c*)node)->getptr()[i]);
-          strcat(tmpstr, tmpbyte);
-        }
-        dbg_printf("%s = '%s' (raw byte string)\n", node->get_name(), tmpstr);
+        dbg_printf("'%s'", tmpstr);
+        if (! xml) dbg_printf(" (raw byte string)");
       } else {
-        dbg_printf("%s = '%s' (string)\n", node->get_name(), ((bx_param_string_c*)node)->getptr());
+        dbg_printf("'%s'", tmpstr);
+        if (! xml) dbg_printf(" (string)");
       }
       break;
     case BXT_LIST:
       {
-	dbg_printf("%s = \n", node->get_name());
-	bx_list_c *list = (bx_list_c*)node;
-	for (i=0; i < list->get_size(); i++) {
-	   print_tree(list->get(i), level+1);
-	}
-	break;
+        dbg_printf("\n");
+        bx_list_c *list = (bx_list_c*)node;
+        for (i=0; i < list->get_size(); i++) {
+          print_tree(list->get(i), level+1, xml);
+        }
+        if (xml) {
+          for (i=0; i<level; i++)
+            dbg_printf("  ");
+        }
+        break;
       }
     case BXT_PARAM_DATA:
-      dbg_printf("%s = 'size=%d' (binary data)\n", node->get_name(), ((bx_shadow_data_c*)node)->get_size());
+      dbg_printf("'binary data size=%d'", ((bx_shadow_data_c*)node)->get_size());
       break;
     default:
-      dbg_printf("%s (unknown parameter type)\n", node->get_name());
+      dbg_printf("(unknown parameter type)");
+  }
+
+  if (xml) dbg_printf("</%s>", node->get_name());
+  dbg_printf("\n");
+}
+#endif
+
+#if BX_ENABLE_STATISTICS
+void print_statistics_tree(bx_param_c *node, int level)
+{
+  for (int i=0; i<level; i++)
+    printf("  ");
+  if (node == NULL) {
+      printf("NULL pointer\n");
+      return;
+  }
+  switch (node->get_type()) {
+    case BXT_PARAM_NUM:
+      {
+        bx_param_num_c* param = (bx_param_num_c*) node;
+        printf("%s = " FMT_LL "d\n", node->get_name(), param->get64());
+        param->set(0); // clear the statistic
+      }
+      break;
+    case BXT_PARAM_BOOL:
+      BX_PANIC(("boolean statistics are not supported !"));
+      break;
+    case BXT_PARAM_ENUM:
+      BX_PANIC(("enum statistics are not supported !"));
+      break;
+    case BXT_PARAM_STRING:
+      BX_PANIC(("string statistics are not supported !"));
+      break;
+    case BXT_LIST:
+      {
+        bx_list_c *list = (bx_list_c*)node;
+        if (list->get_size() > 0) {
+          printf("%s = \n", node->get_name());
+          for (int i=0; i < list->get_size(); i++) {
+            print_statistics_tree(list->get(i), level+1);
+          }
+        }
+        break;
+      }
+    case BXT_PARAM_DATA:
+      BX_PANIC(("binary data statistics are not supported !"));
+      break;
+    default:
+      BX_PANIC(("%s (unknown parameter type)\n", node->get_name()));
+      break;
   }
 }
 #endif
@@ -256,7 +306,6 @@ int bxmain(void)
   // Initialize locale (for isprint() and other functions)
   setlocale (LC_ALL, "");
 #endif
-  bx_user_quit = 0;
   bx_init_siminterface();   // create the SIM object
   static jmp_buf context;
   if (setjmp (context) == 0) {
@@ -278,7 +327,7 @@ int bxmain(void)
 #endif
     }
     else if (!strcmp(ci_name, "win32config")) {
-#if BX_USE_TEXTCONFIG && defined(WIN32) && (BX_WITH_WIN32 || BX_WITH_SDL)
+#if BX_USE_WIN32CONFIG
       init_win32_config_interface();
 #else
       BX_PANIC(("configuration interface 'win32config' not present"));
@@ -402,7 +451,7 @@ int split_string_into_argv(char *string, int *argc_out, char **argv, int max_arg
 }
 #endif /* if defined(__WXMSW__) */
 
-#if defined(__WXMSW__) || (BX_WITH_SDL && defined(WIN32))
+#if defined(__WXMSW__) || ((BX_WITH_SDL || BX_WITH_SDL2) && defined(WIN32))
 // The RedirectIOToConsole() function is copied from an article called "Adding
 // Console I/O to a Win32 GUI App" in Windows Developer Journal, December 1997.
 // It creates a console window.
@@ -440,7 +489,7 @@ int RedirectIOToConsole()
   setvbuf(stderr, NULL, _IONBF, 0);
   return 1;
 }
-#endif  /* if defined(__WXMSW__) || (BX_WITH_SDL && defined(WIN32)) */
+#endif  /* if defined(__WXMSW__) || ((BX_WITH_SDL || BX_WITH_SDL2) && defined(WIN32)) */
 
 #if defined(__WXMSW__)
 // only used for wxWidgets/win32.
@@ -499,7 +548,7 @@ int CDECL main(int argc, char *argv[])
   if (bx_noconsole) {
     FreeConsole();
   } else {
-#if BX_WITH_SDL
+#if BX_WITH_SDL || BX_WITH_SDL2
     // if SDL/win32, try to create a console window.
     if (!RedirectIOToConsole()) {
       return 1;
@@ -519,7 +568,10 @@ void print_usage(void)
     "  -n               no configuration file\n"
     "  -f configfile    specify configuration file\n"
     "  -q               quick start (skip configuration interface)\n"
-    "  -benchmark n     run bochs in benchmark mode for millions of emulated ticks\n"
+    "  -benchmark N     run bochs in benchmark mode for N millions of emulated ticks\n"
+#if BX_ENABLE_STATISTICS
+    "  -dumpstats N     dump bochs stats every N millions of emulated ticks\n"
+#endif
     "  -r path          restore the Bochs state from path\n"
     "  -log filename    specify Bochs log file name\n"
 #if BX_DEBUGGER
@@ -565,7 +617,7 @@ int bx_init_main(int argc, char *argv[])
   SIM->get_param_enum(BXPN_BOCHS_START)->set(BX_RUN_START);
 
   // interpret the args that start with -, like -q, -f, etc.
-  int arg = 1, load_rcfile=1, i = 0;
+  int arg = 1, load_rcfile=1;
   while (arg < argc) {
     // parse next arg
     if (!strcmp("--help", argv[arg]) || !strncmp("-h", argv[arg], 2)
@@ -579,6 +631,9 @@ int bx_init_main(int argc, char *argv[])
 #if BX_SUPPORT_CLGD54XX
           fprintf(stderr, "cirrus\n");
 #endif
+#if BX_SUPPORT_VOODOO
+          fprintf(stderr, "voodoo\n");
+#endif
 #if BX_SUPPORT_PCI
           fprintf(stderr, "pci\n");
 #endif
@@ -617,6 +672,7 @@ int bx_init_main(int argc, char *argv[])
         }
 #if BX_CPU_LEVEL > 4
         else if (!strcmp("cpu", argv[arg+1])) {
+          int i = 0;
           fprintf(stderr, "Supported CPU models:\n\n");
           do {
             fprintf(stderr, "%s\n", SIM->get_param_enum(BXPN_CPU_MODEL)->get_choice(i));
@@ -660,6 +716,12 @@ int bx_init_main(int argc, char *argv[])
       if (++arg >= argc) BX_PANIC(("-benchmark must be followed by a number"));
       else SIM->get_param_num(BXPN_BOCHS_BENCHMARK)->set(atoi(argv[arg]));
     }
+#if BX_ENABLE_STATISTICS
+    else if (!strcmp("-dumpstats", argv[arg])) {
+      if (++arg >= argc) BX_PANIC(("-dumpstats must be followed by a number"));
+      else SIM->get_param_num(BXPN_DUMP_STATS)->set(atoi(argv[arg]));
+    }
+#endif
     else if (!strcmp("-r", argv[arg])) {
       if (++arg >= argc) BX_PANIC(("-r must be followed by a path"));
       else {
@@ -784,6 +846,9 @@ int bx_init_main(int argc, char *argv[])
         BX_PLUGIN_PATH));
     setenv("LTDL_LIBRARY_PATH", BX_PLUGIN_PATH, 1);
   }
+#endif
+#endif  /* if BX_PLUGINS */
+#if BX_HAVE_GETENV && BX_HAVE_SETENV
   if (getenv("BXSHARE") != NULL) {
     BX_INFO (("BXSHARE is set to '%s'", getenv("BXSHARE")));
   } else {
@@ -794,7 +859,6 @@ int bx_init_main(int argc, char *argv[])
 #else
   // we don't have getenv or setenv.  Do nothing.
 #endif
-#endif  /* if BX_PLUGINS */
 
   // initialize plugin system. This must happen before we attempt to
   // load any modules.
@@ -809,6 +873,7 @@ int bx_init_main(int argc, char *argv[])
   // load pre-defined optional plugins before parsing configuration
   SIM->opt_plugin_ctrl("*", 1);
   SIM->init_save_restore();
+  SIM->init_statistics();
   if (load_rcfile) {
     // parse configuration file and command line arguments
 #ifdef WIN32
@@ -911,6 +976,10 @@ bx_bool load_and_init_display_lib(void)
   if (!strcmp(gui_name, "sdl"))
     PLUG_load_plugin (sdl, PLUGTYPE_OPTIONAL);
 #endif
+#if BX_WITH_SDL2
+  if (!strcmp(gui_name, "sdl2"))
+    PLUG_load_plugin (sdl2, PLUGTYPE_OPTIONAL);
+#endif
 #if BX_WITH_SVGA
   if (!strcmp(gui_name, "svga"))
     PLUG_load_plugin (svga, PLUGTYPE_OPTIONAL);
@@ -919,6 +988,10 @@ bx_bool load_and_init_display_lib(void)
   if (!strcmp(gui_name, "term"))
     PLUG_load_plugin (term, PLUGTYPE_OPTIONAL);
 #endif
+#if BX_WITH_VNCSRV
+  if (!strcmp(gui_name, "vncsrv"))
+    PLUG_load_plugin (vncsrv, PLUGTYPE_OPTIONAL);
+#endif
 #if BX_WITH_WIN32
   if (!strcmp(gui_name, "win32"))
     PLUG_load_plugin (win32, PLUGTYPE_OPTIONAL);
@@ -943,6 +1016,7 @@ bx_bool load_and_init_display_lib(void)
 
 int bx_begin_simulation (int argc, char *argv[])
 {
+  bx_user_quit = 0;
   if (SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
     if (!SIM->restore_config()) {
       BX_PANIC(("cannot restore configuration"));
@@ -993,9 +1067,10 @@ int bx_begin_simulation (int argc, char *argv[])
   bx_gui->update_drive_status_buttons();
 
   // iniialize statusbar and set all items inactive
-  if (!SIM->get_param_bool(BXPN_RESTORE_FLAG)->get())
-  {
+  if (!SIM->get_param_bool(BXPN_RESTORE_FLAG)->get()) {
     bx_gui->statusbar_setitem(-1, 0);
+  } else {
+    SIM->get_param_string(BXPN_RESTORE_PATH)->set("none");
   }
 
   // The set handler for mouse_enabled does not actually update the gui
@@ -1119,13 +1194,6 @@ void bx_init_hardware()
 {
   // all configuration has been read, now initialize everything.
 
-  if (SIM->get_param_enum(BXPN_BOCHS_START)->get()==BX_QUICK_START) {
-    for (int level=0; level<N_LOGLEV; level++) {
-      int action = SIM->get_default_log_action(level);
-      io->set_log_action(level, action);
-    }
-  }
-
   bx_pc_system.initialize(SIM->get_param_num(BXPN_IPS)->get());
 
   if (SIM->get_param_string(BXPN_LOG_FILENAME)->getptr()[0]!='-') {
@@ -1152,12 +1220,11 @@ void bx_init_hardware()
   BX_INFO(("  A20 line support: %s", BX_SUPPORT_A20?"yes":"no"));
 #if BX_CONFIGURE_MSRS
   const char *msrs_file = SIM->get_param_string(BXPN_CONFIGURABLE_MSRS_PATH)->getptr();
-  if(strlen(msrs_file) > 0)
+  if ((strlen(msrs_file) > 0) && strcmp(msrs_file, "none"))
     BX_INFO(("  load configurable MSRs from file \"%s\"", msrs_file));
 #endif
   BX_INFO(("IPS is set to %d", (Bit32u) SIM->get_param_num(BXPN_IPS)->get()));
   BX_INFO(("CPU configuration"));
-  BX_INFO(("  level: %d", BX_CPU_LEVEL));
 #if BX_SUPPORT_SMP
   BX_INFO(("  SMP support: yes, quantum=%d", SIM->get_param_num(BXPN_SMP_QUANTUM)->get()));
 #else
@@ -1167,8 +1234,11 @@ void bx_init_hardware()
   unsigned cpu_model = SIM->get_param_enum(BXPN_CPU_MODEL)->get();
   if (! cpu_model) {
 #if BX_CPU_LEVEL >= 5
+    unsigned cpu_level = SIM->get_param_num(BXPN_CPUID_LEVEL)->get();
+    BX_INFO(("  level: %d", cpu_level));
     BX_INFO(("  APIC support: %s", SIM->get_param_enum(BXPN_CPUID_APIC)->get_selected()));
 #else
+    BX_INFO(("  level: %d", BX_CPU_LEVEL));
     BX_INFO(("  APIC support: no"));
 #endif
     BX_INFO(("  FPU support: %s", BX_SUPPORT_FPU?"yes":"no"));
@@ -1180,15 +1250,15 @@ void bx_init_hardware()
 #if BX_CPU_LEVEL >= 6
     bx_bool sep_enabled = SIM->get_param_bool(BXPN_CPUID_SEP)->get();
     BX_INFO(("  SEP support: %s", sep_enabled?"yes":"no"));
-    bx_bool sse4a_enabled = SIM->get_param_bool(BXPN_CPUID_SSE4A)->get();
-    BX_INFO(("  SSE support: %s%s", SIM->get_param_enum(BXPN_CPUID_SSE)->get_selected(),
-      sse4a_enabled ? "+sse4a" : ""));
+    BX_INFO(("  SIMD support: %s", SIM->get_param_enum(BXPN_CPUID_SIMD)->get_selected()));
     bx_bool xsave_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVE)->get();
     bx_bool xsaveopt_enabled = SIM->get_param_bool(BXPN_CPUID_XSAVEOPT)->get();
     BX_INFO(("  XSAVE support: %s %s",
       xsave_enabled?"xsave":"no", xsaveopt_enabled?"xsaveopt":""));
     bx_bool aes_enabled = SIM->get_param_bool(BXPN_CPUID_AES)->get();
     BX_INFO(("  AES support: %s", aes_enabled?"yes":"no"));
+    bx_bool sha_enabled = SIM->get_param_bool(BXPN_CPUID_SHA)->get();
+    BX_INFO(("  SHA support: %s", sha_enabled?"yes":"no"));
     bx_bool movbe_enabled = SIM->get_param_bool(BXPN_CPUID_MOVBE)->get();
     BX_INFO(("  MOVBE support: %s", movbe_enabled?"yes":"no"));
     bx_bool adx_enabled = SIM->get_param_bool(BXPN_CPUID_ADX)->get();
@@ -1205,16 +1275,6 @@ void bx_init_hardware()
     bx_bool mwait_enabled = SIM->get_param_bool(BXPN_CPUID_MWAIT)->get();
     BX_INFO(("  MWAIT support: %s", mwait_enabled?"yes":"no"));
 #endif
-#if BX_SUPPORT_AVX
-    unsigned avx_enabled = SIM->get_param_num(BXPN_CPUID_AVX)->get();
-    bx_bool avx_fma_enabled = SIM->get_param_bool(BXPN_CPUID_AVX_FMA)->get();
-    if (avx_enabled) {
-      BX_INFO(("  AVX support: %d%s", avx_enabled, avx_fma_enabled ? " (with FMA)" : ""));
-    }
-    else {
-      BX_INFO(("  AVX support: no"));
-    }
-#endif
 #if BX_SUPPORT_VMX
     unsigned vmx_enabled = SIM->get_param_num(BXPN_CPUID_VMX)->get();
     if (vmx_enabled) {
@@ -1242,13 +1302,14 @@ void bx_init_hardware()
   BX_INFO(("Devices configuration"));
   BX_INFO(("  NE2000 support: %s", BX_SUPPORT_NE2K?"yes":"no"));
   BX_INFO(("  PCI support: %s, enabled=%s", BX_SUPPORT_PCI?"yes":"no",
-    SIM->get_param_bool(BXPN_I440FX_SUPPORT)->get() ? "yes" : "no"));
+    SIM->get_param_bool(BXPN_PCI_ENABLED)->get() ? "yes" : "no"));
   BX_INFO(("  SB16 support: %s", BX_SUPPORT_SB16?"yes":"no"));
   BX_INFO(("  USB support: %s", BX_SUPPORT_PCIUSB?"yes":"no"));
-  BX_INFO(("  VGA extension support: vbe %s", BX_SUPPORT_CLGD54XX?"cirrus":""));
+  BX_INFO(("  VGA extension support: vbe%s%s", BX_SUPPORT_CLGD54XX?" cirrus":"",
+    BX_SUPPORT_VOODOO?" voodoo":""));
 
   // Check if there is a romimage
-  if (strcmp(SIM->get_param_string(BXPN_ROM_PATH)->getptr(),"") == 0) {
+  if (SIM->get_param_string(BXPN_ROM_PATH)->isempty()) {
     BX_ERROR(("No romimage to load. Is your bochsrc file loaded/valid ?"));
   }
 
@@ -1259,9 +1320,19 @@ void bx_init_hardware()
   if (benchmark_mode) {
     BX_INFO(("Bochs benchmark mode is ON (~%d millions of ticks)", benchmark_mode));
     bx_pc_system.register_timer_ticks(&bx_pc_system, bx_pc_system_c::benchmarkTimer,
-        (Bit64u) benchmark_mode * 1000000, 0, 1, "benchmark.timer");
+        (Bit64u) benchmark_mode * 1000000, 0 /* one shot */, 1, "benchmark.timer");
   }
 
+#if BX_ENABLE_STATISTICS
+  // set periodic timer for dumping statistics collected during Bochs run
+  int dumpstats = SIM->get_param_num(BXPN_DUMP_STATS)->get();
+  if (dumpstats) {
+    BX_INFO(("Dump statistics every %d millions of ticks", dumpstats));
+    bx_pc_system.register_timer_ticks(&bx_pc_system, bx_pc_system_c::dumpStatsTimer,
+        (Bit64u) dumpstats * 1000000, 1 /* continuous */, 1, "dumpstats.timer");
+  }
+#endif
+
   // set up memory and CPU objects
   bx_param_num_c *bxp_memsize = SIM->get_param_num(BXPN_MEM_SIZE);
   Bit64u memSize = bxp_memsize->get64() * BX_CONST64(1024*1024);
@@ -1279,23 +1350,23 @@ void bx_init_hardware()
                       SIM->get_param_num(BXPN_ROM_ADDRESS)->get(), 0);
 
   // Then load the optional ROM images
-  if (strcmp(SIM->get_param_string(BXPN_OPTROM1_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTROM1_PATH)->isempty())
     BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM1_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM1_ADDRESS)->get(), 2);
-  if (strcmp(SIM->get_param_string(BXPN_OPTROM2_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTROM2_PATH)->isempty())
     BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM2_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM2_ADDRESS)->get(), 2);
-  if (strcmp(SIM->get_param_string(BXPN_OPTROM3_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTROM3_PATH)->isempty())
     BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM3_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM3_ADDRESS)->get(), 2);
-  if (strcmp(SIM->get_param_string(BXPN_OPTROM4_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTROM4_PATH)->isempty())
     BX_MEM(0)->load_ROM(SIM->get_param_string(BXPN_OPTROM4_PATH)->getptr(), SIM->get_param_num(BXPN_OPTROM4_ADDRESS)->get(), 2);
 
   // Then load the optional RAM images
-  if (strcmp(SIM->get_param_string(BXPN_OPTRAM1_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTRAM1_PATH)->isempty())
     BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM1_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM1_ADDRESS)->get(), 2);
-  if (strcmp(SIM->get_param_string(BXPN_OPTRAM2_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTRAM2_PATH)->isempty())
     BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM2_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM2_ADDRESS)->get(), 2);
-  if (strcmp(SIM->get_param_string(BXPN_OPTRAM3_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTRAM3_PATH)->isempty())
     BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM3_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM3_ADDRESS)->get(), 2);
-  if (strcmp(SIM->get_param_string(BXPN_OPTRAM4_PATH)->getptr(), "") !=0)
+  if (!SIM->get_param_string(BXPN_OPTRAM4_PATH)->isempty())
     BX_MEM(0)->load_RAM(SIM->get_param_string(BXPN_OPTRAM4_PATH)->getptr(), SIM->get_param_num(BXPN_OPTRAM4_ADDRESS)->get(), 2);
 
 #if BX_SUPPORT_SMP == 0
@@ -1406,6 +1477,7 @@ int bx_atexit(void)
 #endif
 
   SIM->cleanup_save_restore();
+  SIM->cleanup_statistics();
   SIM->set_init_done(0);
 
   return 0;
